#!/usr/bin/env python

from sets import *
from FITS import *
from types import *
from basics import *
from numpy.core.numerictypes import *
import numpy
from vtk import *
from numpy.oldnumeric.mlab import diff
from SimpleQueue import *

def NumToVTKArray(numarray,name=None):
      __typeDict = {uint8:vtkUnsignedCharArray,
              byte:vtkUnsignedCharArray,
              bool:vtkUnsignedCharArray,
              int8:vtkUnsignedCharArray,
              int16:vtkShortArray,
              uint16:vtkUnsignedShortArray,
              int32:vtkIntArray, int64:vtkLongArray,
              float32:vtkFloatArray,
              float64:vtkDoubleArray, complex64:vtkFloatArray, complex128:vtkDoubleArray }
      nt = numarray.dtype
      for td in __typeDict.keys():
         if td == nt: utd=td;break
      vtkarray = __typeDict[utd]()
      dims = numarray.shape
      vtkarray.SetVoidArray(numarray,multiply.reduce(dims),1)
      if len(dims) > 1: vtkarray.SetNumberOfComponents(dims[0])
      else: vtkarray.SetNumberOfComponents(1)
      vtkarray.SetNumberOfValues(multiply.reduce(dims))
      if name: vtkarray.SetName(name)
      return vtkarray

def VTKArrayToNum(i):
      __typeDict = { VTK_CHAR:int8, VTK_UNSIGNED_CHAR:uint8, VTK_ID_TYPE:int64,
      VTK_SHORT:int16, VTK_UNSIGNED_SHORT:uint16, VTK_INT:int32,
      VTK_FLOAT:float32, VTK_DOUBLE:float64, VTK_LONG:int64}
      __vtkDict = { VTK_ID_TYPE: vtkIdTypeArray, VTK_CHAR:vtkCharArray, VTK_UNSIGNED_CHAR:vtkUnsignedCharArray,
      VTK_SHORT:vtkShortArray, VTK_UNSIGNED_SHORT:vtkUnsignedShortArray, VTK_INT:vtkIntArray,
      VTK_FLOAT:vtkFloatArray, VTK_DOUBLE:vtkDoubleArray, VTK_LONG:vtkLongArray}
      it = i.GetDataType()
      ns = i.GetSize()
      nc = i.GetNumberOfComponents()
      nt = i.GetNumberOfTuples()
      na = i.GetActualMemorySize()
      nat = __typeDict[it]
      d = zeros((nc,nt),nat)
      for c in range(nc):
        n = __vtkDict[it]()
        n.SetNumberOfComponents(1)
        n.SetNumberOfValues(nt)
        n.CopyComponent(0,i,c)
        o = VTKImageToNum(VTKArrayToVTKImage(n))
        d[c] = o.astype(nat)
#        del n
      return d

def VTKCalculator(*args):
      __typeDict = {uint8:VTK_UNSIGNED_CHAR, byte:VTK_UNSIGNED_CHAR, bool1:VTK_CHAR,
              int16:VTK_SHORT, int32:VTK_INT, int64:VTK_LONG, float32:VTK_FLOAT,
              float64:VTK_DOUBLE, complex64:VTK_FLOAT, complex128:VTK_DOUBLE }
      task = None
      task = [a for a in args if type(a) == StringType]
      if not len(task): return -1
      variables = [a for a in args if type(a) != StringType]
      images = [VTKArrayToVTKImage(v) for v in variables]
#      print [image.GetScalarTypeAsString() for image in images]
      ia = vtkImageAppendComponents()
      [ia.AddInput(i) for i in images]
      iu = ia.GetOutput()
      ia.Update()
      iu.GetPointData().GetScalars().SetName("InputData")
      m = vtkArrayCalculator()
      m.SetFunction(task[0])
      m.SetInput(iu)
      m.SetResultArrayName("InputData")
      [m.AddScalarVariable(variables[j].GetName(),"InputData",j) for j in range(len(variables))]
      o = m.GetOutput()
      m.Update()
      o.SetNumberOfScalarComponents( o.GetPointData().GetScalars().GetNumberOfComponents())
      return o

def VTKArrayToVTKImage(v,nc=-1,wid=-1,hei=-1):
      ii = vtkImageData()
      ii.SetScalarType(v.GetDataType())
      if nc>0: ii.SetNumberOfScalarComponents(nc)
      else: ii.SetNumberOfScalarComponents(1)
      ii.GetPointData().AllocateArrays(1)
      ii.GetPointData().SetScalars(v)
      if wid>0 and hei>0:
         ii.SetUpdateExtentToWholeExtent()
         ii.SetExtent(0,wid-1,0,hei-1,0,0)
      else:
         shape2 = [v.GetNumberOfComponents(), v.GetNumberOfTuples()]
         ii.SetUpdateExtentToWholeExtent()
         ii.SetExtent(0,shape2[1]-1,0,shape2[0]-1,0,0)
      if nc>0: ii.SetNumberOfScalarComponents(nc)
      else: ii.SetNumberOfScalarComponents(1)
      if nc>0: ii.GetPointData().GetScalars().SetNumberOfComponents(nc)
      else: ii.GetPointData().GetScalars().SetNumberOfComponents(1)
      ii.Update()
      ii.UpdateData()
      ii.UpdateInformation()
      return ii

def NumToVTKImage(numarray,name=None):
      vtkarray = NumToVTKArray(numarray,name=name)
      ii = VTKArrayToVTKImage(vtkarray)
      return ii

def oldNumToVTKImage(numarray,name=None):
      __typeDict = {int8:VTK_UNSIGNED_CHAR, uint8:VTK_UNSIGNED_CHAR, byte:VTK_UNSIGNED_CHAR, bool1:VTK_CHAR,
              int16:VTK_SHORT, in32:VTK_INT, int64:VTK_LONG, float32:VTK_FLOAT,
              float64:VTK_DOUBLE, complex64:VTK_FLOAT, complex128:VTK_DOUBLE }
      ii = vtkImageImport()
      ii.SetDataScalarType(__typeDict[ numarray.dtype])
      if len(numarray.shape) == 1: shape2 = (1,) + numarray.shape
      else: shape2 = numarray.shape
      ii.SetWholeExtent(0,shape2[1]-1,0,shape2[0]-1,0,0)
      ii.SetDataExtentToWholeExtent()
      ii.SetNumberOfScalarComponents(1)
      ii.SetImportVoidPointer(numarray)
      ii.Update()
      im = ii.GetOutput()
      im.UpdateData()
      im.UpdateInformation()
      return im

def VTKImageToNum(i):
      __typeDict = { VTK_ID_TYPE: int64, VTK_CHAR:int8, VTK_UNSIGNED_CHAR:uint8,
      VTK_SHORT:int16, VTK_UNSIGNED_SHORT:uint16, VTK_INT:int32,
      VTK_FLOAT:float32, VTK_DOUBLE:float64, VTK_LONG:int64}
      ie = vtkImageExport()
      d = list(i.GetDimensions())
      d.reverse()
      if d[0] == 1: d = d[1:]
      if d[0] == 1: d = d[1:]
      it = i.GetScalarType()
      scalars = i.GetPointData().GetScalars()
      if scalars: it = scalars.GetDataType()
      else: it = VTK_FLOAT
      nat = __typeDict[it]
      x = zeros(d,nat)
      ie.SetExportVoidPointer(x)
      ie.ImageLowerLeftOn()
      ie.SetInput(i)
      ie.Export()
      return MLab.squeeze(x)

def VTKRecast(i,t):
      rc = vtkImageCast()
      rc.SetInput(i)
      if t == "d": rc.SetOutputScalarTypeToDouble()
      elif t == "f": rc.SetOutputScalarTypeToFloat()
      elif t == "s": rc.SetOutputScalarTypeToUnsignedShort()
      elif t == "b": rc.SetOutputScalarTypeToUnsignedChar()
      elif t == "i": rc.SetOutputScalarTypeToInt()
      elif t == "l": rc.SetOutputScalarTypeToLong()
      oc = rc.GetOutput()
      rc.Update()
      return oc

def VTKContourPoints(x,v,minsides=0,tmp="/tmp/pd.dat",area=2.0,verb=1,pad=0):
    if type(x) == type(arange(10)): i = NumToVTKImage(x)
    else: i = x
    d = i.GetDimensions()
    c = vtkMarchingContourFilter()
#    c = vtkContourFilter()
#    c = vtkMarchingSquares()
    [c.SetValue(j,v[j]) for j in range(len(v))]
    c.ComputeScalarsOn()
#    c.ComputeNormalsOn()
#    c.ComputeGradientsOn()
#    c.UseScalarTreeOn()
#    p = c.GetOutput()
    c.SetInput(i)
#    c.Update()
#    p.GetBounds()
    pc = vtkPolyDataConnectivityFilter()
    pc.SetExtractionModeToAllRegions()
    pc.ScalarConnectivityOff()
    pc.ColorRegionsOn()
    pc.SetInput(c.GetOutput())
    pc.Update()
    N = pc.GetNumberOfExtractedRegions()
    regions = arange(N)

    d = []
    n = 1
    if verb: print "N=",N

    ps = vtkStripper()
    ps.SetMaximumLength(x.shape[0]*x.shape[1])
    ps.SetInput(pc.GetOutput())
    po = ps.GetOutput()
    ps.Update()
    nl = po.GetNumberOfLines()
    nc = po.GetNumberOfCells()
    print "Number left=",nl
    areas = []
    for n in range(nl):
       pl = po.GetCell(n)
       x1,x2,y1,y2 = pl.GetBounds()[:4]
       a = (x2-x1)*(y2-y1)
       if a>area/3:
          pd = pl.GetPoints().GetData()
          pdx, pdy = VTKArrayToNum(pd)[:2]-pad
          np = pl.GetNumberOfPoints()
          if np > 4:
             segments = asarray([[(pdx[j],pdx[j+1]),(pdy[j],pdy[j+1])] for j in range(len(pdx)-1)])
             if Closed(segments,verb=1):
                x1,y1 = transpose(asarray(segments)[::,::,0])
                x2,y2 = transpose(asarray(segments)[::,::,1])
                a = 0.5*abs(sum(x1*y2-y1*x2))
                if a >= area:
                   d.append(segments)
                   areas.append(a)
       if verb and n and not (n % (nl/80+1)):
          sys.stdout.write(".")
          sys.stdout.flush()

    if verb and N:
        print
        print "Left with",len(d)
        if d: print "Areas",extrema(areas),quartiles(array(areas))

    return d

def readVTKpd(tmp="/tmp/pd.dat",minsides=0,verb=0):
    f = open(tmp,"r")
    r = f.readlines()
    f.close()
    n = {}
    w = ["POINTS","VERTICES","LINES","LOOKUP_TABLE"]
    [n.setdefault(r1.split()[0],j) for j,r1 in enumerate(r) if r1[0] in letters]
    m = int(r[n["POINTS"]].split()[1])
    l = int(r[n["LINES"]].split()[1])
    e = n.get("VERTICES",n["LINES"])
    points = reshape(map(eval,"".join(r[n["POINTS"]+1:e]).replace("\n","").split()),(m,3))[::,:2]
    lines = reshape(map(eval,"".join(r[n["LINES"]+1:n["POINT_DATA"]]).replace("\n","").split()),(l,3))[::,1:]
    names = array(map(eval,"".join(r[n["LOOKUP_TABLE"]+1:n["FIELD"]]).replace("\n","").split()))
    lines1 = lines[::,0]
    lines2 = lines[::,1]
    ldict1 = {}
    ldict2 = {}
    [ldict1.__setitem__(l,ldict1.get(l,[])+[j]) for j,l in enumerate(lines1)]
    [ldict2.__setitem__(l,ldict2.get(l,[])+[j]) for j,l in enumerate(lines2)]
    objects = sort(list(Set(names)))
    dict = {}
    [dict.setdefault(o,[]) for j,o in enumerate(names)]
    [dict[o].append(j) for j,o in enumerate(names)]
    lineids = take(names,lines[::,0])
    dict = {}
    [dict.setdefault(o,nonzero(equal(lineids,o))) for o in objects]
    o = objects[0]
    lso,leo = {},{}
    [leo.__setitem__(line[0],leo.get(line[0],[])+[l]) for l,line in enumerate(lines)]
    [lso.__setitem__(line[1],lso.get(line[1],[])+[l]) for l,line in enumerate(lines)]
    le = leo
    ls = lso
    contours = []

    su = {}


    ml = min([min(ls.keys()),min(le.keys())])
    p = []
    if ml in le: sp = le[ml].pop(0)
    else: sp = ls[ml].pop(0)
    s = [sp]
    sd = {}
    sd.setdefault(s[-1])
    while len(p) < 2 or p[0] != p[-1]:
       pe,pb = lines[s[-1]]
       if pe in le and le[pe]: st =  le[pe].pop(0)
       elif pb in le and le[pb]: st =  le[pb].pop(0)
       elif pb in ls and ls[pb]: st =  ls[pb].pop(0)
       elif pe in ls and ls[pe]: st =  ls[pe].pop(0)
       else: st = None
       if st == None: break
       else:
          s.append(st)
          sd.setdefault(s[-1])
       [le[l].remove(l2) for l in le for l2 in le[l] if l2 in sd]
       [ls[l].remove(l2) for l in ls for l2 in ls[l] if l2 in sd]
       [le.pop(l) for l in le.keys() if not le[l]]
       [ls.pop(l) for l in ls.keys() if not ls[l]]

       pb = take(lines[::,1],s)
       pe = take(lines[::,0],s)
       if pe[0] == pb[1]: p = [pb[0],pe[0]]
       elif pe[1] == pb[0]: p = [pe[0],pb[0]]
       elif pe[1] == pe[0]: p = [pb[0],pe[0]]
       elif pb[1] == pb[0]: p = [pe[0],pb[0]]
       for j in range(1,len(pb)):
           if p[-1] == pe[j]: p2 = [pe[j],pb[j]]
           elif p[-1] == pb[j]: p2 = [pb[j],pe[j]]
           else: p2 = [None]
           p.append(p2[-1])
       p = [p1 for p1 in p if p1 != None]
    x,y = transpose(take(points,p))
    [su.setdefault(p1) for p1 in p]
    segments = asarray([[(x[j],x[j+1]),(y[j],y[j+1])] for j in range(len(x)-1)])
    contours.append(segments)
    return o,contours
    

def newVTKImContour(x,v,numret=0):
    if type(x) == numpy.ndarray: i = NumToVTKImage(x)
    else: i = x
    d = i.GetDimensions()
    m = vtkImageMarchingCubes()
    [m.SetValue(j,v[j]) for j in range(len(v))]
    m.SetInput(i)
    m.ComputeScalarsOn()
    m.ComputeNormalsOn()
    m.ComputeGradientsOn()
    p = m.GetOutput()
    m.Update()
    print p


def oldVTKImContour(x,v,numret=0,alias=3,mag=1):
    if type(x) == type(arange(10)): i = NumToVTKImage(x)
    else: i = x
    d = i.GetDimensions()
    c = vtkContourFilter()
    c.SetInput(i)
    [c.SetValue(j,v[j]) for j in range(len(v))]
    c.ComputeScalarsOn(); c.ComputeNormalsOff()
    c.ComputeGradientsOff(); c.UseScalarTreeOn()
    f = c.GetOutput()
    f.Update()
    q = vtkPolyDataMapper()
    q.SetInput( f)
    q.SetScalarRange(v[0],v[-1])
    r = vtkActor()
    r.SetMapper(q)
    s = vtkRenderer()
    s.AddActor(r)
    s.GetActiveCamera().ParallelProjectionOn()
    s.GetActiveCamera().SetParallelScale(d[1]/2.0)
    zp = s.GetActiveCamera().GetPosition()[2]
    s.GetActiveCamera().SetPosition((d[0]-1)/2.,(d[1]-1)/2., zp)
    s.GetActiveCamera().SetFocalPoint((d[0]-1)/2.,(d[1]-1)/2., 0.0)
    u = vtkRenderWindow()
    u.AddRenderer(s)
    u.BordersOff()
    u.LineSmoothingOn()
    u.SetSize(d[0]/alias,d[1]/alias)
    u.OffScreenRenderingOn()
    u.DoubleBufferOff()
    t = vtkRenderLargeImage()
    t.SetInput(s)
    t.SetMagnification(alias)
    e = vtkImageLuminance()
    e.SetInput( t.GetOutput())
    y = vtkImageThreshold()
    y.ThresholdBetween(0,1.0)
    y.SetOutValue(1.0)
    y.ReplaceOutOn()
    y.SetOutputScalarTypeToFloat()
    y.SetInput( e.GetOutput())
    p = vtkImageMirrorPad()
    p.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
    p.SetInput(y.GetOutput())
    w = p.GetOutput()
    w.Update()
    if numret: return VTKImageToNum(w)
    else: return w

def VTKSurfaceReconstruct(x,y,z):
      tx,ty,tz = asarray([x,y,z])
#      tx,ty,tz = map(NumToVTKImage,asarray([x,y,z]))
#      a = vtkImageAppendComponents()
#      a.AddInput(tx)
#      a.AddInput(ty)
#      a.AddInput(tz)
#      print a
#      print dir(a)
#      a.UpdateWholeExtent()
#      A = a.GetOutputPort()
#      V = NumToVTKArray(A)
      polydata = vtkPolyData()
      points = vtkPoints()
      polydata.SetPoints(points)
      [points.InsertNextPoint(ix,iy,iz) for ix,iy,iz in zip(tx.ravel(),ty.ravel(),tz.ravel())]
      surf = vtkSurfaceReconstructionFilter()
#      surf.SetInputConnection(A)
      surf.SetInput(polydata)
      surf.UpdateWholeExtent()
      print surf
      print dir(surf)
      cf = vtkContourFilter()
      cf.SetInputConnection(surf.GetOutputPort())
      cf.SetValue(0, 0.0)
      reverse = vtkReverseSense()
      reverse.SetInputConnection(cf.GetOutputPort())
      reverse.ReverseCellsOn()
      reverse.ReverseNormalsOn()
      r = reverse.GetOutputPort()
      reverse.Update()

#      g = r.GetPoints()
      print r
      print dir(r)

      if 0:
          m = vtkPolyDataMapper()
          m.SetInputConnection(r)
          m.ScalarVisibilityOff()
          surfaceActor = vtkActor()
          surfaceActor.SetMapper(m)
          surfaceActor.GetProperty().SetDiffuseColor(1.0000, 0.3882, 0.2784)
          surfaceActor.GetProperty().SetSpecularColor(1, 1, 1)
          surfaceActor.GetProperty().SetSpecular(.4)
          surfaceActor.GetProperty().SetSpecularPower(50)

          # Create the RenderWindow, Renderer and both Actors
          ren = vtkRenderer()
          renWin = vtkRenderWindow()
          renWin.AddRenderer(ren)
          iren = vtkRenderWindowInteractor()
          iren.SetRenderWindow(renWin)

          # Add the actors to the renderer, set the background and size
          ren.AddActor(surfaceActor)
          ren.SetBackground(1, 1, 1)
          renWin.SetSize(128, 128)
          ren.GetActiveCamera().SetFocalPoint(0, 0, 0)
          ren.GetActiveCamera().SetPosition(1, 0, 0)
          ren.GetActiveCamera().SetViewUp(0, 0, 1)
          ren.ResetCamera()
          ren.GetActiveCamera().Azimuth(20)
          ren.GetActiveCamera().Elevation(30)
          ren.GetActiveCamera().Dolly(1.2)
          ren.ResetCameraClippingRange()

          iren.Initialize()
          renWin.Render()
          iren.Start()


def VTKSimpleTransform(x,origin,pa,offset=[0,0],numret=0,interp=0,cubic=0):
      if type(x) == type(arange(2)): i = NumToVTKImage(x)
      else: i = x
      t = vtkTransform()
      t.Translate(origin[0],origin[1],0)
      t.RotateWXYZ(pa,0,0,1)
      t.Translate(-origin[0]+offset[0],-origin[1]+offset[1],0)
      s = vtkImageReslice()
      s.SetResliceAxesOrigin(origin[0],origin[1],0)
      if cubic: s.SetInterpolationModeToCubic()
      else: s.SetInterpolationModeToLinear()
      s.SetOutputDimensionality(2)
      s.SetResliceTransform(t)
      s.SetInput(i)
      o = s.GetOutput()
      s.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKSlice(x,origin,pa,numret=0,interp=0):
      if type(x) == type(arange(2)): i = NumToVTKImage(x)
      else: i = x
      t = vtkTransform()
      t.Translate(origin[0],origin[1],0)
      t.RotateWXYZ(pa,0,0,1)
      t.Translate(-origin[0],-origin[1],0)
      s = vtkImageReslice()
      s.SetResliceAxesOrigin(origin[0],origin[1],0)
      s.SetInterpolationModeToCubic()
      s.SetOutputDimensionality(1)
      s.SetResliceTransform(t)
      s.SetInput(i)
      o = s.GetOutput()
      s.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKImageShift(x,u,v,numret=0,interp=1,wrap=0,mirror=0,constant=None,cubic=0):
      if type(x) == type(arange(2)): i = NumToVTKImage(x)
      else: i = x
      if 0 and int(u) == u and int(v) == v:
         s = vtkImageTranslateExtent()
         s.SetInput(i)
         s.SetTranslation(u,v,0)
         o = s.GetOutput()
         s.Update()
      else:
         s = vtkImageReslice()
         s.AutoCropOutputOn()
         if wrap: s.WrapOn()
         else: s.WrapOff()
         if mirror or constant != None: s.MirrorOn()
         else: s.MirrorOff()
         if interp:
           s.InterpolateOn()
           if cubic:
             s.SetInterpolationModeToCubic()
           else:
             s.SetInterpolationModeToLinear()
         else: s.InterpolateOff()
         s.OptimizationOn()
#         s.SetOutputSpacing(1,1,1)
         s.SetOutputOrigin(u,v,0)
         s.SetInput(i)
         o=s.GetOutput()
         s.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKInvertTransform(dx,dy,numret=0):
      if type(dx) == type(arange(10)): tx = NumToVTKImage(dx)
      else: tx = dx
      if type(dy) == type(arange(10)): ty = NumToVTKImage(dy)
      else: ty = dy
      dm = tx.GetDimensions()
      dz = zeros(dy.shape,float32)
      tz = NumToVTKImage(dz)
      a = vtkImageAppendComponents()
      a.AddInput(tx)
      a.AddInput(ty)
      a.AddInput(tz)
      t = a.GetOutput()
      r = vtkGridTransform()
      r.SetDisplacementGrid(t)
      r.SetInterpolationModeToLinear()
      r.SetInverseTolerance(0.001)
      r.SetInverseIterations(0)
      r.Update()
      i=r.GetInverse()
      print i
      o = i.GetDisplacementGrid()
      i.Update()
      print o
      if numret: return VTKImageToNum(o)
      else: return o

def VTKImageTransform(x,dx,dy,numret=0,reverse=0,origsize=None,cubic=0,interp=1,scalex=1,scaley=1,constant=0,wrap=0,mirror=0):
      maxdx = max([int(max(abs(dx.ravel()))+1),(dx.shape[1]-x.shape[1])])
      maxdy = max([int(max(abs(dy.ravel()))+1),(dx.shape[0]-x.shape[0])])
      dx = dx.astype(float32)
      dy = dy.astype(float32)
      if scalex > 1:
         xx = arange(x.shape[1])
         dx = (xx[NewAxis,::]+dx).astype(float32)
         dy = VTKGrowN(dy,scalex,1,numret=1)
         dx = VTKGrowN(dx,scalex,1,numret=1)
         dx = (dx-arange(scalex*x.shape[1])[NewAxis,::]).astype(float32)
      if scaley > 1:
         yy = arange(x.shape[0])
         dy = (yy[::,NewAxis]+dy).astype(float32)
         dy = VTKGrowN(dy,1,scaley,numret=1)
         dx = VTKGrowN(dx,1,scaley,numret=1)
         dy = (dy-arange(scaley*x.shape[0])[::,NewAxis]).astype(float32)
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      if type(dx) == numpy.ndarray: tx = NumToVTKImage(dx)
      else: tx = dx
      if type(dy) == numpy.ndarray: ty = NumToVTKImage(dy)
      else: ty = dy
      dm = ty.GetDimensions()
      dz = zeros(dy.shape,dy.dtype)
      tz = NumToVTKImage(dz)
      a = vtkImageAppendComponents()
      a.AddInput(tx)
      a.AddInput(ty)
      a.AddInput(tz)
      t = a.GetOutput()
      r = vtkGridTransform()
      r.SetDisplacementGrid(t)
      r.Update()
      s = vtkImageReslice()
      s.WrapOff()
      s.MirrorOn()
      if interp:
         s.InterpolateOn()
         if cubic: s.SetInterpolationModeToCubic()
         else: s.SetInterpolationModeToLinear()
      s.SetOutputDimensionality(2)
      if reverse:
         r.SetInterpolationModeToLinear()
         r.SetInverseTolerance(0.001)
         r.SetInverseIterations(1000)
         r.DebugOff()
         ir = r.GetInverse()
         ir.SetInterpolationModeToLinear()
         ir.SetInverseTolerance(0.001)
         ir.SetInverseIterations(1000)
         ir.GlobalWarningDisplayOff()
         s.SetResliceTransform(ir)
         s.AutoCropOutputOff()
         if origsize: s.SetOutputExtent(0,origsize[1]-1,0,origsize[0]-1,0,0)
         else: s.SetOutputExtent(0,scalex*dm[0]-1,0,scaley*dm[1]-1,0,0)
      else:
         r.SetInterpolationModeToCubic()
         r.SetInverseTolerance(0.001)
         r.SetInverseIterations(1000)
         r.GlobalWarningDisplayOff()
         s.SetOutputExtent(0,scalex*dm[0]-1,0,scaley*dm[1]-1,0,0)
         s.AutoCropOutputOff()
         s.SetResliceTransform(r)
      if mirror: ip = vtkImageMirrorPad()
      elif wrap: ip = vtkImageWrapPad()
      else: ip = vtkImageConstantPad(); ip.SetConstant(constant)
      ip.SetOutputWholeExtent(0-maxdx,dm[0]-1+maxdx,0-maxdy,dm[1]-1+maxdy,0,0)
      ip.SetInput(i)
      s.SetInput(ip.GetOutput())
      o=s.GetOutput()
      s.Update()
      if numret: ri = VTKImageToNum(o)
      else: ri = o
      return ri

def VTKDivergence(x,numret=0):
      if type(x) == type(arange(10)): i = NumToVTKImage(x)
      else: i = x
      d = vtkImageDivergence()
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKLaplace(x,numret=0):
      if type(x) == type(arange(10)): i = NumToVTKImage(x)
      else: i = x
      d = vtkImageLaplacian()
      d.SetDimensionality(2)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKImageArith(x1,x2,op,numret=0,rep=0.0):
      if type(x1) == numpy.ndarray and type(x2) == numpy.ndarray:
        if x1.dtype == float64 or x2.dtype == float64:
          x1,x2 = x1.astype(float64), x2.astype(float64)
        elif x1.dtype == float32 or x2.dtype == float32:
          x1,x2 = x1.astype(float32), x2.astype(float32)
      if type(x1) == numpy.ndarray: i1 = NumToVTKImage(x1)
      else: i1 = x1
      if type(x2) == numpy.ndarray: i2 = NumToVTKImage(x2)
      else: i2 = x2
      m = vtkImageMathematics()
      numops = [ "multiply","divide","add","subtract"]
      if op == "divide":
        m.SetConstantC(rep); m.DivideByZeroToCOn()
      if type(x1) == numpy.ndarray and type(x2) == numpy.ndarray:
         m.SetInput1(i1)
         m.SetInput2(i2)
         vtkops = [ m.SetOperationToMultiply, m.SetOperationToDivide, m.SetOperationToAdd, m.SetOperationToSubtract]
      elif type(x1) == numpy.ndarray:
         m.AddInput(i1)
         if i2 == 0:
            const = 0.0
         else:
            const = [i2,1.0/i2,i2,-i2][numops.index(op)]
         m.SetConstantK(const)
         m.SetConstantC(const)
         vtkops = [ m.SetOperationToMultiplyByK, m.SetOperationToMultiplyByK, m.SetOperationToAddConstant, m.SetOperationToAddConstant]
      if type(x1) != numpy.ndarray and type(x2) != numpy.ndarray:
         return eval(op)(x1,x2)
      else:
         vtkops[numops.index(op)]()
         o = m.GetOutput()
         m.Update()
         if numret: return VTKImageToNum(o)
         else: return o

def VTKGreaterEqual(x1,th):
      if type(x1) == numpy.ndarray: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageThreshold()
      m.ThresholdByUpper(th)
      m.SetInput(i1)
      m.ReplaceInOn()
      m.ReplaceOutOn()
      m.SetInValue(1.0)
      m.SetOutValue(0.0)
      m.SetOutputScalarTypeToFloat()
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKLessEqual(x1,th):
      if type(x1) == numpy.ndarray: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageThreshold()
      m.ThresholdByLower(th)
      m.SetInput(i1)
      m.ReplaceInOn()
      m.ReplaceOutOn()
      m.SetInValue(1.0)
      m.SetOutValue(0.0)
      m.SetOutputScalarTypeToFloat()
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKBetween(x1,th1,th2):
      if type(x1) == numpy.ndarray: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageThreshold()
      m.ThresholdBetween(th1,th2)
      m.SetInput(i1)
      m.ReplaceInOn()
      m.ReplaceOutOn()
      m.SetInValue(1.0)
      m.SetOutValue(0.0)
      m.SetOutputScalarTypeToFloat()
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKImageLogic(x1,x2=None,op="or",numret=0):
      if type(x1) == numpy.ndarray: i1 = NumToVTKImage(x1)
      else: i1 = x1
      if type(x2) == numpy.ndarray: i2 = NumToVTKImage(x2)
      elif x2: i2 = x2
      m = vtkImageLogic()
      numops = [ "and","or","xor","nand","nor","not"]
      vtkops = [ m.SetOperationToAnd, m.SetOperationToOr, m.SetOperationToXor, m.SetOperationToNand, m.SetOperationToNor, m.SetOperationToNot]
      m.SetOutputTrueValue(1.0)
      vtkops[numops.index(op)]()
      m.SetInput1(i1)
      if x2!= None: m.SetInput2(i2)
      o = m.GetOutput()
      m.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKOr(x1,x2): return VTKImageLogic(x1,x2=x2,op="or",numret=1)
def VTKAnd(x1,x2): return VTKImageLogic(x1,x2=x2,op="and",numret=1)
def VTKNot(x1): return VTKImageLogic(x1,x2=None,op="not",numret=1)

def VTKInvert(x1,rep=0.0):
      if type(x1) == numpy.ndarray: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageMathematics()
      m.SetOperationToInvert()
      m.AddInput(i1)
      m.SetConstantC(rep)
      m.DivideByZeroToCOn()
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKAbs(x1):
      if type(x1) == numpy.ndarray: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageMathematics()
      m.SetOperationToAbsoluteValue()
      m.AddInput(i1)
      o = m.GetOutput()
      m.Update()
      return VTKImageToNum(o)

def VTKSqrt(x0,rep=0.0):
      bad = less(x0,0)
      x1 = VTKAbs(x0)
      if type(x1) == numpy.ndarray: i1 = NumToVTKImage(x1)
      else: i1 = x1
      m = vtkImageMathematics()
      m.SetOperationToSquareRoot()
      m.AddInput(i1)
      o = m.GetOutput()
      m.Update()
      b = VTKImageToNum(o)
      return where(bad,rep,b)

def VTKAdd(x1,x2): return VTKImageArith(x1,x2,"add",numret=1)
def VTKSubtract(x1,x2): return VTKImageArith(x1,x2,"subtract",numret=1)
def VTKMultiply(x1,x2): return VTKImageArith(x1,x2,"multiply",numret=1)
def VTKDivide(x1,x2,rep=0): return VTKImageArith(x1,x2,"divide",numret=1,rep=rep)

def VTKGradient(x,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      g = vtkImageGradient()
      g.SetInput(i)
      g.SetDimensionality(2)
      g.HandleBoundariesOn()
      o = g.GetOutput()
      g.Update()
      if numret:
         c1 = vtkImageExtractComponents()
         c1.SetInput(o)
         c1.SetComponents(0)
         o1 = c1.GetOutput()
         c2 = vtkImageExtractComponents()
         c2.SetInput(o)
         c2.SetComponents(1)
         o2 = c2.GetOutput()
         c1.Update()
         c2.Update()
         a1 = VTKImageToNum(o1)
         a2 = VTKImageToNum(o2)
         a = asarray([a1,a2])
         return a
      else: return o

def VTKGradMag(x,numret=0,dim=2):
      if type(x) == type(arange(10)): i = NumToVTKImage(x)
      else: i = x
      dm = i.GetDimensions()
      maxdx = 5
      maxdy = 5
      ip = vtkImageWrapPad()
      ip.SetOutputWholeExtent(0-maxdx,dm[0]-1+maxdx,0-maxdy,dm[1]-1+maxdy,0,0)
      ip.SetInput(i)
      d = vtkImageGradientMagnitude()
      d.SetDimensionality(dim)
      d.HandleBoundariesOn()
      d.SetInput(ip.GetOutput())
      r = vtkImageReslice()
      r.SetOutputExtent(0,dm[0]-1,0,dm[1]-1,0,0)
      r.SetInput(d.GetOutput())
      o = r.GetOutput()
      r.Update()
      if numret:
         a = VTKImageToNum(o)
         a[0] = a[1]
         a[-1] = a[-2]
         if len(a.shape) > 1:
            a[::,0] = a[::,1]
            a[::,-1] = a[::,-2]
         return a
      else: return o

def VTKAnisotropicDiffusion(x,th,fac,iter,faces=0,edges=1,corners=0,gmag=0,numret=1):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageAnisotropicDiffusion2D()
      d.SetDiffusionThreshold(th)
      d.SetDiffusionFactor(fac)
      d.SetNumberOfIterations(iter)
      d.FacesOff()
      d.EdgesOff()
      d.CornersOff()
      if faces: d.FacesOn()
      if edges: d.EdgesOn()
      if corners: d.CornersOn()
      if gmag: d.GradientMagnitudeThresholdOn()
      else: d.GradientMagnitudeThresholdOff()
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKNonMaxSuppress(x,numret=0,usegrad=0):
      x = x.astype("float")
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      vi = vtkImageGradient()
      vi.SetDimensionality(2)
      vi.HandleBoundariesOn()
      vi.SetInput(i)
      d = vtkImageNonMaximumSuppression()
      d.HandleBoundariesOn()
      d.SetDimensionality(2)
      if usegrad:
         mi = vtkImageGradientMagnitude()
         mi.SetDimensionality(2)
         mi.HandleBoundariesOn()
         mi.SetInput(i)
         d.SetMagnitudeInput(mi.GetOutput())
      else:
         d.SetMagnitudeInput(i)
      d.SetVectorInput(vi.GetOutput())
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKContours(x,v,pad=1):
#    if type(x) == numpy.ndarray: i = NumToVTKImage(x)
#    else: i = x
    t = (-pad,x.shape[1]+pad,-pad,x.shape[0]+pad,0,0)
    i = VTKConstantPad(x,t,0.0,numret=0)
    d = i.GetDimensions()
#    c = vtkContourFilter()
#    c = vtkKitwareContourFilter()
#    c = vtkMarchingContourFilter()
#    c = vtkImageMarchingCubes()
    c = vtkMarchingSquares()
    c.SetInput(i)
    [c.SetValue(j,v[j]) for j in range(len(v))]
#    c.ComputeScalarsOn()
#    c.ComputeNormalsOff()
#    c.ComputeGradientsOff()
#    c.UseScalarTreeOn()
    p = c.GetOutput()
    c.Update()
    return p

def VTKConnectivity(p,x=None,y=None,regions=[],num=0):
    pc = vtkPolyDataConnectivityFilter()
    pc.ScalarConnectivityOff()
    pc.ColorRegionsOn()
    if len(regions):
       pc.SetExtractionModeToSpecifiedRegions()
       pc.InitializeSpecifiedRegionList()
       [pc.AddSpecifiedRegion(region) for region in regions]
#       [pc.DeleteSpecifiedRegion(region) for region in range(num) if region not in regions]
    elif x != None and y != None:
       pc.SetExtractionModeToClosestPointRegion()
       pc.SetClosestPoint([x,y,0])
    else:
       pc.SetExtractionModeToAllRegions()
    pc.SetInput(p)
    pc.Update()
    u = pc.GetOutput()
    u.Update()
    return u

def VTKPolyDataToImageStencil(p,x,bg=500):
    n = ones(x.shape,x.dtype)
    u = NumToVTKImage(n)
    s = vtkPolyDataToImageStencil()
    s.SetInput(p)
    t = vtkImageStencil()
    t.SetStencil(s.GetOutput())
    t.SetInput(u)
    t.SetBackgroundValue(bg)
    t.ReverseStencilOn()
    o = t.GetOutput()
    o.Update()
    o = VTKImageToNum(o)
    return o

def VTKGetContours(x,v,verb=0,nsamp=1,minsides=0,clean=0,pad=2,area=4):
     contours = []
     iso = VTKContours(x,[v],pad=pad)
     nl = iso.GetNumberOfLines()
     if nl < minsides:
        print "No contours found!"
        return contours
#     pc = VTKConnectivity(iso)
#     pc = VTKConnectivity(pc,regions=[])
#     ss = VTKArrayToNum(pc.GetPointData().GetScalars())[0]
#     num = len(sort(list(Set(ss))))
     pc = vtkPolyDataConnectivityFilter()
     pc.ScalarConnectivityOff()
     pc.ColorRegionsOn()
     pc.SetExtractionModeToAllRegions()
     pc.SetInput(iso)
     pc.Update()
     num = pc.GetNumberOfExtractedRegions()
     pc = VTKConnectivity(pc.GetOutput(),regions=[])
     if verb:
        if nsamp > 1: print "Number of regions (sampled): %d (%d)" % (num,num/nsamp)
        else: print "Number of regions: %d" % (num)
     regions = range(0,num,nsamp)
     if verb:
        print "Number of regions to do: %d" % (len(regions))
     ss = pc.GetPointData().GetArray(1)
     if ss != None:
        ss = VTKArrayToNum(ss)[0]
        s = VTKArrayToNum(pc.GetPoints().GetData())[:2]-pad
        print ss.shape,s.shape
        sx,sy = s
        sr = {}
        [sr.__setitem__(i,sr.get(i,[])+[j]) for j,i in enumerate(ss)]
        rnames = sort(sr.keys())
        esx = array([diff(extrema(take(sx,sr[region])))[0] for region in rnames])
        esy = array([diff(extrema(take(sy,sr[region])))[0] for region in rnames])
        esa = esx*esy
        print quartiles(esa)
        regions = compress(greater(esa,area/2.0),regions)
        print "Number left=",len(regions)
        for region in regions:
          if verb:
             sys.stdout.write(".")
             sys.stdout.flush()
          contour = VTKConnectivity(iso,regions=[region])
          lines = contour.GetLines()
#          clean = vtkCleanPolyData()
#          clean.SetInput(contour)
#          clean.PointMergingOn()
#          contour = clean.GetOutput()
#          clean.Update()
          nl = contour.GetNumberOfLines()
          nc = contour.GetNumberOfCells()
          if nl >= minsides:
             segments = []
             segids = []
             for q in range(nl):
                line = contour.GetCell(q)
                nids = line.GetPointIds().GetNumberOfIds()
                lids = [line.GetPointIds().GetId(nid) for nid in range(nids)]
                s1 = VTKArrayToNum(line.GetPoints().GetData())[:2]-pad
                segment = s1.astype(float64)
                segments.append(segment)
                segids.append(lids)
             segments = asarray(segments)
             segids = asarray(segids)
#             segments = oldRearrangeContour(segments)
             segments = RearrangeContour(segments,segids)
             if Closed(segments,verb=1):
                contours.append(segments)
#             raw_input("<cr> ")
     if verb: print
#     contours = CleanContours(contours,area=area)
     return contours

def CleanContours(contours,area=4):
    cleaned = []
    for contour in contours:
        x1,y1 = transpose(asarray(contour)[::,::,0])
        x2,y2 = transpose(asarray(contour)[::,::,1])
        a = 0.5*abs(sum(x1*y2-y1*x2))
        if a > area:
           cleaned.append(contour)
    return cleaned


def ConnectSegments(lines,verb=0):
    contours = []
    lines = 1.*asarray(lines)
    xe,ye = transpose(lines[::,::,0])
    xb,yb = transpose(lines[::,::,1])
    while len(xe):
        n = len(xe)
        if verb:
           print n,
           sys.stdout.flush()
        o = []
        p = {}
        j = [0]
        while len(j):
            o.append(j[0])
            p.setdefault((xe[o[-1]],ye[o[-1]]),len(o)-1)
            j = nonzero(equal(xb,xe[o[-1]])*equal(yb,ye[o[-1]]))
            j = [j1 for j1 in j if j1 not in o]
        s = [i for i in arange(n) if i not in o]
        l = p.keys()
        o = [p[k] for k in l]
        x,y = transpose(take(l,argsort(o),0))
        x = concatenate([x,x[:1]])
        y = concatenate([y,y[:1]])
        segments = asarray([[(x[j],x[j+1]),(y[j],y[j+1])] for j in range(len(x)-1)])
        contours.append(segments)
        xe,ye,xb,yb = take( [xe,ye,xb,yb], s, 1)
    if verb: print
    return contours


def OrderSegments(segments):
    segments = 1.*asarray(segments)
    xe,ye = transpose(segments[::,::,0])
    xb,yb = transpose(segments[::,::,1])
    n = len(xe)
    o = []
    p = {}
    j = [0]
    while len(j):
        o.append(j[0])
        p.setdefault((xe[o[-1]],ye[o[-1]]),len(o)-1)
        j = nonzero(equal(xb,xe[o[-1]])*equal(yb,ye[o[-1]]))
        j = [j1 for j1 in j if j1 not in o]
    l = p.keys()
    o = [p[k] for k in l]
    x,y = transpose(take(l,argsort(o),0))
    x = concatenate([x,x[:1]])
    y = concatenate([y,y[:1]])
    segments = asarray([[(x[j],x[j+1]),(y[j],y[j+1])] for j in range(len(x)-1)])
    return segments


def mangle(t):
    t = where(greater(t,pi),t-2*pi,t)
    t = where(less(t,-pi),t+2*pi,t)
    return t

def Interior(x,y,contour,verb=0):
    (xb,yb),(xe,ye) = transpose(contour)
    np = len(xb)
    a = mangle(arctan2(ye-y,xe-x) - arctan2(yb-y,xb-x))
    t = sum(a)
    interior = abs(t-2*pi)<1e-4
    if verb: print x,y,t,interior
    return interior

def RearrangeContour(contour,pointids,verb=0):
    (xb0,yb0),(xe0,ye0) = transpose(contour)
    s,e = transpose(pointids)
    sd = {}
    [sd.setdefault(s1,j) for j,s1 in enumerate(s)]
    ed = {}
    [ed.setdefault(e1,j) for j,e1 in enumerate(e)]
    xb, yb, xe, ye = [],[],[],[]
    n = len(pointids)
    p = [s[0]]
    for j in range(n):
        b = e[sd[p[-1]]]
        p.append(b)
#    p.append(p[0])
    j = [sd[i] for i in p]
    x = take(xb0,j)
    y = take(yb0,j)
    segments = asarray([[(x[j],x[j+1]),(y[j],y[j+1])] for j in range(len(x)-1)])
    return segments

def oldRearrangeContour(contour,verb=0):
#    (xb0,yb0),(xe0,ye0) = transpose(contour)
#    (xb0,yb0),(xe0,ye0) = floor(10*transpose(contour)+0.5).astype(int32)/10.0
    (xb0,yb0),(xe0,ye0) = floor(transpose(contour)+0.5).astype(int32)
#    (xb0,yb0),(xe0,ye0) = transpose(contour).astype(int32)
    xb = []
    yb = []
    xe = []
    ye = []
    n = arange(len(xb0))
    de = {}
    db = {}
    p = [0]
    for i,xb1,yb1,xe1,ye1 in zip(n,xb0,yb0,xe0,ye0):
       if i not in p:
          db[(xb1,yb1)] = db.get((xb1,yb1),[]) + [i]
          de[(xe1,ye1)] = de.get((xe1,ye1),[]) + [i]
    while db or de and len(p) < len(xb0):
#        print len(p),len(xb0),len(db.keys()),len(de.keys())
        lx = xe0[p[-1]]
        ly = ye0[p[-1]]
        if (lx,ly) in de and not de[(lx,ly)]: de.pop((lx,ly))
        if (lx,ly) in db and not db[(lx,ly)]: db.pop((lx,ly))
        j = -99
        if (lx,ly) in db:
           j = db[(lx,ly)].pop(0)
           if (xe0[j],ye0[j]) in de and j in de[(xe0[j],ye0[j])]: de[(xe0[j],ye0[j])].remove(j)
           if (xb0[j],yb0[j]) in db and j in db[(xb0[j],yb0[j])]: db[(xb0[j],yb0[j])].remove(j)
        elif (lx,ly) in de:
           j = de[(lx,ly)].pop(0)
           xb0[j],yb0[j],xe0[j],ye0[j] = xe0[j],ye0[j],xb0[j],yb0[j]
           if (xe0[j],ye0[j]) in de and j in de[(xe0[j],ye0[j])]: de[(xe0[j],ye0[j])].remove(j)
           if (xb0[j],yb0[j]) in db and j in db[(xb0[j],yb0[j])]: db[(xb0[j],yb0[j])].remove(j)
        elif 1:
           break
           lx = xe0[p[-1]]
           ly = ye0[p[-1]]
           u = ones(xe0.shape)
           for p1 in p: u[p1] = 0
           if u:
              uxb0,uyb0,uxe0,uye0 = compress(u,[xb0,yb0,xe0,ye0],1)
              un = nonzero(u)
              dsb = sqrt((uxb0-lx)**2+(uyb0-ly)**2)
              dse = sqrt((uxe0-lx)**2+(uye0-ly)**2)
              mb = argmin(dsb)
              me = argmin(dse)
              if dsb[mb] < dse[me]:
                 j = un[mb]
              else:
                 j = un[me]
                 xb0[j],yb0[j],xe0[j],ye0[j] = xe0[j],ye0[j],xb0[j],yb0[j]
        else:
           print p
           print db
           print de
        if j>-99 and j not in p:
            p.append(j)
            rb = [k for k in db if j in db[k]]
            re = [k for k in de if j in de[k]]
            for k in rb: db[k].remove(j)
            for k in re: de[k].remove(j)
        rb = [k for k in db if db[k] == []]
        re = [k for k in de if de[k] == []]
        for k in rb: db.pop(k)
        for k in re: de.pop(k)
#    print len(p),len(xb0)
    xb,yb,xe,ye = take([xb0,yb0,xe0,ye0],p,1)
    contour = transpose([[xb,yb],[xe,ye]])
    return contour

def newInteriors(xs,ys,contour,verb=0):
#    contour = RearrangeContour(contour)
    (xb,yb),(xe,ye) = transpose(contour)
    ae = arctan2(ye[::,NewAxis]-ys[NewAxis,::],xe[::,NewAxis]-xs[NewAxis,::])
    ab = arctan2(yb[::,NewAxis]-ys[NewAxis,::],xb[::,NewAxis]-xs[NewAxis,::])
    a = mangle(ae - ab)
    t = add.reduce(a,0)
    return t

def Interiors(xs,ys,contour,verb=0):
    (xb,yb),(xe,ye) = transpose(contour)
    y0, y1 = sort([yb,ye],0)
    rx = logical_or(less(xs[NewAxis,::],xe[::,NewAxis]),less(xs[NewAxis,::],xb[::,NewAxis]))
    ry = greater(ys[NewAxis,::],y0[::,NewAxis])*less_equal(ys[NewAxis,::],y1[::,NewAxis])
    r = logical_and(rx,ry).astype(float)
    n = add.reduce(r,0)
    i = mod(n,2)
    t = 2*pi*i
    return t

def angleInteriors(xs,ys,contour,verb=0):
#    contour = RearrangeContour(contour)
    (xb,yb),(xe,ye) = transpose(contour)
    ae = arctan2(ye[::,NewAxis]-ys[NewAxis,::],xe[::,NewAxis]-xs[NewAxis,::])
    ab = arctan2(yb[::,NewAxis]-ys[NewAxis,::],xb[::,NewAxis]-xs[NewAxis,::])
    a = ae - ab
    hi = greater(a,pi)
    lo = less(a,-pi)
    mid = logical_not(logical_or(hi,lo))
    a = hi*(a-2*pi) + mid*a + lo*(a+2*pi)
    t = abs(add.reduce(a,0))
    t2 = abs(add.accumulate(a,0))
    t = array([t21[argmin(abs(t21-2*pi))] for t21 in transpose(t2)])
    return t


def Closed(contour,verb=0):
    (xb,yb),(xe,ye) = transpose(contour)
    np = len(xb)
    x,y = xb[0],yb[0]
    a = mangle(arctan2(ye-y,xe-x) - arctan2(yb-y,xb-x))
    aa = add.accumulate(a)
    i = sum(a)
    if less(abs(aa),1e-8).any(): return True
    else: return False

def oldVTKDrawContour(ix,contours,color=1,fill=0,verb=0):
    if type(ix) == numpy.ndarray: i = NumToVTKImage(ix)
    else: i = ix
    c = vtkImageCanvasSource2D()
    e = i.GetExtent()
    c.SetExtent(e[0],e[1],e[2],e[3],e[4],e[5])
#    c.SetScalarType( VTK_DOUBLE)
    c.SetScalarType( i.GetScalarType())
    c.DrawImage(0,0,i)
    c.Update()
    for jc,contour in enumerate(contours):
       if verb:
          if jc and not (jc % (len(contours)/40+1)):
             sys.stdout.write(".")
             sys.stdout.flush()
       if isinstance(color,ArrayType) or isinstance(color,ListType): color1 = color[jc]
       else: color1 = color
       if isinstance(fill,ArrayType) or isinstance(fill,ListType): fill1 = fill[jc]
       else: fill1 = fill
       (xb,yb),(xe,ye) = transpose(contour)
       xmin,xmax = extrema(concatenate([xb,xe]))
       ymin,ymax = extrema(concatenate([yb,ye]))
       xmin,ymin = int(xmin-1), int(ymin-1)
       xmax,ymax = int(xmax+2), int(ymax+2)
       xmin,xmax = clip( [xmin,xmax],0,ix.shape[1])
       ymin,ymax = clip( [ymin,ymax],0,ix.shape[0])
#       print xmin,xmax,ymin,ymax
#       c2 = vtkImageCanvasSource2D()
#       c2.SetExtent(0,xmax-xmin,0,ymax-ymin,e[4],e[5])
#       c2.SetScalarType( c.GetScalarType())
##       c2.Update()
#       c2.UpdateWholeExtent()
#       c2.DrawImage(0,0,c.GetOutput(),xmin,ymin,xmax-xmin,ymax-ymin)
       if 0:
           cx1,cy1 = 1.*transpose(asarray(contour)[::,::,0]).astype(float64)
           cx2,cy2 = 1.*transpose(asarray(contour)[::,::,1]).astype(float64)
           cz1 = zeros(cx1.shape,cx1.dtype).astype(float64)
           cx = concatenate([cx1,cx1[:1]])
           cy = concatenate([cy1,cy1[:1]])
           cz = concatenate([cz1,cz1[:1]])
           cp1 = 1.*asarray([cx,cy,cz])
           ca1 = NumToVTKArray(cp1[0])
           ca2 = NumToVTKArray(cp1[1])
           ca3 = NumToVTKArray(cp1[2])
           ca = vtkDoubleArray()
           ca.SetNumberOfComponents(3)
           ca.SetNumberOfTuples(cx.shape[0])
           ca.CopyComponent(0,ca1,0)
           ca.CopyComponent(1,ca2,0)
           ca.CopyComponent(2,ca3,0)
           po = vtkPoints()
           po.SetData(ca)
           cl = vtkPolyData()
           cl.SetPoints(po)
           cl.Update()
#####       print VTKArrayToNum(cl.GetPoints().GetData())
       for segment in contour:
          if color1:
             x,y = floor(segment+0.5).astype(int32)
             c.SetDrawColor(color1)
             c.DrawSegment3D(x[0],y[0],0.0,x[1],y[1],0.0)
#             c2.SetDrawColor(color1)
#             c2.DrawSegment3D(x[0]-xmin,y[0]-ymin,0.0,x[1]-xmin,y[1]-ymin,0.0)
       if fill1 and len(contour): # and Closed(contour,verb=0):
          (xb,yb),(xe,ye) = transpose(contour)+0.5
          x0,x1 = int(min(xb)), int(max(xb)+0.5)
          y0,y1 = int(min(yb)), int(max(yb)+0.5)
          x2 = clip(arange(x0,x1,1.0,int32),e[0],e[1]-1)
          y2 = clip(arange(y0,y1,1.0,int32),e[2],e[3]-1)
          for y21 in y2:
              xa = floor(x2-xmin+0.5).astype(int32)
              ya = floor(y21*ones(xa.shape)-ymin+0.5).astype(int32)
              c2.Update()
              go = c2.GetOutput()
              go.Update()
              ovs = array([go.GetScalarComponentAsDouble(xa1,ya1,0,0) for xa1,ya1 in zip(xa,ya)])
              xy = array([(xa1,ya1) for ov,xa1,ya1 in zip(ovs,xa,ya) if ov not in [fill1,color1]])
              if len(xy): xa,ya = transpose(xy)
              else: xa,ya = [],[]
              if len(xa):
                 ta = Interiors(xa+xmin,ya+ymin,contour)
                 dta = abs(ta-2*pi)
                 u = nonzero(less(dta,1e-4))
#                 for xa1,ya1 in zip(xa,ya):
#                   if 0<xa1<(xmax-xmin) and 0<ya1<(ymax-ymin):
#                     c2.SetDrawColor(4*fill1)
#                     c2.DrawSegment(xa1,ya1,xa1,ya1)
                 if len(u):
                    xa,ya = take([xa,ya],u,1)
#                    for xa1,ya1 in zip(xa,ya):
#                        c2.SetDrawColor(fill1)
#                        c2.DrawSegment(xa1,ya1,xa1,ya1)
                    if 1:
                      while len(xa):
                          c2.Update()
                          c2.SetDrawColor(fill1)
                          c2.FillPixel(xa[0],ya[0])
                          c2.Update()
                          go = c2.GetOutput()
                          ovs = array([go.GetScalarComponentAsDouble(xa1,ya1,0,0) for xa1,ya1 in zip(xa,ya)])
                          xy = array([(xa1,ya1) for ov,xa1,ya1 in zip(ovs,xa,ya) if ov not in [fill1,color1]])
                          if len(xy): xa,ya = transpose(xy)
                          else: xa,ya = [],[]

##       c2.Update()
#       c.DrawImage(xmin,ymin,c2.GetOutput(),0,0,xmax-xmin,ymax-ymin)
##       c.DrawImage(xmin,ymin,c2.GetOutput())
##       c.DrawImage(0,0,c2.GetOutput())
##       c.Update()

    o = c.GetOutput()
    c.Update()
    if o: n = VTKImageToNum(o)
    else: n = zeros(ix.shape,ix.dtype)
    if verb: print
    return n

def VTKDrawContour(ix,contours,color=1,fill=0,verb=0):
#    ix = ix.astype(float64)
    if type(ix) == numpy.ndarray: i = NumToVTKImage(ix)
    else: i = ix
    c = vtkImageCanvasSource2D()
    e = i.GetExtent()
    c.SetExtent(e[0],e[1],e[2],e[3],e[4],e[5])
    c.Update()
#    c.SetScalarType( VTK_DOUBLE)
    c.SetScalarType( i.GetScalarType())
    c.DrawImage(0,0,i)
#    c.DrawImage(0,0,i,1,1,ix.shape[1],ix.shape[0])
    c.Update()
    for jc,contour in enumerate(contours):
       if verb:
          if jc and not (jc % (len(contours)/40+1)):
             sys.stdout.write(".")
             sys.stdout.flush()
       if isinstance(color,ArrayType) or isinstance(color,ListType): color1 = color[jc]
       else: color1 = color
       if isinstance(fill,ArrayType) or isinstance(fill,ListType): fill1 = fill[jc]
       else: fill1 = fill
       if fill1:
          (xb,yb),(xe,ye) = transpose(contour)
          x0,x1 = int(min(xb)), int(max(xb)+0.5)
          y0,y1 = int(min(yb)), int(max(yb)+0.5)
          cx1,cy1 = 1.*transpose(asarray(contour)[::,::,0]).astype(float64)
          cx2,cy2 = 1.*transpose(asarray(contour)[::,::,1]).astype(float64)
          if (cx1[-1],cy1[-1]) == (cx1[-1],cy1[-1]):
             cx1,cy1 = cx1[:-1],cy1[:-1]
          cx1 = clip(cx1,0,ix.shape[1]-1)
          cy1 = clip(cy1,0,ix.shape[0]-1)
          cz1 = zeros(cx1.shape,cx1.dtype).astype(float64)
          cp1 = 1.*asarray([cx1-x0,cy1-y0,cz1])
          ca1 = NumToVTKArray(cp1[0])
          ca2 = NumToVTKArray(cp1[1])
          ca3 = NumToVTKArray(cp1[2])
          ca = vtkDoubleArray()
          ca.SetNumberOfComponents(3)
          ca.SetNumberOfTuples(cp1.shape[1])
          ca.CopyComponent(0,ca1,0)
          ca.CopyComponent(1,ca2,0)
          ca.CopyComponent(2,ca3,0)
          po = vtkPoints()
          po.SetData(ca)
          cl = vtkPolyData()
          cl.SetPoints(po)
          cl.Update()
          sl = vtkImplicitSelectionLoop()
          sl.AutomaticNormalGenerationOn()
          sl.SetLoop(po)
          ii = vtkImplicitFunctionToImageStencil()
          ii.SetInput(sl)
          ii.SetOutputWholeExtent(0,x1-x0,0,y1-y0,e[4],e[5])
          t = vtkImageStencil()
          t.SetStencil( ii.GetOutput())
#          ixc = ix[y0:y1,x0:x1].astype(ix.dtype)
#          ic = NumToVTKImage(ixc)
          o2 = c.GetOutput()
          c.Update()
          o2 = vtkImageReslice()
          o2.SetInput( c.GetOutput())
          o2.SetOutputOrigin(x0,y0,e[4])
          o2.SetOutputExtent(0,x1-x0,0,y1-y0,0,e[5]-e[4])
          ic = o2.GetOutput()
          o2.Update()
          ic.SetOrigin(0,0,0)
          t.SetInput(ic)
          t.SetBackgroundValue(fill1)
          t.ReverseStencilOn()
          o = t.GetOutput()
          o.Update()
          c.DrawImage(x0,y0,o)
       for segment in contour:
          if color1:
             x,y = floor(segment+0.5).astype(int32)
             x = clip(x,e[0],e[1])
             y = clip(y,e[2],e[3])
             c.SetDrawColor(color1)
             c.DrawSegment3D(x[0],y[0],0.0,x[1],y[1],0.0)

    if verb: print
    o = c.GetOutput()
    c.Update()
    n = VTKImageToNum(o)
    return n

def VTKEnclosedContour(ix,contours,verb=0,speed=0,pixels=0,new=1,centers=0):
    enclosed = []
    mags = []
    ims = {}
    for jc,contour in enumerate(contours):
       if verb:
          if jc and not (jc % (len(contours)/40+1)):
             sys.stdout.write(".")
             sys.stdout.flush()
       (xb,yb),(xe,ye) = transpose(contour)
       x0,x1 = int(min(xb)), int(max(xb)+1.0)
       y0,y1 = int(min(yb)), int(max(yb)+1.0)
       xwid, ywid = x1-x0, y1-y0
       if speed:
          mwid = min([xwid,ywid])
          mag = int(1.*mwid/speed+1)
          mcontour = 1.*contour/mag
          if mag not in ims:
             print mag
             if mag == 1: ims[mag] = ix
             else: ims[mag] = VTKShrinkN(ix,mag,mag,numret=1,usemean=1)*pow(mag,2)
             if verb: print mag,xwid,ywid,mean(xb),mean(yb),ix.shape,ims[mag].shape
          im = ims[mag]
       else:
          im = ix
          mcontour = 1.*contour
          mag = 1.0
       (xb,yb),(xe,ye) = transpose(mcontour)
       if new:
          x0,x1 = int(min(xb)), int(max(xb)+1.0)
          y0,y1 = int(min(yb)), int(max(yb)+1.0)
          x0,x1 = clip([x0,x1],0,im.shape[1]-1)
          y0,y1 = clip([y0,y1],0,im.shape[0]-1)
          ixc = im[y0:y1,x0:x1].astype(ix.dtype)
          yco,xco = indices(ixc.shape)
          t = Interiors(xco.ravel()+x0,yco.ravel()+y0,mcontour)
          interior = less(abs(t-2*pi),1e-4)
          n = ixc*reshape(interior,ixc.shape).astype(ix.dtype)
       else:
          x0,x1 = int(min(xb)), int(max(xb)+1.0)
          y0,y1 = int(min(yb)), int(max(yb)+1.0)
          x0,x1 = clip([x0,x1],0,im.shape[1]-1)
          y0,y1 = clip([y0,y1],0,im.shape[0]-1)
          cx1,cy1 = 1.*transpose(asarray(mcontour)[::,::,0]).astype(float64)
          cx2,cy2 = 1.*transpose(asarray(mcontour)[::,::,1]).astype(float64)
          cx1 = clip(cx1,0,im.shape[1]-1)
          cy1 = clip(cy1,0,im.shape[0]-1)
          cz1 = zeros(cx1.shape,cx1.dtype).astype(float64)
          cp1 = 1.*asarray([cx1-x0,cy1-y0,cz1])
          ca1 = NumToVTKArray(cp1[0])
          ca2 = NumToVTKArray(cp1[1])
          ca3 = NumToVTKArray(cp1[2])
          ca = vtkDoubleArray()
          ca.SetNumberOfComponents(3)
          ca.SetNumberOfTuples(cp1.shape[1])
          ca.CopyComponent(0,ca1,0)
          ca.CopyComponent(1,ca2,0)
          ca.CopyComponent(2,ca3,0)
          po = vtkPoints()
          po.SetData(ca)
          sl = vtkImplicitSelectionLoop()
          sl.AutomaticNormalGenerationOn()
          sl.SetLoop(po)
          ii = vtkImplicitFunctionToImageStencil()
          ii.SetInput(sl)
          ii.SetOutputWholeExtent(0,(x1-x0),0,(y1-y0),0,0)
          t = vtkImageStencil()
          t.SetStencil( ii.GetOutput())
          ixc = im[y0:y1,x0:x1].astype(ix.dtype)
          ic = NumToVTKImage(ixc)
          t.SetInput(ic)
          t.SetBackgroundValue(0.0)
          t.ReverseStencilOff()
          o = t.GetOutput()
          o.Update()
          n = VTKImageToNum(o)
       if pixels:
          if len(n.shape)==2:
             yco,xco = indices(n.shape,n.dtype)
             xco = mag*(xco + x0)
             yco = mag*(yco + y0)
             enc1 = compress(n.ravel(),[xco.ravel(),yco.ravel(),n.ravel()],1)
          else:
             enc1 = zeros((3,0,),float64)
          enclosed.append(enc1)
          mags.append(mag)
       else:
          if len(n.shape)==2:
             yco,xco = indices(n.shape,n.dtype)
             xco = mag*(xco + x0)
             yco = mag*(yco + y0)
             xco, yco, zco = compress(n.ravel(),[xco.ravel(),yco.ravel(),n.ravel()],1)
             xcen = divz(sum(xco*zco), sum(zco))
             ycen = divz(sum(yco*zco), sum(zco))
             nixc = n
             if centers:
                enc1 = sum(nixc.ravel()),xcen,ycen
             else:
                enc1 = sum(nixc.ravel())
          else:
             if centers:
                enc1 = 0.0,0.0,0.0
             else:
                enc1 = 0.0
          enclosed.append(enc1)
    if verb: print
    if not pixels:
       enclosed = asarray(enclosed)
       return enclosed
    else:
       mags = asarray(mags)
       return enclosed,mags


def VTKDrawContours(x,vs,color=1,fill=0,verb=0,nsamp=1,minsides=0):
    if type(x) == numpy.ndarray: i = NumToVTKImage(x)
    else: i = x
    c = vtkImageCanvasSource2D()
    e = i.GetExtent()
    c.SetExtent(e[0],e[1],e[2],e[3],e[4],e[5])
    c.SetScalarType( i.GetScalarType())
    n = zeros(x.shape,x.dtype)
    for v in sort(vs)[::-1]:
      if color <= 0: c.SetDrawColor(v)
      else: c.SetDrawColor(color)
      contours = VTKGetContours(i,v,verb=verb,nsamp=nsamp,minsides=minsides)
      if contours:
         n = n + VTKDrawContour(x,contours,color=1,fill=0)
    return n

def VTKSeedConnectivity(t,x=[],y=[],val=None):
    if type(t) == numpy.ndarray:
       i = NumToVTKImage(t)
    else: i = t
    i2 = VTKRecast(i,"b")
    sc = vtkImageSeedConnectivity()
    sc.SetInput( i2)
    if not len(x):
       y,x = indices(t.shape)
       y,x = compress(t.ravel(),[y.ravel(),x.ravel()],1)
    for j in range(len(x)):
        sc.AddSeed(int(x[j]),int(y[j]))
        ival = t[int(y[j]),int(x[j])]
        if val: oval = val
        else: oval = 1
        sc.SetInputConnectValue(1)
        sc.SetOutputConnectedValue(oval)
        sc.SetOutputUnconnectedValue(0)
    so = sc.GetOutput()
    so.SetScalarTypeToUnsignedChar()
    sc.Update()
    so2 = VTKRecast(so ,"f")
    si = VTKImageToNum(so2)
    return si

def VTKIslandRemoval(x,isval,repval,area,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageIslandRemoval2D()
      d.SetIslandValue(isval)
      d.SetReplaceValue(repval)
      d.SetAreaThreshold(area)
      d.SquareNeighborhoodOn()
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKOpenClose(x,openval,closeval,k=5,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageOpenClose3D()
      d.SetKernelSize(k,k,1)
      d.SetOpenValue(openval)
      d.SetCloseValue(closeval)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKErode(x,k=5,l=0,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageContinuousErode3D()
      if not l: l = k
      d.SetKernelSize(k,l,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKDilate(x,k=5,l=None,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      if l == None: l = k
      d = vtkImageContinuousDilate3D()
      d.SetKernelSize(k,l,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKSkeleton(x,iter,prune=0,numret=1):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageSkeleton2D()
      d.SetNumberOfIterations(iter)
      d.SetPrune(prune)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKGauss(x,sigma=5,numret=1,radius=3,sigmax=None,sigmay=None):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      if sigmax == None: sigmax=sigma
      if sigmay == None: sigmay=sigma
      d = vtkImageGaussianSmooth()
      d.SetStandardDeviations(sigmax,sigmay,0)
      d.SetRadiusFactors(radius,radius,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKConstantPad(im,t,c,numret=0):
      i = NumToVTKImage(im)
      d = i.GetDimensions()
      e = i.GetExtent()
      ip = vtkImageConstantPad()
      ip.SetOutputWholeExtent(t[0],t[1],t[2],t[3],0,0)
      ip.SetConstant(c)
      ip.SetInput(i)
      o = ip.GetOutput()
      ip.Update()
      if numret: return VTKImageToNum(o)
      else: return o


def VTKMirrorPad(im,t,numret=0):
      i = NumToVTKImage(im)
      d = i.GetDimensions()
      e = i.GetExtent()
      ip = vtkImageMirrorPad()
      ip.SetOutputWholeExtent(t[0],t[1],t[2],t[3],0,0)
      ip.SetInput(i)
      o = ip.GetOutput()
      ip.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKCorrelation(im1,im2,numret=0,clip=1,x=1,y=1,wrap=0,mirror=1,constant=None,kernel=None):
      if type(im1) == numpy.ndarray: i1 = NumToVTKImage(im1)
      else: i1 = im1
      if type(im2) == numpy.ndarray: i2 = NumToVTKImage(im2)
      else: i2 = im2
      d = i1.GetDimensions()
      e = i1.GetExtent()
      d2 = i2.GetDimensions()
      e2 = i2.GetExtent()
      kx = im2.shape[1]/2
      ky = im2.shape[0]/2
      if constant != None:
          ip = vtkImageConstantPad()
          if d[0] > 1: x0,x1=-kx,d[0]+kx-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-kx,d[1]+kx-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetConstant(constant)
          ip.SetInput(i1)
          i1 = ip.GetOutput()
          ip.Update()
      elif mirror:
          ip = vtkImageMirrorPad()
          if d[0] > 1: x0,x1=-kx,d[0]+kx-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-ky,d[1]+ky-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetInput(i1)
          i1 = ip.GetOutput()
          ip.Update()
      elif wrap:
          ip = vtkImageWrapPad()
          if d[0] > 1: x0,x1=-kx,d[0]+kx-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-ky,d[1]+ky-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetInput(i1)
          i1 = ip.GetOutput()
          ip.Update()
      c = vtkImageCorrelation()
      c.SetInput1(i1)
      c.SetInput2(i2)
      o = c.GetOutput()
      c.Update()
      if clip:
         ic = vtkImageClip()
         notx = -kx
         noty = -ky
         ic.SetOutputWholeExtent(notx,d[0]-1+notx,noty,d[1]-1+noty,0,0)
         ic.SetInput(o)
         ic.ClipDataOn()
         o = ic.GetOutput()
         ic.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKConvolve(im,numret=0,k=5,clip=1,x=1,y=1,wrap=0,mirror=1,constant=None,kernel=None):
      if type(im) == numpy.ndarray: i = NumToVTKImage(im)
      else: i = im
      d = i.GetDimensions()
      e = i.GetExtent()
      dtest = sort(d)
      if sum(dtest[:2]) == 2: onedim=1
      else: onedim = 0
      if (d[0] == 1): oned = 1
      elif (d[1] == 1): oned = 2
      else: oned = 0
      if x and not y: oned = 2
      if y and not x: oned = 1
      if onedim: oned = 1
      if constant != None:
          ip = vtkImageConstantPad()
          if d[0] > 1: x0,x1=-k,d[0]+k-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-k,d[1]+k-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetConstant(constant)
          ip.SetInput(i)
          i = ip.GetOutput()
          ip.Update()
      elif mirror:
          ip = vtkImageMirrorPad()
          if d[0] > 1: x0,x1=-k,d[0]+k-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-k,d[1]+k-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetInput(i)
          i = ip.GetOutput()
          ip.Update()
      elif wrap:
          ip = vtkImageWrapPad()
          if d[0] > 1: x0,x1=-k,d[0]+k-1
          else: x0,x1=0,0
          if d[1] > 1: y0,y1=-k,d[1]+k-1
          else: y0,y1=0,0
          ip.SetOutputWholeExtent(x0,x1,y0,y1,0,0)
          ip.SetInput(i)
          i = ip.GetOutput()
          ip.Update()
      c = vtkImageConvolve()
      if kernel == None:
          if k == 3: k1 = asarray([0.25,0.5,0.25])
          elif k == 5: k1 = asarray([0.0625,0.25,0.375,0.25,0.0625])
          if onedim: ke = ones((k,k),float64) * k1
          elif not oned: ke = k1[::,NewAxis]*k1[NewAxis,::]
          elif oned == 1: ke = zeros((k,k),float64); ke[::,2] = k1
          elif oned == 2: ke = zeros((k,k),float64); ke[2] = k1
          ke = ravel(ke)
          ke = ke / sum(ke)
          if onedim: ke = len(k1)*ke
      else:
          k = kernel.shape[0]
          ke = ravel(kernel)
      if k == 3: c.SetKernel3x3(ke)
      if k == 5: c.SetKernel5x5(ke)
      if k == 7: c.SetKernel7x7(ke)
      c.SetInput(i)
      o = c.GetOutput()
      c.Update()
      if clip:
         ic = vtkImageClip()
         notx = -(not x) * 0
         noty = -(not y) * 0
         ic.SetOutputWholeExtent(notx,d[0]-1+notx,noty,d[1]-1+noty,0,0)
         ic.SetInput(o)
         ic.ClipDataOn()
         o = ic.GetOutput()
         ic.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKImageClip(x,d,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      e = i.GetDimensions()
      ic = vtkImageClip()
      ic.SetOutputWholeExtent( e[0]/2-d[1]/2, e[0]/2+d[1]/2-1, e[1]/2-d[0]/2, e[1]/2+d[0]/2-1,0,0)
      ic.SetInput(i)
      ic.ClipDataOn()
      o = ic.GetOutput()
      ic.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKHybridMedian(x,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = vtkImageHybridMedian2D()
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKVariance(x,n=3,m=None,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      if not m: m = n
      d = vtkImageVariance3D()
      d.SetKernelSize(n,m,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o


def VTKMedian(x,n=3,m=None,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      if not m: m = n
      d = vtkImageMedian3D()
      d.SetKernelSize(n,m,1)
      d.SetInput(i)
      o = d.GetOutput()
      d.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKShrinkN(x,n=2,m=None,numret=0,xshift=0,yshift=0,usemean=0,usemed=0,usemax=0,usemin=0,verb=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if not m: m = n
      if d[0] == 1: n = 1
      if d[1] == 1: m = 1
      rx,ry,rz = i.GetDimensions()
      tx,ty = rx,ry
      sn = n
      sm = m
      pad = 0
      if rx % sn or ry % sm:
         tx = int(1.*rx/sn+0.5)*sn
         ty = int(1.*ry/sm+0.5)*sm
         if rx == 1: tx = 1
         if ry == 1: ty = 1
         if verb: print "padding to",tx,ty,rx,ry
         pad = 1
         ip = vtkImageMirrorPad()
         ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
         ip.SetInput(i)
         i = ip.GetOutput()
         ip.Update()
      im = vtkImageShrink3D()
      im.SetShrinkFactors(n,m,1)
      im.MeanOff(); im.MedianOff(); im.MaximumOff(); im.MinimumOff()
      if usemed: im.MedianOn()
      elif usemean: im.MeanOn()
      elif usemax: im.MaximumOn()
      elif usemin: im.MinimumOn()
      im.SetShift(xshift,yshift,0)
      im.SetInput(i)
      o = im.GetOutput()
      o.SetSpacing(1,1,1)
      o.Update()
      im.Update()
      if numret: return VTKImageToNum(o)
      else: return o

def VTKMedianSmooth(x,n,m=None,numret=0):
     if not m: m = n
     a = VTKShrinkN(x,n,m,usemean=0,usemed=1,numret=0)
     xs = -0.5*(n-1)
     ys = -0.5*(m-1)
     b = VTKGrowN(a,n,m,origsize=x.shape,interp=1,numret=1,mirror=1,xshift=xs,yshift=ys)
     return b

def VTKMeanUp(x,l,radix=2):
    i = x
    for j in range(l):
      m = pow(radix,j+1)+1
      i = VTKMeanSmooth(i,m,numret=1)
    return i

def VTKMedianUp(x,l,radix=2):
    i = x
    for j in range(l):
      m = pow(radix,j+1)+1
      i = VTKMedianSmooth(i,m,numret=1)
    return i

def VTKMeanSmooth(x,n,m=None,numret=0):
     if not m: m = n
     a = VTKShrinkN(x,n,m,usemean=1,numret=0)
     xs = -0.5*(n-1)
     ys = -0.5*(m-1)
     b = VTKGrowN(a,n,m,origsize=x.shape,interp=1,numret=1,mirror=1,xshift=xs,yshift=ys)
     return b

def oldVTKMeanSmooth(x,n,m=None,numret=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      if m == None: m = n
      s = VTKResample(i,1.0/n,1.0/m,interp=1)
      o = VTKResample(s,n,m,interp=1)
      if numret: return VTKImageToNum(o)
      else: return o

def VTKResample(x,n,m=None,numret=0,origsize=None,interp=0,cubic=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      r = vtkImageResample()
      if interp: r.InterpolateOn()
      else: r.InterpolateOff()
      if cubic: r.SetInterpolationModeToCubic()
      if not m: m=n
      r.SetAxisMagnificationFactor(0,n)
      r.SetAxisMagnificationFactor(1,m)
      r.SetInput(i)
      o = r.GetOutput()
      r.Update()
      if origsize == None:
        origsize = o.GetDimensions()
      p = vtkImageMirrorPad()
      p.SetOutputWholeExtent(0,origsize[0]-1,0,origsize[1]-1,0,0)
      p.SetInput(o)
      f = p.GetOutput()
      p.Update()
      o = f
      if numret: return VTKImageToNum(o)
      else: return o


def VTKImageCorrelation(x,y):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      if type(y) == numpy.ndarray: j = NumToVTKImage(y)
      else: j = y
      c = vtkImageCorrelation()
      c.SetDimensionality(2)
      c.SetInput1(i)
      c.SetInput2(j)
      o = c.GetOutput()
      c.Update()
      return VTKImageToNum(o)

def VTKFFT(x,y):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      c = vtkImageFFT()
      c.SetInput(i)
      o = c.GetOutput()
      c.Update()
      return VTKImageToNum(o)

def VTKGrowN(x,n=2,m=None,numret=0,origsize=None,interp=0,wrap=0,constant=None,mirror=0,xshift=0,yshift=0):
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if not m: m = n
      if d[0] == 1: n = 1
      if d[1] == 1: m = 1
      im = vtkImageMagnify()
      im.SetMagnificationFactors(n,m,1)
      if interp: im.InterpolateOn()
      else: im.InterpolateOff()
      im.SetInput(i)
      o = im.GetOutput()
      im.Update()
      o.SetSpacing(1,1,1)
      o.Update()
      o = VTKImageShift(o,xshift,yshift,interp=1,wrap=wrap,constant=constant,mirror=mirror)
      if origsize:
            if len(origsize) == 1: origsize=(1,)+origsize
            p = vtkImageMirrorPad()
            p.SetOutputWholeExtent(0,origsize[1]-1,0,origsize[0]-1,0,0)
            p.SetInput(o)
            f = p.GetOutput()
            p.Update()
            o = f
      if numret: return VTKImageToNum(o)
      else: return o

def oldVTKLevel(x,level=1,k=5,radix=2,pad=0,verb=0):
      x = x.astype(float32)
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if level: 
         level = level
      else: 
         nx = int(log2(d[0]-1))
         ny = int(log2(d[1]-1))
         level = min([nx,ny])
      f = i

      for j in range(level):
         rx,ry,rz = f.GetDimensions()
         tx,ty = rx,ry
         s = radix
         if rx % s or ry % s:
            tx = int(1.*rx/s+1)*s
            ty = int(1.*ry/s+1)*s
            if rx == 1: tx = 1
            if ry == 1: ty = 1
            if verb: print "padding to",tx,ty,rx,ry
            ip = vtkImageMirrorPad()
            ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
            ip.SetInput(f)
            f = ip.GetOutput()
            ip.Update()

         f = VTKShrinkN(f,n=radix)

         if verb: print j,f.GetDimensions()

      for l in range(j+1):
             f = VTKConvolve(VTKGrowN(f,n=radix),k=k,clip=0)

      ic = vtkImageClip()
      ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
      ic.SetInput(f)
      ic.ClipDataOn()
      f = ic.GetOutput()
      ic.Update()

      y = VTKImageToNum(f)
      return y

def VTKLevelErode(x,level=1,k=5,radix=2,pad=0,verb=0):
      x = x.astype(float32)
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if level: 
         level = level
      else: 
         nx = int(log2(d[0]-1))
         ny = int(log2(d[1]-1))
         level = min([nx,ny])
      f = i

      for j in range(level):
         rx,ry,rz = f.GetDimensions()
         tx,ty = rx,ry
         s = radix
         if rx % s or ry % s:
            tx = int(1.*rx/s+1)*s
            ty = int(1.*ry/s+1)*s
            if rx == 1: tx = 1
            if ry == 1: ty = 1
            if verb: print "padding to",tx,ty,rx,ry
            ip = vtkImageMirrorPad()
            ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
            ip.SetInput(f)
            f = ip.GetOutput()
            ip.Update()

         f = VTKShrinkN(f,n=radix)

         if verb: print j,f.GetDimensions()

      for l in range(j+1):
             f = VTKErode(VTKGrowN(f,n=radix),k=k)

      ic = vtkImageClip()
      ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
      ic.SetInput(f)
      ic.ClipDataOn()
      f = ic.GetOutput()
      ic.Update()

      y = VTKImageToNum(f)
      return y

def VTKLevelMedian(x,level=1,k=5,radix=2,pad=0,verb=0):
      x = x.astype(float32)
      if type(x) == numpy.ndarray: i = NumToVTKImage(x)
      else: i = x
      d = i.GetDimensions()
      if level: 
         level = level
      else: 
         nx = int(log2(d[0]-1))
         ny = int(log2(d[1]-1))
         level = min([nx,ny])
      f = i

      for j in range(level):
         rx,ry,rz = f.GetDimensions()
         tx,ty = rx,ry
         s = radix
         if rx % s or ry % s:
            tx = int(1.*rx/s+1)*s
            ty = int(1.*ry/s+1)*s
            if rx == 1: tx = 1
            if ry == 1: ty = 1
            if verb: print "padding to",tx,ty,rx,ry
            ip = vtkImageMirrorPad()
            ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
            ip.SetInput(f)
            f = ip.GetOutput()
            ip.Update()

         f = VTKShrinkN(f,n=radix,usemed=1)

         if verb: print j,f.GetDimensions()

      for l in range(j+1):
             f = VTKMedian(VTKGrowN(f,n=radix),n=k,clip=0)

      ic = vtkImageClip()
      ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
      ic.SetInput(f)
      ic.ClipDataOn()
      f = ic.GetOutput()
      ic.Update()

      y = VTKImageToNum(f)
      return y

def VTKWave(x,level=0,radix=2,pad=0,wt=None,verb=0):

      if type(x) == type(arange(10)): i = NumToVTKImage(x)
      else: i = x
      if wt:
        if type(wt) == type(arange(10)):
           wt = wt/mean(wt.ravel())
           wt = wt.astype(x.dtype)
           iw = NumToVTKImage(wt)
        else: iw = wt
      d = i.GetDimensions()

      if level: 
         level = level
      else: 
         if d[0] >1: nx = int(log2(d[0]-1)/log2(radix))
         else: nx = 99999
         if d[1] >1: ny = int(log2(d[1]-1)/log2(radix))
         else: ny = 99999
         level = min([nx,ny])
      f = i
      if wt:
        fw = iw
#        tw = VTKWave(wt,level=level)
#        atw = asarray([sum(tw[l:]) for l in range(level+1)])

      at = []
      for j in range(level):

         rx,ry,rz = f.GetDimensions()
         tx,ty = rx,ry
         s = radix
         if rx % s or ry % s:
            tx = int(1.*rx/s+1)*s
            ty = int(1.*ry/s+1)*s
            if rx == 1: tx = 1
            if ry == 1: ty = 1
            if verb: print "padding to",tx,ty

            ip = vtkImageMirrorPad()
            ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
            ip.SetInput(f)
            f = ip.GetOutput()
            ip.Update()

            if wt:
              ip = vtkImageMirrorPad()
              ip.SetOutputWholeExtent(0,tx-1,0,ty-1,0,0)
              ip.SetInput(fw)
              fw = ip.GetOutput()
              ip.Update()

         cf = VTKShrinkN(f,n=radix)
         if wt:
           cfw = VTKShrinkN(fw,n=radix)
           y = VTKImageToNum(cf)
           z = VTKImageToNum(cfw)
           y = y*z
           cf = NumToVTKImage(y)

#           op = vtkImageMathematics()
#           op.SetOperationToMultiply()
#           op.SetInput1(cf)
#           op.SetInput2(cfw)
#           op.Update()
#           cf = op.GetOutput()

         gf = VTKConvolve(VTKGrowN(cf,n=radix),k=5)

         op = vtkImageMathematics()
         op.SetOperationToSubtract()
         op.SetInput1(f)
         op.SetInput2(gf)
         op.Update()
         gf = op.GetOutput()

         for k in range(j): gf = VTKConvolve(VTKGrowN(gf,n=radix),k=5)

         ic = vtkImageClip()
         ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
         ic.SetInput(gf)
         ic.ClipDataOn()
         gf = ic.GetOutput()
         ic.Update()
         y = VTKImageToNum(gf)

#         if wt:
#            gfw = VTKConvolve(VTKGrowN(cfw),k=5)
#            for k in range(j): gfw = VTKConvolve(VTKGrowN(gfw),k=5)
#            ic = vtkImageClip()
#            ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
#            ic.SetInput(gfw)
#            ic.ClipDataOn()
#            gfw = ic.GetOutput()
#            ic.Update()
#            yw = VTKImageToNum(gfw)
#            y = divz(y,yw)
#            y = y * atw[level]

         at.append(y)

         f = cf
         if wt: fw = cfw

      for k in range(j+1):
         cf = VTKConvolve(VTKGrowN(cf,n=radix),k=5)
         cd = cf.GetDimensions()
         if cd[0] > d[0] or cd[1] > d[1]:
           ic = vtkImageClip()
           ic.SetOutputWholeExtent(0,d[0]-1,0,d[1]-1,0,0)
           ic.SetInput(cf)
           ic.ClipDataOn()
           cf = ic.GetOutput()
           ic.Update()
      y = VTKImageToNum(cf)
      at = concatenate([at,[y]])

      return asarray(at)


def VTKWtWave(x,w,level=0,verb=0):
      d = list(x.shape)
      d.reverse()
      if level: 
         level = level
      else: 
         if d[0] >1: nx = int(log2(d[0]-1))
         else: nx = 99999
         if d[1] >1: ny = int(log2(d[1]-1))
         else: ny = 99999
         level = min([nx,ny])
      f = x
      at = []
      for j in range(level):
         if verb: print "level=",j
         xwj = VTKLevel(x*w,j+1,verb=verb)
         wj = VTKLevel(w,j+1,verb=verb)
         l = divz(xwj,wj).astype(float32)
         a = (f - l).astype(float32)
         at.append(a)
         f = l
      at = asarray(at).astype(float32)
      p = (x - sum(at)).astype(float32)
      at = concatenate([at,[p]])
      return at


def VTKBoxToActor(x,y,color=(1,1,1),wid=1):
     eacts = []
     for j in range(4):
       edge = vtkLineSource()
       edge.SetPoint1(x[j],y[j],0)
       edge.SetPoint2(x[j+1],y[j+1],0)
       edge.SetResolution(10)
       emap = vtkPolyDataMapper()
       emap.SetInput(edge.GetOutput())
       eact = vtkActor()
       eact.SetMapper(emap)
       eact.VisibilityOn()
       eact.GetProperty().SetColor(color)
       eact.GetProperty().SetLineWidth(wid)
       eact.GetProperty().SetOpacity(0.7)
       eacts.append(eact)
     return eacts

def VTKAtrous(c,lev=None,radix=2,k=5,x=1,y=1,interp=1,mirror=1,wrap=0,verb=0):
     if x: xradix = radix
     else: xradix = 1
     if y: yradix = radix
     else: yradix = 1
     if not lev:
       nx = int(log2(c.shape[0]-1)/log2(xradix))
       ny = int(log2(c.shape[1]-1)/log2(yradix))
       lev = min([nx,ny])
     imb = copy.copy(c)
     imsizes = []
     tot = zeros(c.shape)
     if interp: xs, ys = -0.5*(xradix-1),-0.5*(yradix-1)
     else: xs, ys = 0.0, 0.0
     for p in range(lev):
       imsizes.append( imb.shape)
       imo = VTKConvolve(imb,k=k,numret=1,clip=1)
       imc = VTKShrinkN(imo,xradix,yradix,numret=1,usemean=1)
       imd = VTKGrowN(imc,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap)
       imw = (imb - imd).astype(float32)
       del imd,imo
       if verb: print p,imc.shape
       imb = imc
       for q in range(p):
          imw = VTKGrowN(imw,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imsizes[p-q-1],interp=interp,mirror=mirror,wrap=wrap)
       tot = (tot + imw).astype(float32)
       if p: uwt = concatenate([uwt,[imw.astype(float32)]])
       else: uwt = [imw.astype(float32)]
       del imw
     imw = (c - tot).astype(float32)
     uwt = concatenate([uwt,[imw.astype(float32)]])
     return uwt

def VTKAtrousWt(c,wc,lev=None,radix=2,k=5,x=1,y=1,interp=1,mirror=1,wrap=0,verb=0):
     if x: xradix = radix
     else: xradix = 1
     if y: yradix = radix
     else: yradix = 1
     if not lev:
       nx = int(log2(c.shape[0]-1)/log2(xradix))
       ny = int(log2(c.shape[1]-1)/log2(yradix))
       lev = min([nx,ny])
     imb = copy.copy(c)
     wimb = copy.copy(wc)
     imsizes = []
     tot = zeros(c.shape)
     if interp: xs, ys = -0.5*(xradix-1),-0.5*(yradix-1)
     else: xs, ys = 0.0, 0.0
     for p in range(lev):
       imsizes.append( imb.shape)
       imo1 = VTKConvolve(imb*wimb,k=k,numret=1,clip=1)
       imc1 = VTKShrinkN(imo1,xradix,yradix,numret=1,usemean=1)
       imd1 = VTKGrowN(imc1,xradix,yradix,numret=1,xshift=xs,yshift=ys,origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap)
       imo2 = VTKConvolve(wimb,k=k,numret=1,clip=1)
       imc2 = VTKShrinkN(imo2,xradix,yradix,numret=1,usemean=1)
       imd2 = VTKGrowN(imc2,xradix,yradix,numret=1,xshift=xs,yshift=ys,origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap)
       imc = VTKDivide(imc1,imc2)
       imd = VTKDivide(imd1,imd2)
       imw = (imb - imd).astype(float32)
       del imd1,imd2,imo1,imo2,imd,imc1
       if verb: print p,imc.shape
       imb = imc
       wimb = imc2
       for q in range(p):
          imw = VTKGrowN(imw,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imsizes[p-q-1],interp=interp,mirror=mirror,wrap=wrap)
       tot = (tot + imw).astype(float32)
       if p: uwt = concatenate([uwt,[imw.astype(float32)]])
       else: uwt = [imw.astype(float32)]
       del imw
     imw = (c - tot).astype(float32)
     uwt = concatenate([uwt,[imw.astype(float32)]])
     return uwt

def VTKLevel(c,lev=None,radix=2,k=5,x=1,y=1,interp=1,mirror=1,wrap=0,constant=None,verb=0,usemean=1,usemed=0,usemax=0,usemin=0,useext=0):
     if len(c.shape) == 1: x,y=1,0
     if x: xradix = radix
     else: xradix = 1
     if y: yradix = radix
     else: yradix = 1
     imb = c.astype(float32)
     imsizes = []
     tot = zeros(c.shape,float32)
     if interp and (usemean or usemed or usemax or usemin or useext):
          xs = -0.5*0.5
          ys = -0.5*0.5
     else: xs, ys = 0.0, 0.0
     for p in range(lev):
       imsizes.append( imb.shape)
       imo = VTKConvolve(imb,k=k,numret=1,clip=1,x=x,y=y,mirror=mirror,wrap=wrap,constant=constant)
       if useext:
         usemax = not (p % 2)
         usemin = (p % 2)
       imc = VTKShrinkN(imo,xradix,yradix,numret=1,usemean=usemean,usemed=usemed,usemax=usemax,usemin=usemin,verb=0)
       if verb: print p+1,imc.shape
       imd = VTKGrowN(imc,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap or constant != None)
       imw = VTKSubtract(imb,imd)
       del imd,imo
       imb = imc
       for q in range(p):
          imw = VTKGrowN(imw,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imsizes[p-q-1],interp=interp,mirror=mirror,wrap=wrap or constant != None)
       tot = VTKAdd(tot,imw)
       del imw
     imw = VTKSubtract(c.astype(tot.dtype),tot)
     del imb, imc, tot
     return imw

def VTKAtrousReduce(c,lev=None,radix=2,k=3,x=1,y=1,interp=1,mirror=1,wrap=0,verb=0):
     if x: xradix = radix
     else: xradix = 1
     if y: yradix = radix
     else: yradix = 1
     if not lev:
       nx = int(log2(c.shape[0]-1)/log2(xradix))
       ny = int(log2(c.shape[1]-1)/log2(yradix))
       lev = min([nx,ny])
     imb = copy.copy(c)
     imsizes = []
     tot = zeros(c.shape)
     if interp: xs, ys = -0.5*(xradix-1),-0.5*(yradix-1)
     else: xs, ys = 0.0, 0.0
     for p in range(lev+1):
       imsizes.append( imb.shape)
       imo = VTKConvolve(imb,k=k,numret=1,clip=1)
       imc = VTKShrinkN(imo,xradix,yradix,numret=1,usemean=1)
       imd = VTKGrowN(imc,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
                origsize=imb.shape,interp=interp,mirror=mirror,wrap=wrap)
       if p < lev: imw = (imb - imd).astype(float32)
       else: imw = imb
       del imd,imo
       if verb: print p,imc.shape
       imb = imc
       if p: uwt += [imw.astype(float32)]
       else: uwt = [imw.astype(float32)]
     del imb, imc
     return uwt,imsizes

def VTKAtrousRescale(imw,p,imsizes,radix=2,k=3,x=1,y=1,interp=1,mirror=1,wrap=0,verb=0):
    if x: xradix = radix
    else: xradix = 1
    if y: yradix = radix
    else: yradix = 1
    if interp: xs, ys = -0.5*(xradix-1),-0.5*(yradix-1)
    else: xs, ys = 0.0, 0.0
    for q in range(p):
       imw = VTKGrowN(imw,xradix,yradix,numret=1,xshift=xs,yshift=ys,\
             origsize=imsizes[p-q-1],interp=interp,mirror=mirror,wrap=wrap)
       if verb: print q,imw.shape
    return imw

def VTKAniModel(image,bad=2.5,th=3.5,mx=25,my=3,fac=0.2,iter=100,spec=1,mx2=11,my2=7):
   mimage = VTKMedian(image,n=mx,m=1,numret=1)
   mimage = VTKMedian(mimage,n=1,m=my,numret=1)
   if spec:
     mimage = VTKGauss(mimage,sigmax=4,sigmay=2,numret=1)
   else:
     mimage = VTKGauss(mimage,sigmax=4,sigmay=4,numret=1)
   rimage = image - mimage
   if spec: simage = 1.49*VTKMedian(abs(rimage),mx,1,numret=1)
   else: simage = 1.49*VTKMedian(abs(rimage),mx,my,numret=1)
   eimage = divz(rimage,simage)
   good = between(eimage,-bad,bad)
   vimage1 = VTKGauss(image*good,sigmax=mx/2,sigmay=my/2,numret=1)
   vimage2 = VTKGauss(good.astype(float32),sigmax=mx/2,sigmay=my/2,numret=1)
   vimage =  divz( vimage1, vimage2)
   wt = good * getsqrt(vimage)
   sol = linear_least_squares(transpose([mimage.ravel(),vimage.ravel()])*wt.ravel()[::,NewAxis],image.ravel()*wt.ravel())
   oimage = sol[0][0]*mimage+sol[0][1]*vimage
   for j in range(1):
      nimage = image - oimage
      timage = 1.49*VTKMedian(abs(nimage),mx2,my2,numret=1)
      nimage = divz(nimage,timage)
      good = between(nimage,-th,th)
      uimage = where(good,nimage,0)
      aimage = VTKAnisotropicDiffusion(uimage,th=th,fac=fac,iter=iter,edges=1,faces=0,corners=0,gmag=1)
      oimage = aimage*timage + oimage
   return oimage

MPI = {}

def MPsmode(x,m,n,pct=25.0):
    nx = int((x.shape[1]-1.0)/m)+1
    ny = int((x.shape[0]-1.0)/n)+1
    o = zeros((ny,nx),float32)
    o2 = zeros((ny,nx),float32)
    MPI["x"] = x
    args = []
    y0s = arange(ny)*n
    y1s = clip(y0s+n,0,x.shape[0])
    x0s = arange(nx)*m
    x1s = clip(x0s+m,0,x.shape[1])
    args = [[None,x0,x1,y0,y1,pct] for y0,y1 in zip(y0s,y1s) for x0,x1 in zip(x0s,x1s)]
    print "Running in parallel?"
    t0 = time.time()
    result = RunMinitasks(MPsmode1,args,globals())
    t1 = time.time()
    print "Time=%.2s"%(t1-t0)
    o,o2 = transpose(asarray(result))
    o = reshape(o,(ny,nx)).astype(float)
    o2 = reshape(o2,(ny,nx)).astype(float)
    return o,o2

def smode(x,m,n,pct=25.0):
    nx = int((x.shape[1]-1.0)/m)+1
    ny = int((x.shape[0]-1.0)/n)+1
    o = zeros((ny,nx),float32)
    o2 = zeros((ny,nx),float32)
    for j in range(ny):
        y0,y1 = clip([j*n,(j+1)*n],0,x.shape[0])
        for i in range(nx):
            x0,x1 = clip([i*m,(i+1)*m],0,x.shape[1])
            l,l2 = smode1(x,x0,x1,y0,y1,pct=pct)
            o[j,i] = l
            o2[j,i] = l2
    return o,o2

def MPsmode1(args):
    x,x0,x1,y0,y1,pct = args
    return smode1(x,x0,x1,y0,y1,pct=pct)

def smode1(x,x0,x1,y0,y1,pct=25):
    if x == None: d = MPI["x"][y0:y1,x0:x1].ravel()
    else: d = x[y0:y1,x0:x1].ravel()
    d = compress(not_equal(d,0),d)
    l,l2 = 0.0,0.0
    if len(d):
       d = sort(d)
       s = diff(d)
       p = d[:-1]+s/2.
       u = int(0.01*pct*len(s))+1
       s,p = take([s,p],argsort(s)[:u],1)
       l = median(p)
       d = sort(abs(d-l))
       s = diff(d)
       p = d[:-1]+s/2.
       u = int(1.0*len(s))+1
       s,p = take([s,p],argsort(s)[:u],1)
       l2 = 1.49*median(p)
    return l,l2

def ModalSmooth(data,n,k,cut=10,pct=10):
    udata = data
    okay = not_equal(udata,0)*numpy.isfinite(udata).astype("float32")

    M,R = MPsmode(udata,n,n,pct=pct)
    if k>1:
       M = VTKGauss(VTKMedian(M,k,k,numret=1),0.5,1)
       R = VTKGauss(VTKMedian(R,k,k,numret=1),0.5,1)
#       M = VTKConvolve(M,k=3,numret=1)
#       R = VTKConvolve(R,k=3,numret=1)
#       M = VTKConvolve(where(M,M,VTKMeanSmooth(M,k,k,numret=1)),k=5,numret=1)
#       R = VTKConvolve(where(R,R,VTKMeanSmooth(R,k,k,numret=1)),k=5,numret=1)
#       G = not_equal(M,0).astype(float32)
#       M = VTKDivide(VTKConvolve(G*M,k=5,numret=1),VTKConvolve(G,k=5,numret=1))
#       R = VTKDivide(VTKConvolve(G*R,k=5,numret=1),VTKConvolve(G,k=5,numret=1))

    xs, ys = -0.5*(n-1), -0.5*(n-1)
    s = [data.shape[1],data.shape[0]]
    back = VTKImageShift(VTKResample(M,n,n,numret=1,origsize=s,interp=1,cubic=1),xs,ys,interp=1,mirror=1,numret=1)
    noise = VTKImageShift(VTKResample(R,n,n,numret=1,origsize=s,interp=1,cubic=1),xs,ys,interp=1,mirror=1,numret=1)
    obj = VTKSubtract(data,back)
    if cut:
       r = 1.49*median(compress(data.ravel(),obj.ravel()))
       obj = where(VTKLessEqual(obj,-cut),0,obj)

    return back,obj,noise





