#!/usr/bin/env python

# TO DO:
#    Derive separate astrometric solutions for J,H,K?

from twisted.python import log as logging
from exceptions import *
from basics import *
from numpy.core.numerictypes import *
import numpy
from scipy.special import *
from numpy.oldnumeric.mlab import diff
from Mestimators import *
from sets import Set
import time
from VTKHelperFunctions import *
from FITS import *
from pgplot import *
from masks import *
import cPickle, getpass
#from TwistedHub import TwistedTask, EvalTask, HubDispatch, reactor, defer, AddManhole
#from TwistedHub import task, failure
from TwistedHub import *
from FunctionMarshal import *
from SimpleQueue import *
from multiprocessing import Process
from Wavelets import *
from ReadSex import *

from CIRCEDB import *
from CIRCEHelpers import *
from CIRCEWatcher import *

FSFullWellGains = array([1.38,1.35,1.30,1.29])
FSLowNoiseGains = array([2.65,2.59,2.51,2.49])

MASKD = {1:{},2:{}}
MASKN = {1:{},2:{}}
FLATD = {1:{},2:{}}
FLATN = {1:{},2:{}}
DARKD = {1:{},2:{}}
DARKN = {1:{},2:{}}
SKYFD = {1:{},2:{}}
SKYFN = {1:{},2:{}}
JXY = {}
WDICT = {1:{},2:{}}

ZDBS = {}
STORAGES = {}

irdbfile = os.getenv("CIRCEDB")
irtarget = os.getenv("CIRCETARG")
irfilter = os.getenv("CIRCEFILTER")

if irdbfile and not os.path.isdir(irdbfile):
    IRSetup( irdbfile,irtarget,irfilter)

def IRSetup( irdbfile,irtarg,irfilter,flat=False,fclass=newfirm,nume=3):
   if flat and "flat" not in irtarg and "dome" not in irtarg and "dark" not in irfilter.lower() and "dark" not in irtarg: irtarget = "flat_"+irtarg
   else: irtarget = irtarg
   CIRCEDataset = CollateDB(irdbfile,irtarg,irfilter,fclass=fclass,nume=nume)
   if irtarget and os.path.basename(os.getcwd()) != irtarget:
      if not os.path.exists(irtarget):
          os.mkdir(irtarget)
      darks = glob("dark*c?.fits")
      if glob("flat_"+irtarg+"/Flat"+irfilter+"c?.fits"): flats = glob("flat_"+irtarg+"/Flat"+irfilter+"c?.fits")
      else: flats = glob("Flat"+irfilter+"c?.fits")
      iflats = glob("Illum"+irfilter+"c?.fits")
      if glob("flat_"+irtarg+"/mask"+irfilter+"c?.fits"): masks = glob("flat_"+irtarg+"/mask"+irfilter+"c?.fits")
      else: masks = glob("mask"+irfilter+"c?.fits")
      wpars = glob("wavelets?.pars")
#      print "Changing to directory",irtarget
#      os.chdir(irtarget)
      links = [".Twisted","sex","apertures.dat"]
      if type(irdbfile) is StringType:
         links = links + [irdbfile,irdbfile.replace(".db","")]
      links = links + darks
      if not flat: links = links + flats + iflats + masks + wpars
      for l in links:
#        if os.path.exists(l): os.remove(l)
#        if os.path.exists("../"+l) and not os.path.exists(os.path.basename(l)):
        if os.path.exists(l) and not os.path.exists(irtarget+"/"+os.path.basename(l)):
            print "Creating symlink to",l
            os.symlink("../"+l,irtarget+"/"+os.path.basename(l))
      return CIRCEDataset

SEXINTKEYS = ["NUMBER","VECTOR_ASSOC"]

def readsex(filename,verb=0):
   fu = open(filename,"r")
   icol = []
   keys = []
   fmts = []
   data = {}
   n=0
   for l in fu.xreadlines():
     if l[0] == "#":
       w = split(l)
       icol.append(int(w[1])-1)
       keys.append(w[2])
       if w[2] in SEXINTKEYS: fmts.append("%6d")
       elif w[2] == "FLAGS": fmts.append("%4d")
       elif w[2] == "CLASS_STAR": fmts.append("%5.2f")
       elif w[2] == "X_IMAGE": fmts.append("%9.3f")
       elif w[2] == "Y_IMAGE": fmts.append("%9.3f")
       else: fmts.append("%9.4f")
       data[w[2]] = []
       n+=1
     else:
       w = split(l)
       [data[keys[j]].append(w[j]) for j in range(n)]
   fu.close()
   if verb:
     print "Found Keys="
     print keys
     print "Number of Objects = %d" % ( len(data[keys[0]]))
   for k in keys:
     data[k] = array([[float,int][k in SEXINTKEYS](d) for d in data[k]])
   return data

class Exposure:
   def __init__(self,N0):
      N = copy.copy(N0)
      self.N = N
      self.target = N.target
      self.fclass = N.fclass
      self.bad = N.bad
      self.objname = N.object
      self.target = N.target
      self.obstype = N.type
      self.sky = N.sky
#      self.fits = N.info["Filename"]
      self.name = N.name.replace(".fits","")
      self.namec = N.name
      self.fits = N.bdir+"/"+N.base
      self.ra = N.ra
      self.dec = N.dec
      self.raoff = N.info["RAOFF"]
      self.decoff = N.info["DECOFF"]
      self.exptime = N.info["EXPTIME"]
      if self.fclass == "newfirm":
         self.noctim = N.info["NOCTIM"]
         self.ncoadd = N.info["NOCCOADD"]
         self.scale = 0.4
      elif self.fclass == "fourstar":
         self.noctim = N.info["EXPTIME"]
         self.ncoadd = 1
         self.gain = N.info["GAIN"]
         self.scale = 0.16
      else:
         self.scale = 0.2
         self.noctim = N.info["EXPTIME"]
         self.ncoadd = 1
      self.filter = N.info["FILTER"]
      self.mjd = N.info["MJD-OBS"]
      self.base = N.base
#      if "image" in N.base:
#         self.expno = int(N.base.replace("image",""))
#      elif "raw" in N.base:
#         self.expno = int(N.base.replace("raw",""))
      self.expno = N.jno
      self.loopno = N.kno
      self.scratchspace = os.environ.get("SCRATCHSPACE",os.getcwd()+"/"+self.target)
      self.cache = True
      self.data = {}
      self.masks = {}
      self.sdata = {}
      self.lift = {}
      self.Bad()

   def __repr__(self):
      return "%s[%d(%d)]" % (self.base,self.exptime,self.ncoadd)

   def scratch(self,f):
      if self.scratchspace:
         s=self.scratchspace+"/"
         easydir(s)
      else:
         s="./"
      d = f.split("/")
      n = len(d)-1
      e = s+"/"
      for i in range(n):
        e = e+d.pop(0)+"/"
        easydir(e)
      r = s+f
      return r

   def mfits1(self,c):
       return self.scratch("mode1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def rfits1(self,c):
       return self.scratch("back1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def wfits1(self,c):
       return self.scratch("mall1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def qfits1(self,c):
       return self.scratch("smow1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def ufits1(self,c):
       return self.scratch("uall1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def bfits1(self,c):
       return self.scratch("mbg1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def sfits1(self,c):
       return self.scratch("sky1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def mfits2(self,c):
       return self.scratch("mode2/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def rfits2(self,c):
       return self.scratch("back2/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def wfits2(self,c):
       return self.scratch("mall2/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def qfits2(self,c):
       return self.scratch("smow2/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def ufits2(self,c):
       return self.scratch("uall2/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def bfits2(self,c):
       return self.scratch("mbg2/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def sfits2(self,c):
       return self.scratch("sky2/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def mask1(self,c):
       return self.scratch("mask1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def pers1(self,c):
       return self.scratch("pers1/"+self.namec.replace(".fits","c%d.fits"%(c)))
   def cat1(self,c):
       return self.scratch("cat1/"+self.namec.replace(".fits","c%d.fits"%(c)).replace(".fits",".cat"))
   def cat2(self,c):
       return self.scratch("cat2/"+self.namec.replace(".fits","c%d.fits"%(c)).replace(".fits",".cat"))
   def shift1(self,c):
       return self.scratch("shift1/"+self.namec.replace(".fits","png"))
   def shift2(self,c):
       return self.scratch("shift2/"+self.namec.replace(".fits","png"))

   def Bad(self):
      if self.bad or len([c for c in [1,2,3,4] if \
                (os.path.exists(self.mfits1(c)+".bad") or \
                os.path.exists(self.sfits1(c)+".bad") or \
                os.path.exists(self.mfits2(c)+".bad") or \
                os.path.exists(self.sfits2(c)+".bad"))]):
                   bad = True
      else: bad = False
      self.bad = bad

   def ReadModes(self,first=True,second=False,data=0):
      chips = [1,2,3,4]
      m = asarray([self.ReadMode(chip,first=first,second=second,data=data) for chip in chips])
      return m
   def ReadMode(self,chip,first=True,second=False,data=0):
      if first:
        mfits = self.mfits1(chip)
      elif second:
        mfits = self.mfits2(chip)
      G = FITS(mfits)
      cent = G["mode"]
      if cent == "N/A": cent = G["median"]
      scat = G["scat"]
      G.close()
      if not data: return cent,scat
      if data:
         G = FITS(self.fits+"crap")
         d = G.data()
         G.close()
         return cent,scat,d
   def Job(self,func,args):
      os.nice(15)
      chips = [1,2,3,4]
      dicts = [{"args":(self.N,chip),"keywords":args} for chip in chips]
      if "verb" in args and args["verb"]:
         [apply(func,dict["args"],dict["keywords"]) for dict in dicts]
      else:
         print func.__name__
         self.queue = TwistedObjectQueue(func.__name__,dicts)
         self.queue.repeatable = False
         self.queue.Run(forks=False,sleeptime=0.02)
         print self.queue.results
   def Modes(self,first=True,second=False,msh=10,cat=None,histo=0,mag=3,flat=0,verb=1):
      args = {"first":first,"second":second,"msh":msh,"cat":cat,"mag":mag,"flat":flat,
        "histo":histo,"verb":verb}
      self.Job(CIRCEMode,args)
   def Blank(self):
      self.data = {}
   def Data(self,chip,first,second,verb=1,force=0,flat=0):
      if first:
        mfits = self.mfits1(chip)
        easydir(self.scratch("mode1"))
        ci = 1
      elif second:
        mfits = self.mfits2(chip)
        easydir(self.scratch("mode2"))
        ci = 2

      ReadCalibs(self,chip,nfilt=self.filter,flat=flat,first=first,second=second)

      if force or not hasattr(self,"data") or not self.data.has_key(chip):
         if os.path.exists(mfits):

            print "Reading",mfits,
            G = FITS(mfits,readonly=1)
            self.data[chip] = G.data()
            self.exptime = G["exptime"]
            self.cent = G["mode"]
            if self.cent == "N/A": self.cent = G["median"]
            self.scat = G["scat"]
            G.close()
            if second and os.path.exists(self.pers1(chip)) and os.path.exists(self.mask1(chip)):
               print self.pers1(chip),
               print self.mask1(chip),
               G1 = FITS(self.pers1(chip),readonly=1)
               G2 = FITS(self.mask1(chip),readonly=1)
               self.masks[chip] = (G1.data(), G2.data())
               G1.close()
               G2.close()
            print
         else:
            if self.fclass == "newfirm":
               F = FITS(self.fits)
               Fn = F.name()
               print "Reading",Fn
               exptime = F["exptime"]
               ncoadd = F["ncoadd"]
               if ncoadd == "N/A": ncoadd = 1
               F.goto(1+chip)
               data = F.data()
               data = data[:2048,:2048]
               data = where(between(data,-1e24,1e24),data,0).astype(float32)
               F.close()
            elif self.fclass == "fourstar":
               F = FITS(self.fits+"c%d.fits" % (chip))
               Fn = F.name()
               print "Reading",Fn
               exptime = F["exptime"]
               data = F.data()
               data = data[:2048,:2048]
               data = where(between(data,-1e24,1e24),data,0).astype(float32)
               F.close()
            self.data[chip] = data

            if DARKD[ci].get(chip,None) != None:
               if (self.noctim, self.ncoadd) in DARKD[ci].get(chip):
                  self.data[chip] = self.data[chip] - exptime*DARKD[ci][chip][self.noctim,self.ncoadd]

            if self.filter != "Dark":
               if self.fclass == "newfirm":
                  self.data[chip] = ncoadd*NEWFIRMLinearize1D(self.data[chip]/ncoadd,chip,exptime/ncoadd)
               elif self.fclass == "fourstar":
                  self.data[chip] = FourStarLinearize1D(self.data[chip],chip,mode=self.gain[0])

            if not flat and MASKD[ci].get(chip,None) != None:
               self.data[chip] = self.data[chip]*MASKD[ci][chip]

            if not flat and FLATD[ci].get(chip,None) != None:
               self.data[chip] = divz(self.data[chip],FLATD[ci][chip])

            print "Done reading",Fn

   def SData(self,chip,first,second,verb=1,force=0):
      if first:
        sfits = self.sfits1(chip)
        easydir(self.scratch("sky1"))
      elif second:
        sfits = self.sfits2(chip)
        easydir(self.scratch("sky2"))

      if self.cache and (force or not hasattr(self,"sdata") or not self.sdata.has_key(chip)):
         if not os.path.exists(sfits):
            raise IOError, "Where is %s (%s)?" % (sfits, os.getcwd())
         if os.path.exists(sfits):
            G = FITS(sfits,readonly=1)
            self.sdata[chip] = G.data()
            G.close()

   def WData(self,chip,first,second,verb=1,force=0):
      if first:
        wfits = self.wfits1(chip)
        ufits = self.ufits1(chip)
      elif second:
        wfits = self.wfits2(chip)
        ufits = self.ufits2(chip)
      if self.cache and (force or not hasattr(self,"lift") or not self.lift.has_key(chip) or self.lift.get(chip) == None):
         if os.path.exists(wfits):
            print "Reading",wfits,ufits
            F = FITS(wfits,readonly=1)
            G = FITS(ufits,readonly=1)
            self.lift[chip] = F.data(),G.data()
            F.close()
            G.close()

   def Test(self,*args,**kw):
      print "Howdy!",os.getpid()
      n = len(args)
      m = kw
      return n,m.keys()

   def Mode(self,chip,other=None,first=True,second=False,msh=10,cat=None,histo=0,norm=0,mag=3,flat=0,verb=1):
      start = time.time()
      print "Calculating location for %r [%d]" % (self,chip)
      ReadCalibs(self,chip,nfilt=self.filter,flat=flat,first=first,second=second)
      if not hasattr(self,"data") or not self.data.has_key(chip):
         self.Data(chip,first,second,flat=flat)
      data = self.data[chip]
      if first: ci = 1
      elif second: ci = 2
      if other:
         other.Data(chip,first,second,flat=flat)
         odata = other.data[chip]
      else:
         odata = None
      maskd = MASKD[ci].get(chip,None)
      if cat:
         mask,pers = DrawMask(chip,cat,self,data.shape,mag=mag,verb=verb)
         sfits1 = self.sfits1(chip)
         mask1 = self.mask1(chip)
         pers1 = self.pers1(chip)
         print "Writing %s" % (mask1)
         G = FITS(mask1,create=1,template=sfits1)
         G.modkey("bitpix",16)
         G.putdata(mask)
         G.close()
         print "Writing %s" % (pers1)
         G = FITS(pers1,create=1,template=sfits1)
         G.modkey("bitpix",16)
         G.putdata(pers)
         G.close()
         mask = logical_not(logical_or(mask,pers))
      else:
         mask = not_equal(data,0)
      if maskd!=None: mask = logical_and(mask,maskd).astype(int16)
      if odata!=None:
         osc1 = median(data.ravel())
         osc2 = median(odata.ravel())
         if osc1 and osc2:
           osc = osc1/osc2
         else:
           osc = 1.0
         odiff = data-osc*odata
         odiff2 = compress(mask.ravel(),odiff.ravel())
         omed2 = median(odiff2)
         odiff2 = odiff2 - omed2
         oscat = 1.49*median(abs(odiff2))
         masko = equal(VTKConvolve(1.*less(abs(odiff-omed2),5*oscat),k=3,numret=1),1.0)
         mask = logical_and(mask,masko).astype(int16)
      kind = "median"
      cent, scat = self.QuickMedian(data.ravel(),mask=mask,verb=verb)
      if not (first and not histo) and msh and cent > 0:
         kind = "mode"
         cent, scat = self.FindMode(data.ravel(),msh=msh,mask=mask,histo=histo,verb=verb)
      finish = time.time()
      ctime = finish-start
      if first:
        mfits = self.mfits1(chip)
      elif second:
        mfits1 = self.mfits1(chip)
        mfits = self.mfits2(chip)
      print "Writing %s[%d,%d] using (%s,scat) = (%.1f,%.1f), computed in %.2fs" % \
        (mfits,data.shape[1],data.shape[0],kind,cent,scat,ctime)
      if self.fclass == "newfirm":
         G = FITS(mfits,create=1,readonly=0)
         F = FITS(self.fits)
         G.copyheader(F)
         G.delkey("extend")
         G.delkey("nextend")
         F.goto(1+chip)
         bad = ["xtension","bitpix","naxis","naxis1","naxis2","pcount","gcount",
         "extname","inherit","extver",]
         [G.modkey(key,F[key]) for key in F.keys() if key not in bad]
         F.close()
      elif self.fclass == "fourstar":
         G = FITS(mfits,create=1,readonly=0,template=self.fits+"c%d.fits"%(chip))
         G.modkey("bitpix",-32)
         G.resize( [data.shape[1], data.shape[0]])
      if self.filter != "Dark":
         G.modkey("darkfr",DARKN[ci][chip].get((self.noctim,self.ncoadd),"None"))
         G.modkey("maskfr",MASKN[ci].get(chip,"None"))
         G.modkey("flatfr",FLATN[ci].get(chip,"None"))
      G.modkey(kind,cent)
      G.modkey("scat",scat)
#      if flat and "flats" not in self.objname:
#         divide(self.data[chip],cent,self.data[chip])
      G.putdata(self.data[chip])
      G.close()
      MakeThumbnail(mfits,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
      MakeThumbnail(mfits,mag=16,pct=40,s1=-10,s2=10,inv=1,q=85)
      finish = time.time()
      ctime = finish-start
      print "Total elapsed time for this frame was %.02fs" % (ctime)
      if not self.cache:
         self.data.pop(chip)
      return cent,scat

   def flwt2d(self,chip,allmed=None,first=True,second=False,verb=1,nn=4,nt=4):
       fsub = lambda c: self.flwt2dchild(c,allmed=allmed,first=first,second=second,verb=verb,nn=nn,nt=nt)
       p = Process(target=fsub,args=(chip,))
       p.start()
       p.join()

   def flwt2dchild(self,chip,allmed=None,first=True,second=False,verb=1,nn=4,nt=4):
      p = (second and 2) or (first and 1)
      o = getattr(self,"mfits%d"%(p))(chip)
      t0 = time.time()
      print "Starting",o
      ReadCalibs(self,chip,nfilt=self.filter,flat=False,first=first,second=second)
      self.Data(chip,first,second)
      data = self.data[chip]
      F = FITS(self.sfits1(chip))
      Fm = F["median"]
      Fr = F["anoise"]
      sdata = F.data()
      if F["skyframe"] != "N/A":
         G = FITS(self.scratch("sky1/"+os.path.basename(F["skyframe"])))
         tdata = Fm*G.data()
         G.close()
      else:
         tdata = None
      F.close()
      if hasattr(self,"masks"):
         persim,maskim = self.masks[chip]
         maskim = (logical_not(logical_or(maskim,persim)) * MASKD[p][chip]).astype(float32)
      else: maskim = MASKD[p][chip]
      maskim = VTKMultiply(maskim,VTKConvolve(VTKIslandRemoval(maskim,0,1,20000,1),k=5,numret=1))
      maskim = VTKMultiply(maskim,VTKConvolve(VTKIslandRemoval(maskim,0,1,20000,1),k=5,numret=1))
      cent,scat = self.ReadMode(chip,first=first,second=second,data=0)
      easydir(self.scratch("mall%d" % (p)))
      easydir(self.scratch("back%d" % (p)))
      easydir(self.scratch("uall%d" % (p)))
      i1,w,u = MakeWaves(data,sdata,sig=5,mask=maskim,tdata=tdata,nn=nn,nt=nt,Fm=Fm,Fr=Fr)
      m = getattr(self,"wfits%d"%(p))(chip)
      mu = getattr(self,"ufits%d"%(p))(chip)
      m2 = getattr(self,"rfits%d"%(p))(chip)
#      self.lift[chip]={
#        "m2":i1, "m":w, "mu":u,
#        }
      for f,a in zip([m2,m,mu],[i1,w,u]):
         print "Writing",f
         G = FITS(f,create=1,template=o)
         G.putdata(a)
         G.close()
         MakeThumbnail(f,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
         MakeThumbnail(f,mag=16,pct=40,s1=-10,s2=10,inv=1,q=85)
      t1 = time.time()
      print "Time for computation was %.2fs" % (t1-t0)
      if chip in self.data: self.data.pop(chip)
      if chip in self.masks: self.masks.pop(chip)

   def QuickMedian(self,s,mask=None,verb=0):
      if mask.any(): s = compress(mask.ravel(),s)
      s = sort(s)
      me = s[len(s)/2]
      sc = (s[int(0.84*len(s))] - s[int(0.16*len(s))])/2.
      return me, sc
   def FindMode(self,s,th=0,sat=65e3,mask=None,msh=10,verb=0,histo=0):
      mode = 0.0
      scat = 0.0
      if mask != None: s = compress(mask.ravel(),s)
      s = compress(greater(abs(s),th),s)
      if sat: s = compress(less(s,sat),s)
      s = sort(s)
      if not msh: msh=10
      if histo:
         me = s[len(s)/2]
         ms = 1.49*median(abs(s-me))
         ml = me-3.0*ms
         mh = me+1.0*ms
         md = ms/msh
         bins = arange(ml,mh,md,float64)
         mi = diff(searchsorted(s,bins))
         mb = diff(bins)/2. + bins[:-1]
         lmi = log(mi)
         bases = polynom(mb,2)
         sol = linear_least_squares(transpose(bases),lmi)[0]
         print sol
         scat = sqrt(-0.5/sol[2])
         mode = pow(scat,2)*sol[1]
         print me,mode,scat
         if verb:
            lmi1 = max(lmi)
            mi1 = max(mi)
            xm=[mb[0],mb[-1]]
            ym=[lmi1-2,lmi1+0.3]
            newbox(xm,ym,ch=2.0)
            xlabel("Counts",ch=2.0)
            ylabel("ln N",ch=2.0)
            drawln(mb,lmi,ls=1,hist=1)
            drawln([mode,mode],ym,ls=1)
            drawln([mode-scat,mode-scat],ym,ls=2)
            drawln([mode+scat,mode+scat],ym,ls=2)
            raw_input("hit <cr> ")
      else:
         x = VTKShrinkN(0.5+arange(len(s)),msh,usemean=1,numret=1)/msh
         s = VTKShrinkN(s,msh,usemean=1,numret=1)
         p = x/(1.*len(s))
         m = s[3*len(s)/8]
         n = ndtri(p)
#         n = (pdtri(m,1-p)-m)/sqrt(m)
         q = (s[int(0.84*len(s)+0.5)]-s[int(0.16*len(s)+0.5)])/2.
         guess = [m,q]
         w = exp(-0.5*(p-0.5)**2/pow(0.25,2))
         def fitp(g):
           y = g[0]+g[1]*n
           r = s-y
           u = r*w
           if verb: print g,len(s)
           return u
         result = leastsq(fitp,guess)
         if verb: print result
         mode,scat = result[0]
      return mode,scat

   def SkySubtracts(self,NDS,first=1,second=0,nf=3,verb=0,flat=0):
      NDS = [N1 for N1 in NDS if not Exposure(N1).bad]
      args = {"NDS":NDS,"first":first,"second":second,"nf":nf,"verb":verb,"flat":flat}
      self.Job(CIRCESkySubtract,args)

   def SkySubtract(self,chip,Exposures,first=1,second=0,nf=3,verb=0,flat=0,yskip=16,sigclip=0,test=0,Hub=None,bigsky=0,NP=-1,already=0):
      start = time.time()
      if first: ci = 1
      elif second: ci = 2
      Exposures = [e for e in Exposures if not e.bad]
      fsky = array([e.sky for e in Exposures])
      bab = not flat and (sum(fsky) and (sum(fsky) < len(fsky)))
      if bab:
        sExposures = [e for e in Exposures if e.sky]
        sdir = self.scratch("").replace("obj","sky")
      else:
        sExposures = Exposures
        sdir = self.scratch("./")
      if first:
         mfits = self.mfits1(chip)
         sfits = self.sfits1(chip)
         nes = [j for j in range(len(Exposures)) if Exposures[j].base == self.base]
         jno = array([E.expno for E in Exposures])
         sjno = array([E.expno for E in sExposures])
         kno = array([E.loopno for E in Exposures])
         skno = array([E.loopno for E in sExposures])
         fsky = array([E.sky for E in Exposures])
         jnos = sort(list(Set(jno)))
         nj = len(jnos)
         kuse = 1
#         if chip == 1: print nes
         if nes:
            ne = nes[0]
         else:
            ne = len(Exposures)-1
         if flat and "dark" in Exposures[0].objname:
            ne30 = len(Exposures)
            skyframe = sdir + "sky1/Darkc%d.fits" % (chip)
         elif flat and "flat" in Exposures[0].objname:
            ne30 = len(Exposures)
            skyframe = sdir + "sky1/Flat"+Exposures[0].filter+"c%d.fits" % (chip)
         else:
            ne = argmin(abs(jno[ne] - sjno)+abs(kno[ne]-skno))
            jk = {}
            for j1 in jno: jk.setdefault(j1,0)
            [jk.__setitem__(j1,jk.get(j1)+1) for j1,k1 in zip(jno,kno)]
            kuse = max((jk.values()))
#            print kuse
#            os._exit(1)
#            kuse = median(kno)
#            kuse = max([k1 for j1,k1 in zip(jno,kno) if j1 == jno[ne]])
            nf = nf * kuse
            nf = min([nf,len(sjno)])
            nf = len(sjno)/(len(sjno)/nf)
            ne30 = (ne / nf * nf) + 1
            tskyframe = sdir+"sky1/.FirstSky%04dc%d.fits" % (ne30,chip)
            skyframe = sdir+"sky1/FirstSky%04dc%d.fits" % (ne30,chip)
#         if chip == 1: print ne,ne30
         if (ne+1) == ne30: await = False
         else: await = True
         if await and not os.path.exists(skyframe): print "Need to await",skyframe
      elif second:
         await = False
         mfits = self.mfits2(chip)
         sfits = self.sfits2(chip)
         pfits = self.pers1(chip)
         if flat:
            ne30 = len(Exposures)
            if "dark" in Exposures[0].objname:
              skyframe = sdir+"sky2/Darkc%d.fits" % (chip)
            else:
              skyframe = sdir+"sky2/Flat"+Exposures[0].filter+"c%d.fits" % (chip)
         else:
            skyframe = sdir + "sky2/SecondSky%04dc%d.fits" % (self.expno,chip)
      if flat and second: await = False
      if test: return await,skyframe,ne,ne30
      if bigsky and FileCheck(skyframe): return []
      ReadCalibs(self,chip,nfilt=self.filter,flat=flat,first=first,second=second)

      if skyframe in SKYFD[ci] or FileCheck(skyframe):
         use = None
      else:
        if first and await and not (first and flat):
          print "Looking for",skyframe
          self.WaitSkySubtract(skyframe,chip,first=first,second=second,already=already)
          use = None
        elif not await and nf and not first and not (second and flat):
          sfits1 = self.sfits1(chip)
          ne = [j for j in range(len(Exposures)) if Exposures[j].base == self.base][0]
          F = FITS(sfits1)
          xpos1 = F["xdiff1"]
          ypos1 = F["ydiff1"]
          F.close()
          sfitsE = lambda E: E.sfits1(chip)
          F = [FITS(sfitsE(E)) for E in Exposures]
          xdiff1 = array([f["xdiff1"] for f in F])
          ydiff1 = array([f["ydiff1"] for f in F])
          [f.close() for f in F]
          ds = sqrt((xpos1-xdiff1)**2 + (ypos1-ydiff1)**2)
          off = nonzero(greater(ds,2.5))
          neo = clip(searchsorted(off,ne),0,len(off)-1)
          off = off[max([0,neo-nf]):min([neo+nf,len(off)])]
          use = [Exposures[o] for o in off]
        elif flat:
          use = Exposures
        else:
          use = [E for E in sExposures[ne30-1:ne30-1+nf] if (not bab or E.sky)]
          if len(use) < 3*kuse and (ne30-1)>=3*kuse:
             use = sExposures[max([0,(ne30-1)-3]):min([len(sExposures),ne30-1+nf])]

      if use and not FileCheck(skyframe):
         print "Computing",skyframe
         print "At",self
         print "From",use
         M = self.MedianFrames(skyframe,chip,use,first,second,yskip=yskip,verb=verb,flat=flat,sigclip=sigclip,Hub=Hub,NP=NP)
         if Hub != None: return M
         finish = time.time()
         ctime = finish-start
         print "Writing %s, computed in %.2fs" % (skyframe,ctime)
      if not (first and flat and ("Dark" in skyframe or self.objname[:8] == "flat_obj")) and FileCheck(skyframe):
         self.DoSkySubtract(skyframe,chip,first=first,second=second)

   def WaitSkySubtract(self,skyframe,chip,first=0,second=0,already=0):
       print "Waiting for",skyframe
       if already and not os.path.exists(skyframe):
          raise IOError, "%s sky frame missing!" % (skyframe)
       while not os.path.exists(skyframe):
          time.sleep(0.2)
       ssize = 0
       while ssize != os.path.getsize(skyframe):
          ssize = os.path.getsize(skyframe)
          time.sleep(0.2)
          print "Waiting for completion of",skyframe
       print "It (%s) seems to be there now!" % (skyframe)

   def DoSkySubtract(self,skyframe,chip,first=0,second=0):
      ReadCalibs(self,chip,nfilt=self.filter,flat=0,first=first,second=second)
      if first:
         mfits = self.mfits1(chip)
         sfits = self.sfits1(chip)
         ci = 1
      elif second:
         mfits = self.mfits2(chip)
         sfits = self.sfits2(chip)
         ci = 2
      print "Subtracting from",mfits
      print "To make",sfits
      print "Using",skyframe
      if chip not in SKYFD[ci]: SKYFD[ci][chip] = {}
      if skyframe in SKYFD[ci][chip]:
         S = SKYFD[ci][chip][skyframe]
      else:
         FS = FITS(skyframe)
         S = FS.data()
         SKYFD[ci][chip][skyframe] = S
         FS.close()
      mval = []
      for chip2 in [1,2,3,4]:
         cent,scat = self.ReadMode(chip,first=first,second=second)
         mval.append(cent)
      cent = median(mval)
      self.Data(chip,first,second)
      D = self.data[chip]
      G = less(D,1e24)*not_equal(S,0)
      D = (D-cent*S)*G
      if chip in MASKD[ci] and MASKD[ci][chip] != None:
        D = D * MASKD[ci][chip]
      if second and chip in self.masks:
         print "Mask!"
         D = where(self.masks[chip][0],0,D)
      alen = len(compress(D.ravel(),D.ravel()))
      adiff = median(compress(D.ravel(),D.ravel()))
      anoise = 1.48*median(abs(compress(D.ravel(),D.ravel())))
      Dax = 1.49*median(abs(D-adiff))
      Dax1,Dax2 = bwt(Dax)
      Day = 1.49*median(abs(transpose(D)-adiff))
      Day1,Day2 = bwt(Day)
#      D = D*less(Dax-Dax1,10*Dax2)[NewAxis,::]*less(Day-Day1,10*Day2)[::NewAxis]
      print "N = %d Median = %.8f ,1.48MAD = %.8f" % (alen,adiff,anoise)
      print Dax1,Dax2,Day1,Day2
#      if FileCheck(sfits):
#        ns = len(glob(sfits+"*"))
#        sfits = sfits+str(ns)
      G = FITS(sfits,create=1,template=mfits)
      G.modkey("adiff",adiff)
      G.modkey("anoise",anoise)
      G.modkey("skyframe",os.path.basename(skyframe))
      G.putdata(D)
      G.close()
      if first: cat = self.cat1(chip)
      elif second: cat = self.cat2(chip)
      MakeThumbnail(sfits,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
      MakeThumbnail(sfits,mag=16,pct=40,s1=-10,s2=10,inv=1,q=85)
      Sextractor(sfits,cat,scale=self.scale,thresh=5,saturate=50e3)
      if not self.cache and chip in self.data: self.data.pop(chip)
      if not self.cache and chip in self.masks: self.masks.pop(chip)
      print "Writing %s" % (sfits)

   def MedianFrames(self,skyframe,chip,Frames,first,second,yskip=32,verb=0,norm=0,flat=0,sigclip=5,Hub=None,NP=-1):
      [E.Data(chip,first,second) for E in [self]+Frames if E.cache]
      ReadCalibs(self,chip,nfilt=self.filter,flat=flat,first=first,second=second)
      if NP == -1: NP =  int(os.environ.get("NP",0))
      elif NP == -2: NP =  int(os.environ.get("NP",0))/2
      mfits = lambda f: (first and f.mfits1) or (second and f.mfits2)
      F0 = FITS(mfits(Frames[0])(chip))
      naxis1,naxis2 = F0.naxes()
      F0.close()
      M = zeros((naxis2,naxis1),float32)
      F = {}
      K,L = {},{}
      Fmodes = []
      Fno = []
      for f in Frames:
        if f.cache and ((not getattr(f,"data") or not f.data.has_key(chip))):
            f.Data(chip,first,second,verb=1)
        mval = []
        for chip2 in [1,2,3,4]:
           F[f] = FITS(mfits(f)(chip))
           if F[f]["mode"] == "N/A":
             mval.append(F[f]["median"])
           else:
             mval.append(F[f]["mode"])
           F[f].close()
        Fmodes.append(median(mval))
        Fno.append(f.expno)
      Fmodes = asarray(Fmodes)
      Fno = asarray(Fno)
#      [F[f].close() for f in F]
      Fnos = sort(list(Set(Fno)))
      for f in Frames:
          F[f] = mfits(f)(chip)
          if second:
               K[f],L[f]  = f.mask1(chip), f.pers1(chip)
      Ny = (M.shape[0]-1)/yskip+1
      sumF = median(Fmodes)
      if first: mfits = self.mfits1(chip)
      elif second: mfits = self.mfits2(chip)
      tempframe = os.path.dirname(skyframe)+"/."+os.path.basename(skyframe)
      map(os.remove,glob(os.path.dirname(tempframe)+"/"+os.path.basename(tempframe).replace(".fits","_????.fits")))
      G = FITS(tempframe,create=1,template=mfits)
      G.modkey("ncombine",len(Frames))
      for j,E in enumerate(Frames):
         G.modkey("im%03d"%(j+1),E.name)
      G.close()
      Sections = [j*yskip for j in range(Ny)]
      if Hub != None:
          testsize = len(cPickle.dumps(self))
          print "len(pickle(self))=",testsize
          doj = [TwistedTask(-1, \
                 "lambda *args,**kw: args[0].MedianSection(*args[1:],**kw)",\
                 self,tempframe,skyframe,j0,yskip,chip,Frames,F,K,L,Fmodes,M.shape,\
                 first,second,verb=verb,flat=flat,sigclip=sigclip) \
                 for j0 in Sections]
          [item.setFromModules(["CIRCEProcess"]) for item in doj]
          [E.data.pop(chip) for E in [self]+Frames if E.cache and chip in E.data]
          [E.masks.pop(chip) for E in [self]+Frames if E.cache and chip in E.masks]
          return doj,tempframe,skyframe
#          mhost,mport = Hub.split(":")
#          mport = int(mport)
#          print "hub host is %s, port %d" % (mhost,mport)
#          print "go!"
#          H = HubDispatch(doj,(mhost,mport),mport=0,namespace=globals(),verb=1,sleeptime=0.01)
      elif NP > 1:
         doj = lambda j0: self.MedianSection(tempframe,skyframe,j0,yskip,chip,Frames,F,K,L,Fmodes,M.shape,\
                first,second,verb=verb,flat=flat,sigclip=sigclip)
         RunForkedQueue(doj,Sections,MaxThreads=NP,sleeptime=0.001,monitor=0,ThreadOutput=True)
      else:
         doj = lambda j0: self.MedianSection(tempframe,skyframe,j0,yskip,chip,Frames,F,K,L,Fmodes,M.shape,\
                first,second,verb=verb,flat=flat,sigclip=sigclip)
         map(doj,Sections)
      [E.data.pop(chip) for E in [self]+Frames if E.cache and chip in E.data]
      [E.masks.pop(chip) for E in [self]+Frames if E.cache and chip in E.masks]
      os.rename(tempframe,skyframe)
      MakeThumbnail(skyframe,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
      MakeThumbnail(skyframe,mag=16,pct=40,s1=-10,s2=10,inv=1,q=85)
      if verb: print

   def MedianSection(self,tempframe,skyframe,j0,yskip,chip,Frames,F,K,L,Fmodes,Mshape,\
           first,second,verb=0,flat=0,sigclip=5):
       j1 = min([j0+yskip,Mshape[1]])
       Ma = zeros((len(Frames),j1-j0,Mshape[1]),float32)
       print j0,j1
       sumF = median(Fmodes)
       if second:
         Ma2 = zeros((len(Frames),j1-j0,Mshape[1]),float32)
         mfits = (first and self.mfits1) or (second and self.mfits2)
         if self.cache:
           Ma3 = logical_not(self.masks[chip][0][j0:j1])
         else:
           Ff = FITS(mfits(chip))
           Ma3 = logical_not(Ff.readsec([0,j0],[Mshape[1]-1,j1-1]))
           Ff.close()
       for i,f in enumerate(Frames):
           if f.cache and hasattr(f,"data") and f.data.has_key(chip):
              Ma[i] = f.data[chip][j0:j1]
              if second:
                 Ma2[i] = logical_or(f.masks[chip][0][j0:j1],f.masks[chip][1][j0:j1])
           else:
              Ff = FITS(F[f])
              Ma[i] = Ff.readsec([0,j0],[Mshape[1]-1,j1-1])
              Ff.close()
              if second:
                 Kf = FITS(K[f])
                 Lf = FITS(L[f])
                 Ma2[i] = logical_or(Kf.readsec([0,j0],[Mshape[1]-1,j1-1]),\
                                Lf.readsec([0,j0],[Mshape[1]-1,j1-1]))
                 Kf.close()
                 Lf.close()
       if "Dark" not in skyframe:
           for i,f in enumerate(Frames):
               Ma[i] = Ma[i]/Fmodes[i]
#       Ma = asarray([((hasattr(f,"data") and getattr(f,"data")[j0:j1]) or  \
#               f.readsec([0,j0],[Mshape[1]-1,j1-1])) for f in F])
       if 0 and flat and (f.objname[:8] != "flat_obj" or "Dark" in f.filter):
          Mj = divz(sum(Ma),sum(not_equal(Ma,0)))
       elif first or second or flat:
#          for i in range(len(Ma)):
#             qdump("Ma%03d.fits" % (i),Ma[i])
          if second:
             Ma = Ma * logical_not(Ma2)

          Mg = not_equal(Ma,0.0)
          Ma = where(Mg,Ma,1e18)
          Ma = sort(Ma,0)
          Mo = add.reduce(less(Ma,1e16),0).astype(int32)

          aF = arange(0,Ma.shape[0],1,int32)[::,NewAxis,NewAxis]

          Mo2i = (Mo/2.).astype(int32)
          Moi = ((Mo+1)/2.).astype(int32)
          Meven = equal(Mo2i,Moi)
          Modd = not_equal(Mo2i,Moi)

          Rodd = add.reduce(Ma * equal(aF,Mo2i),0)
          Reven = add.reduce(Ma * (equal(aF,Mo2i-1)+equal(aF,Mo2i)),0)/2.0
          mm = where(Modd,Rodd,Reven)*greater(Mo,0)

          sm = abs(Ma-mm[NewAxis,::,::])
          sms = sort(where(Mg,sm,1e18),0)
          Rodd = add.reduce(sms * equal(aF,Mo2i),0)
          Reven = add.reduce(sms * (equal(aF,Mo2i-1)+equal(aF,Mo2i)),0)/2.0
          rm = 1.49*where(Modd,Rodd,Reven)*greater(Mo,0)
#          if rm.any():
#             rm = median(compress(rm.ravel(),rm.ravel()))
#          else:
#             rm = zeros(mm.shape)

          if sigclip and "Dark" not in skyframe:

             sm = Ma-mm[NewAxis,::,::]
             for mit in range(3):
                um = not_equal(Ma,0.0)*less(Ma,1e18)
                if mit == 2: wm = um*Cauchy(divz(sm,rm),w=1)
                else: wm = um*Huber(divz(sm,rm),w=1)
                mm = divz(sum(Ma*wm),sum(wm))
                sm = abs(Ma-mm[NewAxis,::,::])
                sms = sort(where(Mg,sm,1e18),0)
                Rodd = add.reduce(sms * equal(aF,Mo2i),0)
                Reven = add.reduce(sms * (equal(aF,Mo2i-1)+equal(aF,Mo2i)),0)/2.0
                rm = 1.49*where(Modd,Rodd,Reven)*greater(Mo,0)

#             Mg = not_equal(Ma,0.0)*less_equal(sm,sigclip*rm)
#             mm = divz(add.reduce(Mg*Ma,0), add.reduce(Mg,0))

          Mj = mm
       elif second:
          if second:
             Mk = lambda t: 1-logical_or(t[0][j0:j1],t[1][j0:j1])
             Mb = asarray([Mk(f.masks[chip]) for f in Frames])
             Ma = Mb*Ma
          Mg = not_equal(Ma,0.0).astype(float32)
          Mj = VTKDivide(sum(Ma*Mg),sum(Mg))
          Mjo = logical_not(self.masks[chip][0][j0:j1])
          Mj = VTKMultiply(Mj,Mjo)
#       if (second and flat and f.objname[:8]=="flat_obj") or (flat:
       if "Flat" in os.path.basename(skyframe):
          Mj = Mj * sumF
#       M[j0:j1] = Mj.astype(float32)
#       qdump("Mj.fits",Mj)
#       exit()

       tname1 = os.path.dirname(tempframe)+"/"+os.path.basename(tempframe).replace(".fits","_%04d.fits"%(j0))
       qdump(tname1,Mj)
#       G = FITS(tempframe,readonly=0)
#       G.writesec(Mj,[0,j0],[Mshape[1]-1,j1-1])
#       G.close()

       if verb: sys.stdout.write("."); sys.stdout.flush()

   def Shift(self,Exposures,first=1,second=0,verb=0,mport=-1,chips=[1,2,3,4]):
      NP =  int(os.environ.get("NP",0))
      Exposures = [e for e in Exposures if not e.bad]
      fsky = array([e.sky for e in Exposures])
      if sum(fsky) < len(fsky):
         Exposures = [e for e in Exposures if not e.sky]
      ne = [j for j in range(len(Exposures)) if Exposures[j].name == self.name]
      if not len(ne):
         raise NameError, "%r seems to not be in the list?" % (self)
      ne = ne[0]
      if first: stage = 1
      elif second: stage = 2

      F = FITS(getattr(self,"sfits%d"%(stage))(chips[0]))
      exptime = F["exptime"]
      F.close()
      if ne:
         ref = Exposures[ne-1]
         Fref = FITS(getattr(ref,"sfits%d"%(stage))(chips[0]))
         refexptime = Fref["exptime"]
         Fref.close()
         refim = Exposures[ne-1].name
         yoff1 = self.raoff
         xoff1 = -self.decoff
         yoff2 = ref.raoff
         xoff2 = -ref.decoff
# THIS IS THE OPPOSITE SIGN OF WHAT IS EXPECTED
         xo = -(xoff2-xoff1)
         yo = -(yoff2-yoff1)
         xo0 = xo
         yo0 = yo
         ux1,uy1,uf1,um1,ut1,ura1,udec1 = [[] for j in range(7)]
         ux2,uy2,uf2,um2,ut2,ura2,udec2 = [[] for j in range(7)]
         ax1,ay1,af1,am1,at1,ara1,adec1 = [[] for j in range(7)]
         ax2,ay2,af2,am2,at2,ara2,adec2 = [[] for j in range(7)]
         sexes = []
         cats = []
         for chip in chips:
            if first:
               cat = self.cat1(chip)
               sfits = self.sfits1(chip)
            elif second:
               cat = self.cat2(chip)
               sfits = self.sfits2(chip)
            if os.path.exists(cat):
               f = open(cat,"r")
               c = f.read()
               f.close()
               if c: runsex = False
               else: runsex = True
            else: runsex = True
            if runsex:
               if len(chips) > 1 and mport>-1:
                  ttask = TwistedTask(-1,"CIRCEProcess.Sextractor",sfits,cat,scale=self.scale,thresh=5,saturate=50e3)
                  ttask.setModules(["CIRCEProcess"])
                  ttask.setPath(os.getcwd())
                  sexes.append(ttask)
                  cats.append(cat)
               elif len(chips) > 1 and NP:
                  ttask = lambda sc: Sextractor(sc[0],sc[1],scale=self.scale,thresh=5,saturate=50e3)
                  sexes.append([sfits,cat])
                  cats.append(cat)
               else:
                  Sextractor(sfits,cat,scale=self.scale,thresh=5,saturate=50e3)
         if sexes:
            print sexes
            if mport > -1:
                Hub = HubDispatch( sexes, hub=("localhost",9000+mport), mport=0,sleeptime=0.1,verb=1)
#                Hub.tasks.extend(sexes)
                print "Hub done."
                while [cat for cat in cats if not os.path.exists(cat)]:
                  print "waiting for cats"
#                  reactor.iterate(1)
                  print [cat for cat in cats if not os.path.exists(cat)]
                  time.sleep(1)
            elif NP:
               RunForkedQueue(ttask,sexes,MaxThreads=NP,monitor=0,sleeptime=0.01)
         for chip in chips:
            if first:
               cat = self.cat1(chip)
               sfits = self.sfits1(chip)
            elif second:
               cat = self.cat2(chip)
               sfits = self.sfits2(chip)
            if first:
              rcat = ref.cat1(chip)
              rfits = ref.sfits1(chip)
            elif second:
              rcat = ref.cat2(chip)
              rfits = ref.sfits2(chip)
            sex1 = readsex(cat)
            sex2 = readsex(rcat)
            x1,y1 = sex1["X_IMAGE"],sex1["Y_IMAGE"]
            x2,y2 = sex2["X_IMAGE"],sex2["Y_IMAGE"]
            w1 = sex1["FWHM_IMAGE"]
            w2 = sex2["FWHM_IMAGE"]
            a1 = sex1["A_IMAGE"]
            a2 = sex2["A_IMAGE"]
            b1 = sex1["B_IMAGE"]
            b2 = sex2["B_IMAGE"]
            f1 = sex1["FLAGS"]
            f2 = sex2["FLAGS"]
            m1 = sex1["MAG_APER"]
            m2 = sex2["MAG_APER"]
            if "MAG_AUTO" in sex1 and "MAG_AUTO" in sex2:
               t1 = sex1["MAG_AUTO"]
               t2 = sex2["MAG_AUTO"]
            else:
               t1 = sex1["MAG_BEST"]
               t2 = sex2["MAG_BEST"]
            d1 = t1 - m1
            d2 = t2 - m2
            thisto0 = time.time()

            ix1 = (x1/50).astype(int32)
            iy1 = (y1/50).astype(int32)
            ix2 = (x2/50).astype(int32)
            iy2 = (y2/50).astype(int32)
            sx1,sy1 = {},{}
            sx2,sy2 = {},{}
            [sx1.__setitem__(i1,sx1.get(i1,0)+1) for j,i1 in enumerate(ix1)]
            [sy1.__setitem__(i1,sy1.get(i1,0)+1) for j,i1 in enumerate(iy1)]
            [sx2.__setitem__(i2,sx2.get(i2,0)+1) for j,i2 in enumerate(ix2)]
            [sy2.__setitem__(i2,sy2.get(i2,0)+1) for j,i2 in enumerate(iy2)]
            nx1 = array([sx1[i1] for i1 in sx1])
            ny1 = array([sy1[i1] for i1 in sy1])
            nx2 = array([sx2[i2] for i2 in sx2])
            ny2 = array([sy2[i2] for i2 in sy2])
            mx1 = median(compress(nx1,nx1))
            my1 = median(compress(ny1,ny1))
            mx2 = median(compress(nx2,nx2))
            my2 = median(compress(ny2,ny2))
            gx1 = array([sx1[i1] for i1 in ix1])
            gy1 = array([sy1[i1] for i1 in iy1])
            gx2 = array([sx2[i2] for i2 in ix2])
            gy2 = array([sy2[i2] for i2 in iy2])
            histo1 = less(gx1,2*mx1)*less(gy1,2*my1)
            histo2 = less(gx2,2*mx2)*less(gy2,2*my2)
            x1,y1,f1,m1,t1,w1,d1,a1,b1 = compress(histo1,[x1,y1,f1,m1,t1,w1,d1,a1,b1],1)
            x2,y2,f2,m2,t2,w2,d2,a2,b2 = compress(histo2,[x2,y2,f2,m2,t2,w2,d2,a2,b2],1)
            thisto1 = time.time()
#            print "Histogram kill in %s seconds." % (thisto1-thisto0)

            inside1 = between(x1,100,2048-100)*between(y1,100,2048-100)
            inside2 = between(x2,100,2048-100)*between(y2,100,2048-100)
            inside1 = inside1*less(f1,16)*between(w1,2,30)*less(d1,0.25)
            inside2 = inside2*less(f2,16)*between(w2,2,30)*less(d2,0.25)
            inside1 = inside1*greater(a1,0.7)*greater(b1,0.7)
            inside2 = inside2*greater(a2,0.7)*greater(b2,0.7)
            x1,y1,f1,m1,t1 = compress(inside1, [x1,y1,f1,m1,t1], 1)
            x2,y2,f2,m2,t2 = compress(inside2, [x2,y2,f2,m2,t2], 1)
            if self.fclass == "newfirm":
               if self.mjd < 54480:
                   soln = join([os.environ["PYTHONDATA"],"photometry","image3526c%d.soln" % (chip)],"/")
               else:
                   soln = join([os.environ["PYTHONDATA"],"photometry","image52627c%d.soln" % (chip)],"/")
            elif self.fclass == "fourstar":
                soln = join([os.environ["PYTHONDATA"],"photometry","fourstar201101c%d.soln" % (chip)],"/")
            ra1,dec1 = xy2rd(soln,x1,y1,text=0)
            ra2,dec2 = xy2rd(soln,x2,y2,text=0)
            use1 = argsort(m1)[:10]
            use2 = argsort(m2)[:10]
            vx1,vy1,vf1,vm1,vt1,vra1,vdec1 = take([x1,y1,f1,m1,t1,ra1,dec1],use1,1)
            vx2,vy2,vf2,vm2,vt2,vra2,vdec2 = take([x2,y2,f2,m2,t2,ra2,dec2],use2,1)
            ux1,uy1,uf1,um1,ut1,ura1,udec1 = [concatenate([a,b]) for a,b in \
                    zip([ux1,uy1,uf1,um1,ut1,ura1,udec1], [vx1,vy1,vf1,vm1,vt1,vra1,vdec1])]
            ux2,uy2,uf2,um2,ut2,ura2,udec2 = [concatenate([a,b]) for a,b in \
                    zip([ux2,uy2,uf2,ut2,um2,ura2,udec2], [vx2,vy2,vf2,vm2,vt2,vra2,vdec2])]
            ax1,ay1,af1,am1,at1,ara1,adec1 = [concatenate([a,b]) for a,b in \
                    zip([ax1,ay1,af1,am1,at1,ara1,adec1], [x1,y1,f1,m1,t1,ra1,dec1])]
            ax2,ay2,af2,am2,at2,ara2,adec2 = [concatenate([a,b]) for a,b in \
                    zip([ax2,ay2,af2,am2,at2,ara2,adec2], [x2,y2,f2,m2,t2,ra2,dec2])]
         cra = mean(extrema(ura1))
         cdec = mean(extrema(udec1))
         ux1,uy1 = normalcoord(ura1,udec1,cra,cdec,conv=3600*180/pi)
         ux2,uy2 = normalcoord(ura2,udec2,cra,cdec,conv=3600*180/pi)
         x1,y1 = normalcoord(ara1,adec1,cra,cdec,conv=3600*180/pi)
         x2,y2 = normalcoord(ara2,adec2,cra,cdec,conv=3600*180/pi)
         t1 = at1
         t2 = at2
         th = 2.0
         dx1 = (ux1[::,NewAxis] - ux1[NewAxis,::])
         dy1 = (uy1[::,NewAxis] - uy1[NewAxis,::])
         dx2 = (ux2[::,NewAxis] - ux2[NewAxis,::])
         dy2 = (uy2[::,NewAxis] - uy2[NewAxis,::])
         du = (ux1[::,NewAxis] - ux2[NewAxis,::])
         dv = (uy1[::,NewAxis] - uy2[NewAxis,::])
         dr = sqrt(du**2+dv**2)
         ds1 = sqrt(dx1**2+dy1**2)
         ds2 = sqrt(dx2**2+dy2**2)
         dt1 = arctan2(dy1,dx1)*180/pi
         dt2 = arctan2(dy2,dx2)*180/pi
         ds = ds1[::,::,NewAxis,NewAxis] - ds2[NewAxis,NewAxis,::,::]
         dt = dt1[::,::,NewAxis,NewAxis] - dt2[NewAxis,NewAxis,::,::]
         th2d = clip(0.01*th*ds2[NewAxis,NewAxis,::,::],0.5,5)
#         th2d = th
         uds1 = less(abs(ds),th2d)
         uds2 = less(abs(dt),0.5)
         uds = uds1*uds2
         udr = less(dr,250)[::,NewAxis,::,NewAxis]
         uds = uds*udr
#         dx = dx1[::,::,NewAxis,NewAxis] - dx2[NewAxis,NewAxis,::,::]
#         dy = dy1[::,::,NewAxis,NewAxis] - dy2[NewAxis,NewAxis,::,::]
#         ds = sqrt(dx**2+dy**2)
#         ds = between(abs(dx),0,th) * between(abs(dy),0,th)
         ns = add.reduce(add.reduce(uds,3),1)
#         for ns1 in ns: print ns1
#         print
#         kx = abs((ux1+xo)[::,NewAxis,] - ux2[NewAxis,::])
#         ky = abs((uy1+yo)[::,NewAxis,] - uy2[NewAxis,::])
#         ks = less(kx,60)*less(ky,60)
#         ns = ns*ks
#         for ns1 in ns: print ns1
         mns = ns.shape[0]/4
         mns = 0.75*max(compress(ns.ravel(),ns.ravel()))
         jm,km = transpose([(j,argmax(ns1)) for j,ns1 in enumerate(ns) if max(ns1) >= mns])
#         print take(ux2,km)-take(ux1+xo,jm)
#         print take(uy2,km)-take(uy1+yo,jm)
#         print ns.shape
         dx = take(ux2,km)-take(ux1,jm)
         dy = take(uy2,km)-take(uy1,jm)
         dt = take(ut2,km)-take(ut1,jm)
         xb = bwt(dx)
         yb = bwt(dy)
         tb = bwt(dt)
         xo,xr = xb
         yo,yr = yb
         to,tr = tb
         lmat = len(dx)
         print "%3d %4.1f %3d %10.4f+-%.4f %10.4f+-%.4f %10.4f+-%.4f %s %s" % (0,th,lmat,xo,xr,yo,yr,to,tr,ref.base,self.base)
         th = 2.0
         if NP > 1: PGPLOT_DEV = self.scratch("shift%d/%s.png" % (stage,self.name))+"/png"
         else: PGPLOT_DEV = os.getenv("PGPLOT_DEV")
         if PGPLOT_DEV and PGPLOT_DEV[:2] != "/x":
            so = open(self.scratch("shift%d/%s.out" % (stage,self.name)),"w")
            print >>so, "%3d %4.1f %3d %10.4f+-%.4f %10.4f+-%.4f %10.4f+-%.4f %s %s" % (0,th,lmat,xo,xr,yo,yr,to,tr,ref.base,self.base)
         for niter in range(5):
            dx = (x2[::,NewAxis]-(x1+xo)[NewAxis,::])
            dy = (y2[::,NewAxis]-(y1+yo)[NewAxis,::])
            ds = sqrt(dx**2 + dy**2)
            okay = less(ds,th)
            j1 = argmin(ds,1)
            m2 = [j2 for j2 in range(len(x2)) if okay[j2,j1[j2]]]
            m1 = take(j1,m2)
            x1m = take(x1,m1)
            y1m = take(y1,m1)
            t1m = take(t1,m1)
            x2m = take(x2,m2)
            y2m = take(y2,m2)
            t2m = take(t2,m2)
            xb = bwt(x2m-x1m)
            yb = bwt(y2m-y1m)
            tb = bwt(t2m-t1m)+2.5*log10(refexptime)-2.5*log10(exptime)
            xo,xr= xb
            yo,yr= yb
            to,tr= tb
            lmat = len(x2m)
            print "%3d %4.1f %3d %10.4f+-%.4f %10.4f+-%.4f %10.4f+-%.4f %s %s" % (niter+1,th,lmat,xo,xr,yo,yr,to,tr,ref.base,self.base)
#            th = max([15./(1+niter),4*sqrt(xb[1]**2+yb[1]**2)])
            th = max([2.0,5*sqrt(xb[1]**2+yb[1]**2)])
            if verb:
               if (PGPLOT_DEV and PGPLOT_DEV[:2] != "/x" and niter==4) \
                       or (PGPLOT_DEV and PGPLOT_DEV[:2] == "/x"):
                   if (PGPLOT_DEV and PGPLOT_DEV[:2] != "/x" and niter==4) \
                       or (PGPLOT_DEV and PGPLOT_DEV[:2] == "/x" and niter==0):
                      opendev(PGPLOT_DEV,nx=1,ny=1,size=8.5,land=0,asp=1,ask=0)
                      cpgscr(1,0,0,0)
                      cpgscr(0,1,1,1)
                   newbox(extrema(x1),extrema(y1),ch=1.5)
                   xlabel("X",ch=1.5)
                   ylabel("X",ch=1.5)
                   title = "%s-%s [%d]" % (ref.base,self.base,chip)
                   subtitle(title,ch=1.5)
                   drawpt(x2,y2,ps=17,ch=3,ci=2)
                   drawpt(x1+xo,y1+yo,ps=4,ch=2,ci=4)
                   drawpt(x1,y1,ps=4,ch=1.5,ci=7)
                   drawpt(x1+xo0,y1+yo0,ps=17,ch=1,ci=5)
               if PGPLOT_DEV and PGPLOT_DEV[:2] != "/x":
                  print >>so, "%3d %4.1f %3d %10.4f+-%.4f %10.4f+-%.4f %10.4f+-%.4f %s %s" % (niter+1,th,lmat,xo,xr,yo,yr,to,tr,ref.base,self.base)
         if verb:
#            PGPLOT_DEV = os.getenv("PGPLOT_DEV")
            if PGPLOT_DEV and PGPLOT_DEV[:2] == "/x": raw_input("hit <cr>")
            if NP > 1 or PGPLOT_DEV[:2] == "/x": closedev()
            if PGPLOT_DEV and PGPLOT_DEV[:2] != "/x": so.close()
      else:
         sexes = []
         cats = []
         for chip in chips:
            if first:
               cat = self.cat1(chip)
               sfits = self.sfits1(chip)
            elif second:
               cat = self.cat2(chip)
               sfits = self.sfits2(chip)
            if os.path.exists(cat):
               f = open(cat,"r")
               c = f.read()
               f.close()
               if c: runsex = False
               else: runsex = True
            else: runsex = True
            if runsex:
               if len(chips) > 1 and mport>-1:
                  ttask = TwistedTask(-1,"CIRCEProcess.Sextractor",sfits,cat,scale=self.scale,thresh=5,saturate=50e3)
                  ttask.setModules(["CIRCEProcess"])
                  ttask.setPath(os.getcwd())
                  sexes.append(ttask)
                  cats.append(cat)
               elif len(chips) > 1 and NP:
                  ttask = lambda sc: Sextractor(sc[0],sc[1],scale=self.scale,thresh=5,saturate=50e3)
                  sexes.append([sfits,cat])
                  cats.append(cat)
               else:
                  Sextractor(sfits,cat,scale=self.scale,thresh=5,saturate=50e3)
         if sexes:
            print sexes
            if mport > -1:
               Hub = HubDispatch( sexes, hub=("localhost",9000+mport), mport=0,sleeptime=0.1,verb=1)
#               Hub.tasks.extend(sexes)
               print "Hub done."
               while [cat for cat in cats if not os.path.exists(cat)]:
                 print "waiting for cats"
#                 reactor.iterate(1)
                 print [cat for cat in cats if not os.path.exists(cat)]
                 time.sleep(1)
            elif NP:
               RunForkedQueue(ttask,sexes,MaxThreads=NP,monitor=0,sleeptime=0.01)
         xo = 0.0
         yo = 0.0
         to = 0.0
         xr = 0.0
         yr = 0.0
         tr = 0.0
         lmat = 0
         refim = Exposures[0].name
      for chip in chips:
         if first:
            cat = self.cat1(chip)
            sfits = self.sfits1(chip)
         elif second:
            cat = self.cat2(chip)
            sfits = self.sfits2(chip)
         if os.path.exists(sfits):
            F = FITS(sfits,readonly=0)
            F.modkey("refim1",refim)
            F.modkey("lmatch1",lmat)
            F.modkey("xdiff1",xo)
            F.modkey("ydiff1",yo)
            F.modkey("mdiff1",to)
            F.modkey("xscat1",xr)
            F.modkey("yscat1",yr)
            F.modkey("mscat1",tr)
            F.close()

def Sextractor(image,catalog,sexdir="./sex/",scale=0.2,thresh=3.0,saturate=55e3,seeing=1.3,segm=0,clean=1):
      simage = image.replace(".fits","sig.fits")
      if "Stack" not in image and "Mosaic" not in image:
         F = FITS(image)
         d = F.data()
         F.close()
         d = where(between(d,-1e24,1e24),d,0)
         m = where(d,d-VTKMedianSmooth(d,256,256),-1e31)
#         m = VTKGauss(m,1,1)
         G = FITS("/tmp/"+os.path.basename(image),create=1,template=image)
         G.putdata(m)
         G.close()
         flg = None
         uimage = "/tmp/"+os.path.basename(image)
         print uimage,os.path.exists(uimage)
         tw = 0
         while not os.path.exists(uimage) and tw < 20:
           print uimage,os.path.exists(uimage)
           time.sleep(0.1)
           tw += 1
         if not os.path.exists(uimage):
            raise IOError,"%s not found!" % (uimage)
         sexcom = ["sex "+uimage]
         sexcom += ["-c "+sexdir+"default.sex"]
         chip = int(image.split(".")[-2][-1])
#         sexcom += ["-WEIGHT_TYPE BACKGROUND"]
#         sexcom += ["-WEIGHT_IMAGE maskc%d.fits" % (chip)]
      else:
         F = FITS(image)
         d = F.data()
         F.close()
         d = where(between(d,-1e24,1e24),d,0)
         simage = image.replace(".fits","sig.fits")
         if os.path.exists(simage):
            F = FITS(image.replace(".fits","sig.fits"))
            s = F.data()
            F.close()
            s = where(between(s,-1e24,1e24),s,0.0)
            s = where(s,s,-1e31)
         w = not_equal(d,0).astype(float32)
#         b = 3*VTKDivide(VTKMeanSmooth(d*w,128,128,1),VTKMeanSmooth(w,128,128,1)) - \
#                 2*VTKMedianSmooth(d,128,128,1)
         b1 = VTKMedianSmooth(d*w,128,128,1)
         b2 = VTKDivide(VTKMeanSmooth(VTKAbs(VTKSubtract(d,b1))*w,128,128,1), VTKMeanSmooth(w,128,128,1))
         w = w*less_equal(VTKAbs(VTKSubtract(d,b1)),3*b2)
         b = VTKDivide(VTKLevel(d*w,5,k=3,verb=1), VTKLevel(w,5,k=3,verb=1))
         m = where(d,d-b,-1e31)
#         b = VTKMedianSmooth(d*w,512,3,1)
#         m = where(w,d-b,0)
#         b = VTKMedianSmooth(m*w,3,512,1)
#         m = where(w,m-b,0)
         flg = where(logical_or(equal(s,0),equal(m,0)),1,0)
         G = FITS("/tmp/"+os.path.basename(image),create=1,template=image)
         G.putdata(m)
         G.close()
         G = FITS("/tmp/"+os.path.basename(image.replace(".fits","sig.fits")),create=1,template=image)
         G.putdata(s)
         G.close()
         G = FITS(image.replace(".fits","flg.fits"),create=1,template=image)
         G.modkey("bitpix",16)
         G.close()
         G = FITS(image.replace(".fits","flg.fits"),readonly=0)
         G.putdata(flg.astype(int16))
         G.close()
         G = FITS(image.replace(".fits","smo.fits"),create=1,template=image)
         G.putdata(b)
         G.close()
         uimage = "/tmp/"+os.path.basename(image)
         tw = 0
         while not os.path.exists(uimage) and tw < 20:
            print uimage,os.path.exists(uimage)
            time.sleep(0.1)
            tw += 1
         if not os.path.exists(uimage):
            raise IOError,"%s not found!" % (uimage)
         sexcom = ["sex /tmp/"+os.path.basename(image)]
         sexcom += ["-c "+sexdir+"default.sex"]
         sexcom += ["-WEIGHT_TYPE MAP_RMS"]
         sexcom += ["-WEIGHT_IMAGE /tmp/"+os.path.basename(image.replace(".fits","sig.fits"))]
         sexcom += ["-FLAG_IMAGE "+image.replace(".fits","flg.fits")]
         sexcom += ["-FLAG_TYPE AND"]
         chip = 0
      sexcom += ["-MAG_ZEROPOINT 25"]
      sexcom += ["-INTERP_MAXXLAG 32"]
      sexcom += ["-INTERP_MAXYLAG 32"]
      sexcom += ["-INTERP_TYPE ALL"]
      sexcom += ["-PARAMETERS_NAME "+sexdir+"default.param"]
      sexcom += ["-DETECT_MINAREA 5"]
      sexcom += ["-DETECT_THRESH "+str(thresh)]
      sexcom += ["-ANALYSIS_THRESH "+str(thresh)]
      sexcom += ["-PIXEL_SCALE %f" % (scale)]
      sexcom += ["-SEEING_FWHM %f" % (seeing)]
      sexcom += ["-CATALOG_NAME /tmp/%s" % (os.path.basename(catalog))]
      if segm:
         sexcom += ["-CHECKIMAGE_TYPE SEGMENTATION,BACKGROUND"]
         segm = image.replace(".fits","seg.fits")
         bgm = image.replace(".fits","bck.fits")
         sexcom += ["-CHECKIMAGE_NAME "+segm+","+bgm]
      else:
         sexcom += ["-CHECKIMAGE_TYPE NONE"]
      sexcom += ["-FILTER_NAME "+sexdir+"gauss_3.0_5x5.conv"]
      sexcom += ["-STARNNW_NAME "+sexdir+"default.nnw"]
      sexcom += ["-SATUR_LEVEL "+str(saturate)]
      sexcom += ["-VERBOSE_TYPE QUIET"]
      command = " ".join(sexcom)
      verbsh(command)
#      sc = open("/tmp/sc","w")
#      print >>sc, command
#      sc.close()
      while not os.path.exists( "/tmp/%s"%(os.path.basename(catalog))):
          print "/tmp/%s"%(os.path.basename(catalog)),
          print os.path.exists( "/tmp/%s"%(os.path.basename(catalog)))
          time.sleep(1)
          raise IOError, "/tmp/%s does not exist!"%(os.path.basename(catalog))

      f = open("/tmp/%s"%(os.path.basename(catalog)),"r")
      g = open(catalog,"w")
      g.write(f.read())
      g.close()
      f.close()
      if clean:
         if os.path.exists("/tmp/%s"%(os.path.basename(catalog))):
            os.remove("/tmp/%s"%(os.path.basename(catalog)))
         if os.path.exists("/tmp/"+os.path.basename(image)):
            os.remove("/tmp/"+os.path.basename(image))
         if os.path.exists("/tmp/"+os.path.basename(simage)):
            os.remove("/tmp/"+os.path.basename(simage))

def Offsets( Exposures, chip):
   mXs = []
   mYs = []
   sXs = []
   sYs = []
   xscale = Exposures[0].scale
   yscale = Exposures[0].scale
   sfits = lambda E,c: E.sfits1(c)
   cs = [c for c in [1,2,3,4] if os.path.exists(sfits(Exposures[0],c))]
   for c in cs:
      G = [FITS(sfits(E,c)) for E in Exposures]
      gX = array([g["xdiff1"] for g in G])/xscale
      gY = array([g["ydiff1"] for g in G])/yscale
      gXs = add.accumulate(gX)
      gYs = add.accumulate(gY)
      [g.close() for g in G]
      mX0,mX1 = extrema(gXs)
      mY0,mY1 = extrema(gYs)
      sX = mX1 - mX0
      sY = mY1 - mY0
      sXs.append(sX)
      sYs.append(sY)
      mXs.append(mX0)
      mYs.append(mY0)
   aX = argmax(sXs)
   aY = argmax(sYs)
   sX = int(sXs[aX])
   sY = int(sYs[aY])
   mX0 = mXs[aX]
   mY0 = mYs[aY]
   if Exposures[0].fclass == "newfirm":
      if Exposures[0].mjd < 54480:
         s = add((2048,2048),(sX,sY)).astype(int32)+1
         solnfile = join([os.environ["PYTHONDATA"],"photometry","image3526c%d.soln" % (chip)],"/")
      else:
         s = add((2048,2048),(sY,sX)).astype(int32)+1
         solnfile = join([os.environ["PYTHONDATA"],"photometry","image52627c%d.soln" % (chip)],"/")
   elif Exposures[0].fclass == "fourstar":
         s = add((2048,2048),(sX,sY)).astype(int32)+1
         solnfile = join([os.environ["PYTHONDATA"],"photometry","fourstar201101c%d.soln" % (chip)],"/")
   print "Reading",solnfile
   solndat = readsoln(solnfile)
   F = [FITS(sfits(E,chip)) for E in Exposures]
   X = array([f["xdiff1"] for f in F])/xscale
   Y = array([f["ydiff1"] for f in F])/yscale
   Xd = add.accumulate(X)
   Yd = add.accumulate(Y)
   Xs = s[1]/2. - solndat["center"][0]
   Ys = s[0]/2. - solndat["center"][1]
   Xm = mean(extrema(Xd))
   Ym = mean(extrema(Yd))
   Xd = Xd - Xm
   Yd = Yd - Ym
   Xz = Xd
   Yz = Yd
   [f.close() for f in F]
   return Xz,Yz,Xs,Ys

def Job(func,args):
   os.nice(15)
   chips = [1,2,3,4]
   dicts = [{"args":(chip,),"keywords":args} for chip in chips]
   print func.__name__
   queue = TwistedObjectQueue(func.__name__,dicts)
   queue.repeatable = False
   queue.Run(forks=False,sleeptime=0.02)
   print queue.results

def Stacks(NDS,yskip=32,usemed=1,first=1,second=0,name="FirstStack.fits",justsex=0):
   NDS = [N1 for N1 in NDS if not Exposure(N1).bad]
   args = {"NDS":NDS,"yskip":yskip,"first":first,"second":second,"name":name,"justsex":justsex,"usemed":usemed}
   Job(CIRCEStack,args)

def CIRCEStack(chip,NDS=[],yskip=32,usemed=1,first=1,second=0,name="FirstStackcD.fits",\
        justsex=0,sigclip=0,SuperExposures=[],Hub=None,NP=-1,create=0):
   if NP == -1: NP = int(os.environ.get("NP",0))
   if isinstance(NDS[0],newfirm):
      NDS = [N1 for N1 in NDS if not Exposure(N1).bad]
      Exposures = [Exposure(N1) for N1 in NDS]
   elif isinstance(NDS[0],fourstar):
      NDS = [N1 for N1 in NDS if not Exposure(N1).bad]
      Exposures = [Exposure(N1) for N1 in NDS]
   else:
      Exposures = NDS
   if not SuperExposures:
      SuperExposures = Exposures
   Esky = array([E.sky for E in Exposures])
   SuperEsky = array([E.sky for E in SuperExposures])
   bab = (sum(SuperEsky) and (sum(SuperEsky) < len(SuperEsky)))
   if bab:
     ncombine = len(Esky)
   else:
     ncombine = len(Esky) - sum(Esky)
   if first:
     sfits = lambda E,c: E.sfits1(c)
     mfits = lambda E,c: E.mfits1(c)
   elif second:
     sfits = lambda E,c: E.sfits2(c)
     mfits = lambda E,c: E.mfits2(c)
     pfits = lambda E,c: E.pers1(c)
   F = [FITS(sfits(E,chip)) for E in Exposures]
   exptimes = [f["exptime"] for f in F]
   [f.close() for f in F]
   cname = name.replace("cD.fits","c"+str(chip)+".fits")
   ename = cname.replace(".fits","map.fits")
   sname = cname.replace(".fits","sig.fits")
   if not justsex:
      if 1:
         xscale = Exposures[0].scale
         yscale = Exposures[0].scale
         mXs = []
         mYs = []
         sXs = []
         sYs = []
         for c in [1,2,3,4]:
            if os.path.exists(sfits(SuperExposures[0],c)):
               G = [FITS(sfits(E,c)) for E in SuperExposures]
               gX = array([g["xdiff1"] for g in G])/xscale
               gY = array([g["ydiff1"] for g in G])/yscale
               gXs = add.accumulate(gX)
               gYs = add.accumulate(gY)
               [g.close() for g in G]
               mX0,mX1 = extrema(gXs)
               mY0,mY1 = extrema(gYs)
               sX = (mX1 - mX0)
               sY = (mY1 - mY0)
               sXs.append(sX)
               sYs.append(sY)
               mXs.append(mX0)
               mYs.append(mY0)
         aX = argmax(sXs)
         aY = argmax(sYs)
         sX = int(sXs[aX])
         sY = int(sYs[aY])
         mX0 = mXs[aX]
         mY0 = mYs[aY]
         if Exposures[0].mjd < 54480:
            s = add((2048,2048),(sX,sY)).astype(int32)+1
         else:
            s = add((2048,2048),(sY,sX)).astype(int32)+1
         print  "Image size = %d x %d" % (s[1],s[0])
         stack = zeros(s,float32) + 1e38
         emap = zeros(s,float32)
         smap = zeros(s,float32)
         if os.path.exists(cname): os.remove(cname)
         if os.path.exists(ename): os.remove(ename)
         if os.path.exists(sname): os.remove(sname)
         if Exposures[0].fclass == "newfirm":
            if Exposures[0].mjd < 54480:
               solnfile = join([os.environ["PYTHONDATA"],"photometry","image3526c%d.soln" % (chip)],"/")
            else:
               solnfile = join([os.environ["PYTHONDATA"],"photometry","image52627c%d.soln" % (chip)],"/")
         elif Exposures[0].fclass == "fourstar":
             solnfile = join([os.environ["PYTHONDATA"],"photometry","fourstar201101c%d.soln" % (chip)],"/")
         print "Reading",solnfile
         solndat = readsoln(solnfile)
         solndat["ic"] = invsoln(solndat,2048,2048,verb=1)
#         y,x = indices(s,float32)
#         x = x - s[1]/2. #+ solndat["center"][0]
#         y = y - s[0]/2. #+ solndat["center"][1]
         F = [FITS(sfits(E,chip)) for E in SuperExposures]
         X = array([f["xdiff1"] for f in F])
         Y = array([f["ydiff1"] for f in F])
         M = array([f["median"] for f in F])
         W = array([f["anoise"] for f in F])
         Xd = add.accumulate(X)
         Yd = add.accumulate(Y)
         Xd = Xd - mean(extrema(Xd))
         Yd = Yd - mean(extrema(Yd))
         if Exposures[0].mjd < 54480: sX,sY = sY,sX
         supernos = [E.expno for E in SuperExposures]
         ebegin = supernos.index(Exposures[0].expno)
         Xd = Xd[ebegin:]
         Yd = Yd[ebegin:]
         for mname,marray in zip([cname,ename,sname],[stack,emap,smap]):
            print "Creating",mname
            if create or not FileCheck(mname):
               map(os.remove,glob(os.path.dirname(mname)+"/."+os.path.basename(mname).replace(".fits","_????.fits")))
               So = FITS(mname,create=1,template=F[0].name())
               So.modkey("exptime",1.0)
               So.modkey("ncombine",ncombine)
               So.resize([s[1],s[0]])
               So.putdata(marray)
               So.flush()
               So.close()
         [f.close() for f in F]
         start = time.time()
         Ny = stack.shape[0]/yskip+1
         sys.stdout.flush()
         if Hub != None and Hub[0] != None:
             mhost,mport = Hub[0].split(":")
             hport = Hub[1]
             mport = int(mport)
             print "hub host is %s, port %d" % (mhost,mport)
             doj=[TwistedTask(-1,"CIRCEStackSec",\
                j,yskip,cname,sname,ename,s,chip,Exposures,M,W,\
                exptimes,solndat,first,second,Xd,Yd,usemed,sigclip)\
                for j in range(Ny)]
             [item.setFromModules(["CIRCEProcess"]) for item in doj]
             print "go!"
#             H = HubDispatch(doj,(mhost,mport),mport=0,namespace=globals(),verb=1,sleeptime=0.01)
             if create: return doj
         elif NP:
            doj = lambda j: CIRCEStackSec(j,yskip,cname,sname,ename,s,chip,Exposures,M,W,exptimes,solndat,first,second,Xd,Yd,usemed,sigclip)
            RunForkedQueue(doj,range(Ny),MaxThreads=NP,monitor=0,sleeptime=0.01)
         else:
            doj = lambda j: CIRCEStackSec(j,yskip,cname,sname,ename,s,chip,Exposures,M,W,exptimes,solndat,first,second,Xd,Yd,usemed,sigclip)
            for j in range(Ny):
               doj(j)
               dt = (time.time()-start)
               j1 = (j+1)*yskip
               print "Elapsed=%.2fs Remain=%.2fs" % (dt,(stack.shape[0]-j1)*dt/j1)
               sys.stdout.flush()
         print
         finish = time.time()
         print "Total=%.2fs" % (finish-start)
         sys.stdout.flush()
   catalog = cname.replace(".fits",".cat")
   saturate = 50e3 * len(Exposures) / sum(exptimes)
#   while not os.path.exists(cname) or not os.path.exists(ename) or not os.path.exists(sname):
#     print cname, ename, sname
#     print os.getcwd(), Exposures[0].objname
#     print not os.path.exists(cname), not os.path.exists(ename), not os.path.exists(sname)
#     time.sleep(0.5)
   MakeThumbnail(cname,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
   MakeThumbnail(ename,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
   MakeThumbnail(sname,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
   MakeThumbnail(cname,mag=8,pct=40,s1=-10,s2=10,inv=1,q=85)
   MakeThumbnail(ename,mag=8,pct=40,s1=-10,s2=10,inv=1,q=85)
   MakeThumbnail(sname,mag=8,pct=40,s1=-10,s2=10,inv=1,q=85)
   scale = Exposures[0].scale
   Sextractor(cname,catalog,scale=scale,saturate=saturate,thresh=2.5,seeing=3*scale,segm=1)

def CIRCEStackSec(j,yskip,cname,sname,ename,sshape,chip,Exposures,M,W,exptimes,solndat,first,second,Xd,Yd,usemed,sigclip):
   if first:
     sfits = lambda E,c: E.sfits1(c)
     bfits = lambda E,c: E.bfits1(c)
     mfits = lambda E,c: E.mfits1(c)
   elif second:
     sfits = lambda E,c: E.sfits2(c)
     bfits = lambda E,c: E.bfits2(c)
     mfits = lambda E,c: E.mfits2(c)
     pfits = lambda E,c: E.pers1(c)
   j0 = j*yskip
   j1 = min([j0+yskip,sshape[0]])
   if j1>j0:
      dm,em,um,vm = [zeros((len(Exposures),j1-j0,sshape[1]),float64) \
                       for jim in range(4)]
#      xi0 = x[j0:j1] #- solndat["center"][0]
#      yi0 = y[j0:j1] #- solndat["center"][1]
      x = arange(sshape[1]) - sshape[1]/2.
      y = arange(j1-j0)+j0  - sshape[0]/2.
      xi0 = x[NewAxis,::] * ones(y.shape)[::,NewAxis]
      yi0 = y[::,NewAxis] * ones(x.shape)[NewAxis,::]
      si0 = xi0.shape
      ui0,vi0 = mbasis(solndat["c"],xi0,yi0)
      for i,E in enumerate(Exposures):
         if exptimes[i]:
            Xd1 = Xd[i]*pi/180/3600
            Yd1 = Yd[i]*pi/180/3600
            ui = ui0 - Xd1
            vi = vi0 - Yd1
            xi,yi = mbasis(solndat["ic"],ui,vi)
            xi = xi + solndat["center"][0]
            yi = yi + solndat["center"][1]
            x0,x1 = clip(add([0,1],extrema(xi.ravel())),0,2047).astype(int32)
            y0,y1 = clip(add([0,1],extrema(yi.ravel())),0,2047).astype(int32)
            if not E.cache:
               f = FITS(sfits(E,chip))
               d = f.readsec([x0,y0],[x1,y1])
               f.close()
#               mf = FITS(mfits(E,chip))
#               nd = mf.readsec([x0,y0],[x1,y1])
#               mf.close()
            elif E.cache:
#               E.Data(chip,first,second)
               E.SData(chip,first,second)
               d = E.sdata[chip][y0:y1+1,x0:x1+1]
#               nd = E.data[chip][y0:y1+1,x0:x1+1]
            d = where(between(d,-1e12,1e12),d,0.0)
##            bf = FITS(bfits(E,chip))
##            bd = bf.readsec([x0,y0],[x1,y1])
##            bf.close()
            d = divz(d,exptimes[i]).astype(float32)
#            if second:
##               pf = FITS(pfits(E,chip))
#               pd = pF[i].readsec([x0,y0],[x1,y1])
##               pf.close()
#               d = where(pd,0,d)
#            v = (not_equal(d,0)*getsqrt(8*bd + 25**2)/8.).astype(float32)
#            v = divz(v,exptimes[i]).astype(float32)
            v = divz(W[i]*ones(d.shape),exptimes[i]).astype(float32)
            e = (not_equal(d,0)*exptimes[i]).astype(float32)
            ny,nx = d.shape
            xi0 = (xi+0.5-x0).astype(int32).ravel()
            yi0 = (yi+0.5-y0).astype(int32).ravel()
            c0 = (yi0*nx + xi0)
            t0 = clip(c0,0,len(d.ravel())-1)
            o0 = reshape(between(c0,0,len(d.ravel())-1),si0)
            d0 = reshape(take(d.ravel(),t0),si0)
            v0 = reshape(take(v.ravel(),t0),si0)
            e0 = reshape(take(e.ravel(),t0),si0)
            u0 = between(xi,0,2047)*between(yi,0,2047)
            u0 = u0*not_equal(d0,0)*o0*greater(d0,-4*v0)
            u0 = u0.astype(float32)
            v0 = (v0*u0).astype(float32)
            dm[i] = d0.astype(dm.dtype)
            em[i] = e0.astype(em.dtype)
            um[i] = u0.astype(um.dtype)
            vm[i] = v0.astype(vm.dtype)
#      qdump("/tmp/dm.fits",dm)
#      qdump("/tmp/vm.fits",vm)
#      print sort(dm.ravel())[:10], sort(dm.ravel())[-10:]
#      print sort(vm.ravel())[:10], sort(vm.ravel())[-10:]
      ums = sum(um)
      if usemed:
         if 1:
            dms = sort(where(vm,dm,1e18),0)
            Mo = sum(less(dms,1e16)).astype(int32)
            mMo = int(max(Mo.ravel()))
            nope = True
            if mMo:
               nope = False
               dms = dms[:mMo]
               Mo2i = (Mo/2.).astype(int32)
               Moi = ((Mo+1)/2.).astype(int32)
               Meven = equal(Mo2i,Moi)
               Modd = not_equal(Mo2i,Moi)
               aF = arange(0,len(dms),1,int32)[::,NewAxis,NewAxis]

               Rodd = add.reduce(dms * equal(aF,Mo2i),0)
               Reven = add.reduce(dms * (equal(aF,Mo2i-1)+equal(aF,Mo2i)),0)/2.0
               mm = where(Modd,Rodd,Reven)*greater(Mo,0)
               sm = abs(dm-mm[NewAxis,::,::])

               if sigclip:
                  sms = sort(where(vm,sm,1e18),0)[:mMo]
                  Rodd = add.reduce(sms * equal(aF,Mo2i),0)
                  Reven = add.reduce(sms * (equal(aF,Mo2i-1)+equal(aF,Mo2i)),0)/2.0
                  rm = 1.49*where(Modd,Rodd,Reven)*greater(Mo,0)

                  for mit in range(3):
                     wm = um*Huber(divz(sm,rm[NewAxis,::,::]),w=1)
                     wm = divz(wm,vm**2)
                     mm = divz(sum(dm*wm),sum(wm))
                     sm = abs(dm-mm[NewAxis,::,::])
                     sms = sort(where(vm,sm,1e18),0)[:mMo]
                     Rodd = add.reduce(sms * equal(aF,Mo2i),0)
                     Reven = add.reduce(sms * (equal(aF,Mo2i-1)+equal(aF,Mo2i)),0)/2.0
                     rm = 1.49*where(Modd,Rodd,Reven)*greater(Mo,0)

               else:
                  wm = um
               sm = getsqrt(divz(sum(power(sm,2)*wm),(sum(um)-1)*sum(wm)))
               em = sum(em*um)
            else:
               mm = zeros(dms[0].shape,float32)
               sm = zeros(dms[0].shape,float32)
               em = zeros(dms[0].shape,float32)
         elif 0:
            mm = divz(sum(um*dm),sum(um))
            sm = sqrt(divz(sum(um*vm**2),sum(um)*sum(um)))
            em = sum(em*um)
         if 0 and second:
            rm = dm-mm[NewAxis,::,::]
#            km = less(abs(rm),5*vm)
            wm = divz(1,vm**2)
            km = Lp(divz(rm,vm),w=1)
            um = um*not_equal(km,0)
            mm = divz(sum(dm*um*km*wm),sum(um*km*wm))
            em = divz(sum(em*um*km*wm),sum(um*km*wm))
            sm = sqrt(divz(sum((dm-mm)**2*um*km*wm),sum(um*km*wm)*sum(um*km)))
      else:
         wm = divz(1,vm**2)
         mm = divz(sum(dm*um*wm),sum(um*wm))
         em = divz(sum(em*um*wm),sum(um*wm))
         sm = sqrt(divz(sum((dm-mm)**2*um*wm),sum(um*wm)*sum(um)))
      jno = Set([E.expno for E in Exposures])
      minn = len(Exposures)/10.+1
      minn = 2
#      s2n = divz(mm,sm)
#      keep = greater(ums,minn)
      mint = sum(exptimes)/16.0
      keep = greater_equal(em,mint)
      mm = mm*keep
      sm = sm*keep
      em = em*keep
      mm = mm.astype(float32)
      em = em.astype(float32)
      sm = sm.astype(float32)

      cname1 = os.path.dirname(cname)+"/."+os.path.basename(cname).replace(".fits","_%04d.fits"%(j0))
      ename1 = os.path.dirname(ename)+"/."+os.path.basename(ename).replace(".fits","_%04d.fits"%(j0))
      sname1 = os.path.dirname(sname)+"/."+os.path.basename(sname).replace(".fits","_%04d.fits"%(j0))
      qdump(cname1,mm)
      qdump(ename1,em)
      qdump(sname1,sm)

#      So = FITS(cname,readonly=0)
#      So.writesec(mm,[0,j0],[sshape[1]-1,j1-1])
#      So.close()

#      Eo = FITS(ename,readonly=0)
#      Eo.writesec(em,[0,j0],[sshape[1]-1,j1-1])
#      Eo.close()

#      Vo = FITS(sname,readonly=0)
#      Vo.writesec(sm,[0,j0],[sshape[1]-1,j1-1])
#      Vo.close()

def DrawMask(chip,cat,E,size,mag=3.0,verb=0):
   mask = zeros(size,float32)
   pers = zeros(size,float32)
   Xz = cat["Xz"]
   Yz = cat["Yz"]
   Xs = cat["Xs"]
   Ys = cat["Ys"]
   Es = cat["exp"]
   cnaxis1, cnaxis2 = cat["naxes"]
   Ei = [i for i,E1 in enumerate(Es) if E1.name == E.name][0]
   xscale = E.scale
   yscale = E.scale
   if E.fclass == "newfirm":
      if E.mjd < 54480:
         solnfile = join([os.environ["PYTHONDATA"],"photometry","image3526c%d.soln" % (chip)],"/")
      else:
         solnfile = join([os.environ["PYTHONDATA"],"photometry","image52627c%d.soln" % (chip)],"/")
   elif E.fclass == "fourstar":
       solnfile = join([os.environ["PYTHONDATA"],"photometry","fourstar201101c%d.soln" % (chip)],"/")
   solndat = readsoln(solnfile)
   solndat["ic"] = invsoln(solndat,2048,2048,verb=1)
   i = Ei
   ip = 0
   dtol = 50.0

   zim1 = NumToVTKImage(mask)
   zim2 = NumToVTKImage(pers)

   canvas1 = vtkImageCanvasSource2D()
   canvas1.SetExtent(1,mask.shape[1],1,mask.shape[0],0,0)
   canvas1.SetScalarType(VTK_FLOAT)
   canvas1.DrawImage(0,0,zim1)
   canvas1.SetDrawColor(1.0)
   canvas1.Update()

   canvas2 = vtkImageCanvasSource2D()
   canvas2.SetExtent(1,pers.shape[1],1,pers.shape[0],0,0)
   canvas2.SetScalarType(VTK_FLOAT)
   canvas2.DrawImage(0,0,zim2)
   canvas2.SetDrawColor(1.0)
   canvas2.Update()

   cent,scat = Es[i].ReadMode(chip,first=1,second=0)

   if Es[i].fclass == "fourstar":
      maxip = 0
   elif Es[i].fclass == "newfirm":
      maxip = 3

   while i >= 0 and ip <=maxip:
      if (i == Ei) or sqrt((Xz[i]-Xz[Ei])**2+(Yz[i]-Yz[Ei])**2) > dtol:
         if (i != Ei) and sqrt((Xz[i]-Xz[i+1])**2+(Yz[i]-Yz[i+1])**2) > dtol: ip += 1
         if ip == 0:
            dat = readsex( cat["cat"])
            xl,yl = dat["X_IMAGE"]-Xs, dat["Y_IMAGE"]-Ys
            u,v = mbasis(solndat["c"],xl-solndat["center"][0],yl-solndat["center"][1])
            du = -Xz[i]*xscale*pi/180/3600
            dv = -Yz[i]*yscale*pi/180/3600
            xl,yl = mbasis(solndat["ic"],u+du,v+dv)
            xl = xl + 1024
            yl = yl + 1024
         else:
            dat = readsex( Es[i].cat1(chip))
            xl,yl = dat["X_IMAGE"], dat["Y_IMAGE"]

#         TEST THIS...
         cent,scat = Es[i].ReadMode(chip,first=1,second=0)

         for n in range( len(dat["NUMBER"])):
            x = xl[n]
            y = yl[n]
            if "MAG_AUTO" in dat:
              mb = dat["MAG_AUTO"][n]
            else:
              mb = dat["MAG_BEST"][n]
            ma = dat["MAG_APER"][n]
            fw = dat["FWHM_IMAGE"][n]
            a = 1.3*dat["A_IMAGE"][n]
            b = 1.3*dat["B_IMAGE"][n]
            a = max([a / (1.0+ip),1])
            b = max([b / (1.0+ip),1])
            ar = pi*a*b
            bg = dat["BACKGROUND"][n]
            t = dat["THETA_IMAGE"][n]*pi/180
            tc = exp10(-0.4*(mb-25))/ar + bg + cent

            if  0<x<2048 and 0<y<2048 and (i==Ei or tc>10000.0) and \
                        bg and mb != 99.0:
#                        and ma != 99.0:
               dth = (2*pi)/50
               th = arange(0,2*pi+0.5*dth,dth,float64)
               cth = cos(th)
               sth = sin(th)
               ct = cos(t)
               st = sin(t)
               xr = mag*a*cth
               yr = mag*b*sth
               xr,yr = xr*ct - yr*st, xr*st + yr*ct
               xr,yr = xr+x-1,yr+y-1
               m = len(xr)
#               if verb: print Es[i],int(x),int(y),a,b,int(t*180/pi),mb,int(cent),int(tc)
               ixr = (xr+0.5).astype(int32)
               iyr = (yr+0.5).astype(int32)
      
               if i == Ei:
                 canvas = canvas1
                 cim = mask
               else:
                 canvas = canvas2
                 cim = pers

               ixr0,ixr1 = extrema(clip(1+ixr,1,2048))
               iyr0,iyr1 = extrema(clip(1+iyr,1,2048))

               es = vtkImageEllipsoidSource()
               es.SetOutputScalarTypeToFloat()
               es.SetWholeExtent( ixr0,ixr1,iyr0,iyr1,0,0)
               es.SetCenter(x,y,0)
               es.SetRadius(mag*a,mag*b,0)
               es.SetInValue(1.0)
               es.SetOutValue(0.0)

               rr = vtkGeneralTransform()
               rr.Translate(x,y,0)
               rr.RotateZ(-t*180/pi)
               rr.Translate(-x,-y,0)

               er = vtkImageReslice()
               er.SetResliceTransform(rr)
               er.SetInput(es.GetOutput())
               ee = er.GetOutput()
               er.Update()

               tmp = vtkImageClip()
               canvas.Update()
               tmp.SetInput( canvas.GetOutput())
               tmp.SetOutputWholeExtent( ixr0,ixr1,iyr0,iyr1,0,0)
               tmp.ClipDataOn()
               tmp.Update()
               oe = tmp.GetOutput()

               el = vtkImageLogic()
               el.SetOperationToOr()
               el.SetInput2(ee)
               el.SetInput1(oe)
               eb = el.GetOutput()
               el.Update()
               canvas.DrawImage(ixr0,iyr0,eb,0,0,ixr1-ixr0,iyr1-iyr0+1)

      i -= 1

   output1 = canvas1.GetOutput()
   output1.Update()
   mask = VTKImageToNum(output1)
   output2 = canvas2.GetOutput()
   output2.Update()
   pers = VTKImageToNum(output2)

   mask = logical_not(mask).astype(int32)
   mask = VTKMultiply(mask,VTKConvolve(VTKIslandRemoval(mask,0,1,20000,1),k=5,numret=1))
   mask = VTKMultiply(mask,VTKConvolve(VTKIslandRemoval(mask,0,1,20000,1),k=5,numret=1))
   mask = logical_not(mask).astype(int32)

   if Es[i].fclass == "fourstar":
       pers = zeros(mask.shape,int32)
   else:
       pers = logical_not(pers).astype(int32)
       pers = VTKMultiply(pers,VTKConvolve(VTKIslandRemoval(pers,0,1,20000,1),k=5,numret=1))
       pers = VTKMultiply(pers,VTKConvolve(VTKIslandRemoval(pers,0,1,20000,1),k=5,numret=1))
       pers = logical_not(pers).astype(int32)

   mask = (32766*greater(mask,0)).astype(int16)
   pers = (32766*greater(pers,0)).astype(int16)

   return mask,pers

def CIRCEWave(N,chip,first=True,second=False,verb=1):
   E = Exposure(N)
   E.flwt2d(chip,first=first,second=second,verb=verb)

def CIRCESkySubtract(N,chip,NDS=[],flat=0,first=1,second=0,nf=3,verb=0):
   NDS = [N1 for N1 in NDS if not Exposure(N1).bad]
   Exposures = [Exposure(N1) for N1 in NDS]
   u = [j for j in range(len(NDS)) if NDS[j].base == N.base][0]
   E = Exposures[u]
   E.SkySubtract(chip,Exposures,first=first,flat=flat,second=second,nf=nf,verb=verb)

def CIRCEShift(N,chip,NDS=[],first=1,second=0,nf=3,verb=0):
   NDS = [N1 for N1 in NDS if not Exposure(N1).bad]
   Exposures = [Exposure(N1) for N1 in NDS]
   u = [j for j in range(len(NDS)) if NDS[j].base == N.base][0]
   E = Exposures[u]
   E.Shift(chip,Exposures,first=first,second=second,verb=verb)

#def nonexist(f,d):
#   return (f not in os.listdir(d)) or os.stat(d+"/"+f)[6]==0

def invsoln(solndat,naxis1,naxis2,verb=0):
    nterms = len(solndat["c"])
    np = 2*nterms
    x = asarray( range(0,naxis1+naxis1/np,naxis1/np) + [naxis1])
    y = asarray( range(0,naxis2+naxis2/np,naxis2/np) + [naxis2])
    x = VTKSubtract(x, solndat["center"][0])
    y = VTKSubtract(y, solndat["center"][1])
    nx = len(x); ny = len(y)
    o = ones((ny,nx),float32)
    yy = (o*y[::,NewAxis]).ravel()
    xx = (o*x[NewAxis,::]).ravel()
    u,v = mbasis(solndat["c"],xx,yy)
    nterm = [4,6,12,20,30,42,56]
    nords = [0,1, 2, 3, 4, 4, 4]
    m = nords[nterm.index(nterms)]
    b = abasis(m,u,v)
    c = svdfit(b,concatenate([xx,yy]))
    if verb:
      xf,yf = mbasis(c,u,v)
      xres = VTKSubtract(xf,xx); yres = VTKSubtract(yf,yy)
      print "Using",nterm,"terms (order=",m,")"
      print "Inverse solution RMS=",rms(xres),rms(yres)
    return c

def FourStarLinearize1D(data, chip,mode="F"):
    F = {1:(2.927e-6,-3.464e-11,8.476e-16),
         2:(2.603e-6,-2.556e-11,6.636e-16),
         3:(3.215e-6,-4.064e-11,7.393e-16),
         4:(2.490e-6,-1.329e-11,6.897e-16),
         }
    L = {1:(9.338e-7,3.308e-12),
         2:(7.96e-7,5.101e-12),
         3:(1.100e-6,8.068e-13),
         4:(1.366e-6,-1.200e-12),
         }
    coe = eval(mode)[chip]
    cor = ones(data.shape,float)
    [add(cor,c*power(data,j+1),cor) for j,c in enumerate(coe)]
    data = data*cor
    return data

#Anthony's function
def NEWFIRMLinearize1D(data, chip, exptime, digavg=4, nreads=1):
    xsize = data.shape[1]
    ysize = data.shape[0]
    if (digavg == 1):
      tcds = 0.56
    else:
      tcds = 0.564+0.148*digavg

    if (chip == 1):
      treset = 1./ysize*(arange(xsize*ysize).reshape(ysize,xsize)/xsize+1)
    elif (chip == 2):
      treset = 1./xsize*(arange(xsize*ysize).reshape(ysize,xsize)%xsize+1)
    elif (chip == 3):
      treset = 1./xsize*(xsize+1-arange(xsize*ysize).reshape(ysize,xsize)%xsize)
    elif (chip == 4):
      treset = 1./ysize*(ysize+1-arange(xsize*ysize).reshape(ysize,xsize)/xsize+1)
    else:
      treset = zeros((ysize,xsize))+0.

    if (nreads > 1):
      treset = 0.0346 + (tcds+0.0568)*(nreads-1)/2. + tcds*treset
    else:
      treset = 0.0346 + tcds*treset
    tsq = (treset+exptime)**2 - treset**2

    #Below s must be replaced by images for final full solution
    if (chip == 1):
      s= -6.377e-6      #SN019  #Medians off 2d map.
    elif (chip == 2):
      s= -6.714e-6      #SN022
    elif (chip == 3):
      s= -4.528e-6      #SN013
    elif (chip == 4):
      s= -5.148e-6      #SN011
    else:
      s = 0
    #Linearity formula
    output = exptime * ( -1.*exptime +(exptime**2+4*s*tsq*data)**0.5)/(2*s*tsq)
    return output

def SeriousMasks(i1,sig=3,mask=None,allmed=None):
    if mask != None:
       mask = (not_equal(i1,0)*mask).astype(float32)
    else:
       mask = not_equal(i1,0).astype(float32)
    if allmed == None:
        i1 = where(mask,i1,divz(VTKMeanSmooth(VTKMultiply(i1,mask),64,64,numret=1),
                  VTKMeanSmooth(mask,64,64,numret=1)))
        i12 = VTKGrowN(i1,2,numret=1)
        c1 = VTKConvolve(i12,numret=1,kernel=array([[0,-1,0],[-1,4,-1],[0,-1,0]])/4.)
        k1 = not_equal(i12,0).astype(float32)
        r1 = VTKDivide(VTKMeanSmooth(VTKMultiply(VTKAbs(c1),k1),128,128,numret=1),VTKMeanSmooth(k1,128,128,numret=1))
        r1 = VTKMultiply(r1,1.25)
        l1 = VTKLessEqual(VTKDivide(VTKAbs(c1),r1),sig).astype(float32)
        for e1 in range(2):
          r1 = VTKDivide(VTKMeanSmooth(VTKMultiply(VTKAbs(c1),l1),128,128,numret=1),VTKMeanSmooth(l1,128,128,numret=1))
          r1 = VTKMultiply(r1,1.25)
          l1 = VTKLessEqual(VTKDivide(VTKAbs(c1),r1),sig).astype(float32)
        l1 = VTKShrinkN(l1,2,usemean=1,numret=1)
        u1 = VTKMultiply(l1,mask)
        m1 = VTKDivide(VTKMeanSmooth(VTKMultiply(i1,u1),64,64,numret=1),
                    VTKMeanSmooth(u1,64,64,numret=1))
        t1 = where(u1,i1,m1)
        b1 = VTKMedianSmooth(t1,256,256,numret=1)
    else:
        b1 = allmed
        u1 = mask
        l1 = mask
    s1 = VTKMultiply(VTKSubtract(i1,b1),u1)
    r1 = VTKDivide(VTKMeanSmooth(VTKAbs(s1),256,256,numret=1),
            VTKMeanSmooth(u1,256,256,numret=1))
    r1 = VTKMultiply(r1,1.25)
    r1 = clip(getsqrt(b1/8+(25/8.)**2),r1,1e10).astype(float32)
    u1 = (not_equal(i1,0.0)*less(abs(s1),sig*r1)).astype(float32)
    if mask!=None: u1 = where(l1*mask,u1,1)
    else: u1 = where(l1,u1,1)
    u1 = VTKIslandRemoval(u1,0,1,8,numret=1)
    u1 = VTKMeanSmooth(u1,8,8,numret=1)
    u1 = equal(u1,1).astype(float32)
    u1 = l1*mask*u1
    return u1

def MakeWaves(i1,si1,sig=3,mask=None,tdata=None,nn=4,nt=4,Fm=0,Fr=0,allmed=None,nowave=0):
    si1s = 1.49*median(abs(compress(mask.ravel(),si1.ravel())))
    si1m = median(si1.ravel())
    omask = less(abs(si1-si1m),sig*si1s).astype(int32)
    omask = VTKMultiply(omask,VTKConvolve(VTKIslandRemoval(omask,0,1,100,1),k=5,numret=1))
    mask = mask * omask
    if tdata!=None:
       Mi1 = tdata
    else:
       Mi1 = VTKDivide(VTKMeanSmooth(i1*mask,128,128,1), VTKMeanSmooth(mask,128,128,1))
    Di1 = VTKAbs(VTKSubtract(i1,Mi1))
    Si1 = 1.25*VTKDivide(VTKMeanSmooth(Di1*mask,512,512,1),VTKMeanSmooth(mask,512,512,1))
    Si1 = where(greater(Si1,Fr),Si1,Fr)
    Di1 = VTKDivide(Di1,Si1)
    Ri1 = VTKDivide(i1,Mi1)
    Ri1x = sum(Ri1[512:1536])
    Ri1y = sum(transpose(Ri1)[512:1536])
    Ri1x = divz(Ri1x,median( Ri1x))
    Ri1y = divz(Ri1y,median( Ri1y))
    Bi = logical_and(greater(Ri1x,0.7)[NewAxis,::],greater(Ri1y,0.7)[::,NewAxis]).astype(int16)
    u1 = logical_or(VTKLessEqual(Di1,5.0),Bi).astype(float32)
    u1 = equal(Bi*mask*VTKMeanSmooth(u1,4,numret=1),1).astype(float32)
    i1u1 = VTKMultiply(i1,u1)
    b5 = zeros(i1.shape,float64)
    for sc in [64,128,256]:
       if not equal(b5,0).any(): break
       b5c = VTKDivide(VTKMeanSmooth(i1u1,sc,sc,1), VTKMeanSmooth(u1,sc,sc,1))
       b5 = where(b5,b5,b5c)
    Di1 = VTKAbs(VTKSubtract(i1,b5))
    Bmask = Bi*mask
    s = 1.49*median(compress(Bmask.ravel(),Di1.ravel()))
    u1 = less(Di1,sig*s).astype(float32)
#    u1 = equal(VTKConvolve(u1,numret=1,k=3),0).astype(float32)
    if nowave:
       return Bi,u1
    else:
       u1 = Bi*mask*u1
       i2 = where(u1,i1,b5)
       w = tomall(flwt2d(i2,nn=nn,nt=nt),nn=nn,nt=nt)
       u = flwt2df(u1,nn=nn,nt=nt)
       if 0:
           qdump("/tmp/Ri1.fits",Ri1)
           qdump("/tmp/Ri1x.fits",Ri1x)
           qdump("/tmp/Ri1y.fits",Ri1y)
           qdump("/tmp/Mi1.fits",Mi1)
           qdump("/tmp/Si1.fits",Si1)
           qdump("/tmp/Di1.fits",Di1)
           qdump("/tmp/Bi.fits",Bi)
           qdump("/tmp/u1.fits",999999*u1)
           qdump("/tmp/b5.fits",b5)
           qdump("/tmp/i2.fits",i2)
           exit()
       return i2,w,u

def OldMakeWaves(i1,sig=3,mask=None,nn=4,nt=4,allmed=None):
    print "Making masks..."
    u1 = SeriousMasks(i1,sig=3,mask=mask,allmed=allmed)
    print "Making waves..."
    i1u1 = VTKMultiply(i1,u1)
    b1 = VTKDivide(VTKMeanSmooth(i1u1,64,64,numret=1),
                VTKMeanSmooth(u1,64,64,numret=1))
    b4 = VTKDivide(VTKMeanSmooth(i1u1,128,128,numret=1),
                VTKMeanSmooth(u1,128,128,numret=1))
    b5 = VTKDivide(VTKMeanSmooth(i1u1,512,512,numret=1),
                VTKMeanSmooth(u1,512,512,numret=1))
    b1 = where(b1,b1,b4)
    b1 = where(b1,b1,b5)
    b1 = b5
    i2 = where(logical_and(u1,not_equal(i1,0)),i1,b1)
    w = tomall(flwt2d(i2,nn=nn,nt=nt),nn=nn,nt=nt)
    u = flwt2df(u1,nn=nn,nt=nt)
    print "Done making waves."
    return i2,w,u

def PlotModes(Exposures,chips,first=1,second=0,flat=0):
    print "Plotting chips",chips
    if len(chips) == 1:
       if first: skyplot = Exposures[0].scratch("skyplot1c%d.ps" % (chips[0]))+"/vcps"
       if second: skyplot = Exposures[0].scratch("skyplot2c%d.ps" % (chips[0]))+"/vcps"
    else:
       if first: skyplot = Exposures[0].scratch("skyplot1.ps")+"/vcps"
       if second: skyplot = Exposures[0].scratch("skyplot2.ps")+"/vcps"
    opendev(skyplot,nx=1,ny=3,land=2,size=7.5)

    Exposures = Exposures
    fsky = array([e.sky for e in Exposures])
    bab = not flat and (sum(fsky) and (sum(fsky) < len(fsky)))
    if bab:
      print "Justing using the object frames!"
      Exposures = [E for E in Exposures if not E.sky]
    skies = [asarray([E.ReadMode(chip,first=first,second=second) for E in Exposures]) for chip in chips]

    mjd = array([E.mjd for E in Exposures])
    fsky = array([E.sky for E in Exposures])
    etime = array([E.exptime for E in Exposures])
    mjd = mjd - mjd[0]
    cis = [1,2,3,4]
    modes = asarray([sky[::,0] for sky in skies])
    scats = asarray([sky[::,1] for sky in skies])
    if Exposures[0].fclass == "fourstar":
       if Exposures[0].gain[0] == "F":
          preds = getsqrt(modes/2.5+(20./2.5)**2)
       elif Exposures[0].gain[0] == "L":
          preds = getsqrt(modes/1.3+(15./1.3)**2)
    elif Exposures[0].fclass == "newfirm":
       preds = getsqrt(modes/8+(25./8)**2)
    rscats = divz(scats,preds)
    lscat = 2*getpct(transpose(rscats),75)

    bad = greater(rscats,lscat[::,NewAxis])
    if fsky.any():
       bad = logical_and(bad, fsky[NewAxis,::])
    for ic,chip in enumerate(chips):
        if first:
           [os.remove(Exposures[j].mfits1(chip)+".bad") for j in range(len(bad)) \
                   if FileCheck(Exposures[j].mfits1(chip)+".bad")]
        for j in nonzero(bad[ic]):
            bf = Exposures[j].mfits1(chip)+".bad"
            print bf
            open(bf,"w").close()

    [E.Bad() for E in Exposures]
    reallybad = array([E.bad for E in Exposures])

    modes = divz(modes,etime[NewAxis,::])
    scats = divz(scats,etime[NewAxis,::])
    preds = divz(preds,etime[NewAxis,::])
    rats = divz(scats,preds)
    xm = add([-0.002,0.002],extrema(mjd))
    ym1 = add([-10,10],extrema(ravel(modes)))
    ym2 = [0.0,2.0*getpct(ravel(scats),95)]
    ym3 = [0.0,2.0*getpct(ravel(rats),95)]

    newbox(xm,ym1,ch=2.3)
    xlabel("MJD-MJD\d0\u",ch=2.3)
    ylabel("Sky (DN/s)",ch=2.3)
    for sky,chip in zip(skies,chips):
       modes = sky[::,0]/etime
       drawpt(mjd,modes,ps=17,ci=cis[chip-1],ch=2)

       bx,by = compress(reallybad,[mjd,modes],1)
       drawpt(bx,by,ps=4,ci=cis[chip-1],ch=3)

    newbox(xm,ym2,ch=2.3)
    xlabel("MJD-MJD\d0\u",ch=2.3)
    ylabel("\gs (DN/s)",ch=2.3)
    for l,rsig,pred,sky,chip in zip(lscat,rscats,preds,skies,chips):
       scats = sky[::,1]/etime
       noise = pred
       drawpt(mjd,scats,ps=17,ci=cis[chip-1],ch=2)
       drawln(mjd,noise,ls=4,ci=cis[chip-1],lw=2)

       bx,by = compress(reallybad,[mjd,scats],1)
       drawpt(bx,by,ps=4,ci=cis[chip-1],ch=3)

    newbox(xm,ym3,ch=2.3)
    xlabel("MJD-MJD\d0\u",ch=2.3)
    ylabel("\gs/\gs\dp\u",ch=2.3)
    for l,rsig,pred,sky,chip in zip(lscat,rscats,preds,skies,chips):
       scats = sky[::,1]/etime
       noise = pred
       rat = divz(scats,noise)
       print chip,quartiles(rat)
       drawpt(mjd,rat,ps=17,ci=cis[chip-1],ch=2)

       bx,by = compress(reallybad,[mjd,rat],1)
       drawpt(bx,by,ps=4,ci=cis[chip-1],ch=3)

       drawln(xm,array([l,l]),ls=1,ci=cis[chip-1],lw=1)

    closedev(pstopdf=True)

def MakeJXY(nn,nt):
   x = ones((2048,2048),float32)
   if not JXY:
      if FileCheck("jar%d_%d.fits" %(nn,nt)) and FileCheck("xyr%d_%d.fits" %(nn,nt)):
         print "Reading jar%d_%d.fits" %(nn,nt)
         F = FITS("jar%d_%d.fits" %(nn,nt))
         jar = F.data()
         F.close()
         print "Reading xyr%d_%d.fits" %(nn,nt)
         F = FITS("xyr%d_%d.fits" %(nn,nt))
         xyr = F.data()
         F.close()
         JXY["jar"] = jar
         JXY["xyr"] = xyr
      else:
         print "Making jar and xyr..."
         y,x = indices((2048,2048),int16)
         jar,xyr = jar2d(x,nn,nt)
         jar = jar.astype(int16)
         xyr = xyr.astype(int16)
         JXY["jar"] = jar
         JXY["xyr"] = xyr
      if not FileCheck("jar%d_%d.fits" %(nn,nt)):
         F = FITS("jar%d_%d.fits" %(nn,nt),create=1)
         F.newhead("T",16,2,[2048,2048])
         F.putdata(JXY["jar"])
         F.close()
      if not FileCheck("xyr%d_%d.fits" %(nn,nt)):
         F = FITS("xyr%d_%d.fits" %(nn,nt),create=1)
         F.newhead("T",16,2,[2048,2048])
         F.putdata(JXY["xyr"])
         F.close()
      JXY["L"] = extrema(list(Set(JXY["jar"].ravel())))

def WaveAnalyze(Exposures,chip,first,second,l,s,ME=None,mpi=1,noclean=0,\
          inter=0,nn=4,nt=4,verb=0,bdir="",origdir=""):
#    ClearWaves(force=1)
    Wx = arange(len(Exposures))
    if first: stage = "1"
    elif second: stage = "2"
    p = int(stage)
    [f.WData(chip,first=first,second=second) for f in Exposures if f.cache]
    Wt = array([f.mjd for f in Exposures])
    FIN = [FITS(f.mfits2(chip)) for f in Exposures]
    We = array([f["exptime"] for f in FIN])
    Wem = We/60.0
    Wms = array([f["median"] for f in FIN])
#    Wt = array([f["mjd-obs"] for f in FIN])
    [f.close() for f in FIN]
    Wt = Wt*24.0*60.0 + We/60.0/2.
    Hmax =   (Wt[-1]-Wt[0]+2*(Wem[-1]+Wem[0]))
    Hmin = (Wt[0]-Wem[0])
    Wt0 = Wt-Hmin
    Hx = (Wt-Hmin)/Hmax
    MakeJXY(nn=nn,nt=nt)
    L = JXY["L"]
    jar = JXY["jar"]
    xyr = JXY["xyr"]
    wpars = origdir + "wavelets%d.pars" % (p)
    wpars = readtable(wpars,header=0,icols=[0,1,2,3])
    nvar,tvar,wvar,svar = compress(equal(wpars["l"],L[1]-l),[wpars["n"],wpars["t"],wpars["w"],wpars["s"]],1)[::,0]
    pvar = [0,1][not nvar and not tvar and wvar>0]
    y,x = indices(jar.shape,int16)
    u = equal(jar,l)*equal(xyr,s)
    yu,xu = compress(u.ravel(),[y.ravel(),x.ravel()],1)
    ay0,ay1 = extrema(yu)
    ax0,ax1 = extrema(xu)
    ys = int(1024*(64.0/(ax1-ax0)))
    ns = (ay1-ay0)/ys + 1
    px = 2
    if ns > 1: px = 4
    py = ns/(px/2)
    if inter: px,py = 2,1
    if tvar and svar: px = px*3/2
    if inter: pgdev = "/xs"
    else: pgdev = Exposures[0].scratch("smow%d/wtrendc%d_%d%s.png" % (p,chip,L[1]-l,"xyr"[s-1]))+"/png"
    tstart = time.time()
    store = Exposures[0].scratch("smow%d/wtrendc%d_%d%s.fits" % (p,chip,L[1]-l,"xyr"[s-1]))
    for ks in range(ns):
       t = "L=%d | %s (%d/%d)" % (L[1]-l,"xyr"[s-1],ks+1,ns)
       print t,;sys.stdout.flush()
       y0 = ks*ys+ay0
       y1 = min([(ks+1)*ys+ay0,ay1])
       x0 = ax0
       x1 = ax1
       ne = len(Exposures)
       ny,nx = y1-y0+1,x1-x0+1
       W = zeros((ne,ny,nx),Float32)
       Wscs3d = zeros((ne,ny,nx),Float32)
       U = zeros((ne,ny,nx),Float32)
       for k,E in enumerate(Exposures):
#         if verb: print k,E,E.wfits2(chip)
         if E.cache:
            W[k] = E.lift[chip][0][y0:y0+ny,x0:x0+nx].astype(W.dtype)
            U[k] = E.lift[chip][1][y0:y0+ny,x0:x0+nx].astype(W.dtype)
         else:
            WF = FITS(E.wfits2(chip))
            W[k] = WF.readsec([x0,y0],[x0+nx-1,y0+ny-1]).astype(W.dtype)
            WF.close()
            UF = FITS(E.ufits2(chip))
            U[k] = UF.readsec([x0,y0],[x0+nx-1,y0+ny-1]).astype(W.dtype)
            UF.close()
         W[k] = divz(W[k],Wms[k]).astype(float32)
         U[k] = greater(U[k],0).astype(float32)
         if wvar:
#            U[k] = ones(U[k].shape,float32)
            U[k] = VTKAdd(VTKMultiply(U[k],wvar+1),wvar)
       Wmed = median(W.ravel())
       Wspan = 5*rms(W.ravel()-Wmed)
       if l: Wspan = Wspan*10
       sW = getsqrt(Wms/We)*0 + 1
       R = zeros(W.shape,float32)
       if nvar:
          sys.stdout.write("(S) "),;sys.stdout.flush()
          noplot = False
          if not ks:
             fstore = FITS(store,create=1)
             fstore.newhead("T",-32,3,[ax1-ax0+1,ay1-ay0+1,1])
          k1d = concatenate([ nvar*[1.0],[0.0],nvar*[1.0]])
          k2d = k1d[::,NewAxis] * concatenate([ nvar*[0.0],[1.0],nvar*[0.0]])[NewAxis,::]
          k2d = (k2d/sum(k2d.ravel())).astype(float32)
          for j in range(W.shape[2]):
              Wj = W[::,::,j].astype(float32)
              Uj = U[::,::,j].astype(float32)
              mWj = median(Wj)[NewAxis,::]
              sWj = 1.49*median(abs(Wj-mWj))
              if l:
                rWj = divz(1,power(3*sWj,2)+power(Wj-mWj,2))
                rWj = divz(rWj,median(rWj))
              else:
                rWj = 1
              Uj = (sW[::,NewAxis]*Uj*rWj).astype(float32)
              for mp in range(mpi):
                 WUj = VTKMultiply(Wj,Uj)
                 if l:
                    numer = VTKCorrelation(WUj,k2d,numret=1,mirror=1,wrap=0,constant=None)
                    denom = VTKCorrelation(Uj,k2d,numret=1,mirror=1,wrap=0,constant=None)
                 else:
                    numer = VTKCorrelation(WUj,k2d,numret=1,mirror=1,wrap=0,constant=None)
                    denom = VTKCorrelation(Uj,k2d,numret=1,mirror=1,wrap=0,constant=None)
                 Rj = VTKDivide(numer,denom)
                 R[::,::,j] = Rj
                 if ME and mp<mpi-1:
                    d = VTKSubtract(Wj,Rj)
                    sWr = 1.49*median(abs(d))
                    Uj = U[::,::,j].astype(float32)
                    Uj = (sW[::,NewAxis]*U[::,::,j]*rWj).astype(float32)
                    Uj = VTKMultiply(Uj,ME(divz(d,sWr[NewAxis,::]),w=1).astype(float32))
              U[::,::,j] = Uj.astype(U.dtype)
       elif pvar:
          sys.stdout.write("(P) "),;sys.stdout.flush()
          noplot = True
          R = W
          if not ks:
             fstore = FITS(store,create=1)
             fstore.newhead("T",-32,3,[ax1-ax0+1,ay1-ay0+1,len(R)])
          fstore.writesec(R,[x0-ax0,y0-ay0,0],[x0-ax0+nx-1,y0-ay0+ny-1,len(R)-1])
       elif not nvar and not tvar:
          sys.stdout.write("(M) "),;sys.stdout.flush()
          noplot = True
          R1 = U*sW[::,NewAxis,NewAxis]
          Wm = divz(add.reduce(R1*W,0),add.reduce(R1,0))
          R = Wm[NewAxis,::,::]*ones(W.shape)
          d = VTKSubtract(W,R)
          if not ks:
             fstore = FITS(store,create=1)
             fstore.newhead("T",-32,3,[ax1-ax0+1,ay1-ay0+1,1])
          fstore.writesec(R,[x0-ax0,y0-ay0,0],[x0-ax0+nx-1,y0-ay0+ny-1,0])
       elif tvar:
          noplot = False
          if tvar > 0: Nh = int(Hmax/tvar+1)
          else:
             nrate = 2*mean(diff(Wt))
             Nh = int(Hmax/ (abs(tvar)*nrate))
          Hr = zeros((2*Nh+1,)+Hx.shape,float64)
          Hr[0] = ones(Hx.shape,float64)
          for ih in range(1,Nh+1):
            Hr[2*ih-1] = cos(2*pi*ih*Hx)
            Hr[2*ih] = sin(2*pi*ih*Hx)
          if not ks:
             fstore = FITS(store,create=1)
             fstore.newhead("T",-32,3,[ax1-ax0+1,ay1-ay0+1,len(Hr)])
             fstore.modkey("tmin",Hmin)
             fstore.modkey("tspan",Hmax)
             fstore.modkey("Nh",Nh)
          sys.stdout.write("(F %.1f) "%(tvar)),;sys.stdout.flush()
          Pspec = zeros((Hr.shape[0],ny,nx),float64)
          Pspecu = zeros((Nh+1,ny,nx),float64)
          icuts = zeros((ny,nx),int16)
          Srout = zeros((len(Hr),ny,nx),float64)
          for my in range(ny):
             for mx in range(nx):
                 Wr = 1*W[::,my,mx]
                 uW = U[::,my,mx]*sW
                 cWr = compress(uW,Wr)
                 mWr = median(cWr)
                 sWr = 1.49*median(abs(cWr-mWr))
                 if l:
                   rWr = divz(1,sqrt(power(wvar*sWr,2)+power(Wr-mWr,2)))
                   rWr = divz(rWr,median(rWr))
                 else:
                   rWr = 1
                 uW = U[::,my,mx]*sW*rWr
                 NuW = len(nonzero(uW))
#                 print mx,my,len(Hr),NuW
#                 if NuW > len(Hr):
                 if 1:
                    v1 = rms(compress(uW,Wr))
                    for mp in range(mpi):
#                       print uW
#                       print rWr
                       du,dv,dw = singular_value_decomposition(transpose(Hr)*uW[::,NewAxis])
                       sol1 = transpose(dw)
                       if svar:
                          for icut in arange(Hr.shape[0]):
                             sol2 = divz(1,dv[:icut])
                             sol3 = dot(transpose(du[::,:icut]),Wr*uW)
                             Sr = dot(sol1[::,:icut],(sol2*sol3))
                             Er = dot(Sr,Hr)
                             d = Wr-Er
                             uu,ud = compress(uW,[uW,d],1)
                             if len(uu):
                                uu = uu/max(uu)
                                ud2 = divz(sum(uu*ud**2),(len(ud)-icut))
                             else: ud2 = 0.0
                             Pspec[icut,my,mx] = ud2
                          uPspec = Pspec[::,my,mx]
                          gcut = nonzero(greater(getlog10(uPspec),-90))
                          if len(gcut):
                             icut = gcut[argmin(take(uPspec,gcut))]+1
                          else:
                             icut = 1
                          icuts[my,mx] = icut
                       else:
                          icuts[my,mx] = min([Hr.shape[0],NuW])
                       sol2 = divz(1,dv[:icuts[my,mx]])
                       sol3 = dot(transpose(du[::,:icuts[my,mx]]),Wr*uW)
                       Sr = dot(sol1[::,:icuts[my,mx]],(sol2*sol3))
                       Er = dot(Sr,Hr)
                       d = Wr-Er
                       if ME and mp <mpi-1:
                          uW = U[::,my,mx]*sW*rWr
                          uW=uW*ME(divz(d,sWr),w=1)
                       R[::,my,mx] = Er
                    Srout[::,my,mx] = Sr
                 d = Wr-R[::,my,mx]
                 Nu = len(d)
                 vu = compress(uW,d)
                 NuW = len(vu)
                 v2 = rms(vu)
                 v3 = 1.49*median(abs(vu))
#                 if verb: print "%3d %3d %5.1f %3d %4d %4d %8.4e %8.4e" % (mx,my,v1,Nh,Nu,NuW,v2,v3)
          fstore.writesec(Srout,[x0-ax0,y0-ay0,0],[x0-ax0+nx-1,y0-ay0+ny-1,len(Srout)-1])

       print "write!"

       if noplot: pgdev = None

       if pgdev != None:
           print "plot!",pgdev
           if not ks:
              print "opening pgplot device",pgdev
              opendev(pgdev,nx=px,ny=py,land=0,ask=0,size=3.0*px,asp=1.*py/px)
           xm = [Wt0[0]-0.02*(Wt0[-1]-Wt0[0]),Wt0[-1]+0.02*(Wt0[-1]-Wt0[0])]
           if l: ym = [-Wspan/(max(L)-l+1),+Wspan/(max(L)-l+1)]
           else: ym = [Wmed-Wspan,Wmed+Wspan]
           if inter:
              for mx in range(nx):
                 for my in range(ny):
                    W1 = W[::,my,mx]
                    R1 = R[::,my,mx]
                    U1 = U[::,my,mx]
                    D1 = W1-R1
                    uW1,uR1,uU1,uD1,uWt = compress(U1,[ W1,R1,U1,D1,Wt0],1)
                    newbox(xm,ym,ch=2.0)
                    xlabel("time (min)",ch=2.0)
                    ylabel("W",ch=2.0)
                    subtitle(t,ch=2.0)
                    drawpt(Wt0,W1,ps=1,ci=2)
                    if len(uW1):
                      drawpt(uWt,uW1,ps=1,ci=7)
                    drawpt(Wt0,R1,ps=1,ci=4)
                    newbox(xm,[-Wspan/(max(L)-l+1),Wspan/(max(L)-l+1)],ch=2.0)
                    xlabel("time (min)",ch=2.0)
                    ylabel("\gDW",ch=2.0)
                    subtitle(t,ch=2.0)
                    drawpt(Wt0,D1,ps=1,ci=2)
                    if len(uD1):
                       drawpt(uWt,uD1,ps=1,ci=7)
                    raw_input(" <hit cr> ")
           else:
              newbox(xm,ym,ch=2.0)
              xlabel("time (min)",ch=2.0)
              ylabel("W",ch=2.0)
              subtitle(t,ch=2.0)
              medW = mean(W.ravel())
              rmsW = rms(W.ravel(),medW)
              rmsD = sum([sum(power((W[k]-R[k]).ravel(),2)) for k in Wx])
              rmsD = sqrt(rmsD/multiply.reduce(W.shape))
              for mx in range(nx):
                 for my in range(ny):
                    W1 = W[::,my,mx]
                    R1 = R[::,my,mx]
                    U1 = U[::,my,mx]
                    D1 = W1-R1
                    uW1,uR1,uU1,uD1,uWt = compress(U1,[ W1,R1,U1,D1,Wt0],1)
                    drawpt(Wt0,W1,ps=1,ci=2)
                    if len(uW1):
                       drawpt(uWt,uW1,ps=1,ci=7)
                    drawpt(Wt0,R1,ps=1,ci=4)

              newbox(xm,[-Wspan/(max(L)-l+1),Wspan/(max(L)-l+1)],ch=2.0)
              xlabel("time (min)",ch=2.0)
              ylabel("\gDW",ch=2.0)
              subtitle(t,ch=2.0)
              for mx in range(nx):
                 for my in range(ny):
                    W1 = W[::,my,mx]
                    R1 = R[::,my,mx]
                    U1 = U[::,my,mx]
                    D1 = W1-R1
                    uW1,uR1,uU1,uD1,uWt = compress(U1,[ W1,R1,U1,D1,Wt0],1)
                    drawpt(Wt0,D1,ps=1,ci=2)
                    if len(uD1):
                       drawpt(uWt,uD1,ps=1,ci=7)
              rxa = arange(xm[0]-10,xm[-1]+10,1,float32)
              drawln(rxa,rmsW*ones(rxa.shape),ls=2,ci=2)
              drawln(rxa,-rmsW*ones(rxa.shape),ls=2,ci=2)
              drawln(rxa,rmsD*ones(rxa.shape),ls=2,ci=4)
              drawln(rxa,-rmsD*ones(rxa.shape),ls=2,ci=4)

              if tvar and svar:
                 Px = arange(Hr.shape[0])
                 pxm = [Px[0]-0.01,Px[-1]+0.01]
                 Py = getlog10(Pspec.ravel())
                 Py = compress(greater(Py,-90),Py)
                 pymin,pymax = extrema(Py)
                 pym = [pymin-0.5,pymax+0.5]
                 newbox(pxm,pym,ch=2.0,ylog=1)
                 xlabel("k",ch=2.0)
                 ylabel("Scatter",ch=2.0)
                 subtitle(t,ch=2.0)
                 for mx in range(nx):
                    for my in range(ny):
                       P1 = getlog10(Pspec[::,my,mx])
                       sPx,sP1 = compress(greater(P1,-90),[Px,P1],1)
                       if len(sP1):
                          drawln(sPx,sP1,ls=1,ci=2)
                       icut = icuts[my,mx]
                       uPx,uP1 = Px[:icut],P1[:icut]
                       drawln(uPx,uP1,ls=1,ci=4)
    if pgdev:
       print "closing",pgdev
       closedev()
    fstore.close()
    telapse = time.time()-tstart
    print "Time elapsed for construction of %d (%s) was %.2f" % (max(L)-l,"xyr"[s-1],telapse)
    if not l: return Wspan,Wms

def ClearWaves(first=0,second=1):
    if first: ci = 1
    elif second: ci = 2
    [WDICT[ci].pop(k) for k in WDICT[ci].keys()]

def ReadWaves(p=1,chip=0,origdir="",nn=4,nt=4,clear=0,first=0,second=1):
  if first: ci = 1
  elif second: ci = 2
  if clear: ClearWaves(first=first,second=second)
  if chip not in WDICT[ci]:
     MakeJXY(nn=nn,nt=nt)
     L = JXY["L"]
     jar = JXY["jar"]
     xyr = JXY["xyr"]
     y,x = indices(jar.shape,int16)
     wpars = origdir + "/wavelets%d.pars" % (p)
     wpars = readtable(wpars,header=0,icols=[0,1,2,3])
     luse = sort(max(L)-compress(wpars["u"],wpars["l"]))
     wdict = {}
     wdict["a"] = {}
     wdict["b"] = {}
     for l in luse:
        nvar,tvar,wvar,svar = compress(equal(wpars["l"],L[1]-l),[wpars["n"],wpars["t"],wpars["w"],wpars["s"]],1)[::,0]
        pvar = [0,1][not nvar and not tvar and wvar>0]
        u = equal(jar,l).ravel()
        yl,xl,sl = compress(u,[y.ravel(),x.ravel(),xyr.ravel()],1)
        for s in [[3],[1,2,3]][l>0]:
          u = equal(sl,s)
          yu,xu,su = compress(u,[yl,xl,sl],1)
          ay0,ay1 = extrema(yu)
          ax0,ax1 = extrema(xu)
          wdict["a"][(l,s)] = [ay0,ay1],[ax0,ax1]
          print l,s, wdict["a"][(l,s)]
          store = origdir+"/smow%d/wtrendc%d_%d%s.fits" % (p,chip,L[1]-l,"xyr"[s-1])
          print "Reading",store
          fstore = FITS(store)
          if tvar:
            Hmin = fstore["tmin"]
            Hmax = fstore["tspan"]
            Nh = fstore["Nh"]
            wdict["b"][(l,s)] = [Hmin,Hmax,Nh,fstore.data()]
          else:
            wdict["b"][(l,s)] = fstore.data()
          fstore.close()
     WDICT[ci][chip] = wdict

def WaveInvert(E,chip,ii,stage,nn=4,nt=4,origdir=""):
  if stage == "1":
     first = 1
     second = 0
     ci = 1
  elif stage == "2":
     first = 0
     second = 1
     ci = 2
  ReadCalibs(E,chip,nfilt=E.filter,first=first,second=second)
  ci = eval(stage)
  n = eval("E.mfits"+stage)(chip)
  o = eval("E.sfits"+stage)(chip)
  oc = eval("E.cat"+stage)(chip)
  w = eval("E.qfits"+stage)(chip)
  m = eval("E.bfits"+stage)(chip)
  k = eval("E.rfits"+stage)(chip)
  e = eval("E.pers"+str(int(stage)-1))(chip)
  p = int(stage)
  MakeJXY(nn=nn,nt=nt)
  L = JXY["L"]
  jar = JXY["jar"]
  xyr = JXY["xyr"]
  ReadWaves(p=p,chip=chip,origdir=origdir,nn=nn,nt=nt)
  corners = WDICT[ci][chip]["a"]
  wdict = WDICT[ci][chip]["b"]

  wpars = "wavelets%d.pars" % (p)
  if origdir:
    wpars = origdir + "/" + wpars
  wpars = readtable(wpars,header=0,icols=[0,1,2,3])
  luse = sort(max(L)-compress(wpars["u"],wpars["l"]))

  print "Reading",k
  F = FITS(k)
  Wt = F["mjd-obs"]
  Wms = F["median"]
  We = F["exptime"]
  k1 = F.data()
  F.close()
  if Wt == "N/A": Wt = E.mjd
  Wt = Wt*24.0*60.0 + We/60.0/2.
#  F = FITS(w)
#  i1 = F.data()
#  F.close()
  i1 = zeros(k1.shape,float64)
  y,x = indices(jar.shape,int16)

  for l in luse:
     print l,;sys.stdout.flush()
     nvar,tvar,wvar,svar = compress(equal(wpars["l"],L[1]-l),[wpars["n"],wpars["t"],wpars["w"],wpars["s"]],1)[::,0]
     pvar = [0,1][not nvar and not tvar and wvar>0]
     for s in [[3],[1,2,3]][l>0]:
#       u = equal(jar,l)*equal(xyr,s)
#       yu,xu = compress(u.ravel(),[y.ravel(),x.ravel()],1)
#       ay0,ay1 = extrema(yu)
#       ax0,ax1 = extrema(xu)
       [ay0,ay1], [ax0,ax1] = corners[(l,s)]
       if pvar:
          i1[ay0:ay1+1,ax0:ax1+1] = wdict[(l,s)][ii]
       elif not nvar and not tvar:
          i1[ay0:ay1+1,ax0:ax1+1] = wdict[(l,s)]
       elif tvar:
          Hmin,Hmax,Nh,Sr = wdict[(l,s)]
          Hx = (Wt-Hmin)/Hmax
          Hr = zeros((2*Nh+1,),float64)
          Hr[0] = 1.0
          for ih in range(1,Nh+1):
             Hr[2*ih-1] = cos(2*pi*ih*Hx)
             Hr[2*ih] = sin(2*pi*ih*Hx)
          Er = zeros(Sr.shape[1:],float64)
          [add(Er,Sr[h]*Hr[h],Er) for h in range(len(Hr))]
          i1[ay0:ay1+1,ax0:ax1+1] = Er.astype(i1.dtype)
  print
  print "Writing",w
  if FileCheck(w): G = FITS(w,readonly=0)
  else: G = FITS(w,create=1,template=n)
  i1m = VTKMultiply(i1,Wms)
  G.putdata(i1m)
  G.close()
  MakeThumbnail(w,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
  MakeThumbnail(w,mag=16,pct=40,s1=-10,s2=10,inv=1,q=85)
  print "Writing",m
  if FileCheck(m): G = FITS(m,readonly=0)
  else: G = FITS(m,create=1,template=n)
  w = inverse_flwt2d( frmall(i1m,nn,nt),nn,nt).astype(float32)
  mask = MASKD[ci][chip]
  w = VTKMultiply(w,mask)
  G.putdata(w)
  G.close()
  MakeThumbnail(m,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
  MakeThumbnail(m,mag=16,pct=40,s1=-10,s2=10,inv=1,q=85)
  print "Reading",n
  F = FITS(n)
  i1 = F.data()
  F.close()
#  mim = FITS(origdir+"maskc%d.fits" % (chip))
#  mask = mim.data().astype(float32)
#  mim.close()
  r0 = VTKMultiply(VTKSubtract(k1,w),mask)
  r = VTKMultiply(VTKSubtract(i1,w),mask)
  if p > 1:
     print "Reading",e
     F = FITS(e)
     p1 = F.data()
     F.close()
     r = VTKMultiply(r, equal(p1,0).astype(float32))
  s = VTKMultiply(VTKMedianSmooth(VTKAbs(r),32,32,numret=1),1.49)
  ms = median(transpose(divz(r,s)))
#  msr = 1.49*median(abs(ms))
  r = r*greater(ms[::,NewAxis],-10)
  r0 = compress(r.ravel(),r0.ravel())
  anoise = 1.49*median(abs(r0))
  print "Writing",o
  if FileCheck(o): G = FITS(o,readonly=0)
  else: G = FITS(o,create=1,template=n)
  G.modkey("anoise",anoise)
  G.putdata(r)
  G.close()
  MakeThumbnail(o,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
  MakeThumbnail(o,mag=16,pct=40,s1=-10,s2=10,inv=1,q=85)
  Sextractor(o,oc,scale=E.scale,thresh=5,saturate=50e3)

def FinalizeFlats(targ,nfilt,chips,UseExposures,first=1,second=0):
   [ReadCalibs(UseExposures[0],chip,nfilt=nfilt,flat=True,first=first,second=second) for chip in chips]
#   os.chdir("..")
   if first:
      stage = 1
      ci = 1
   elif second:
      stage = 2
      ci = 2
   if "Dark" in nfilt:
      obj = UseExposures[0].objname
      exptime = mean([E.exptime for E in UseExposures])
      for chip in chips:
         F = FITS(targ+"/sky%d/Darkc%d.fits" % (stage,chip))
         i = F.data()
         F.close()
         i = where(between(i,-1e24,1e24),i,0.0)
         i = i/exptime
         r0 = getpct(i.ravel(),5)
         m = getpct(i.ravel(),50)
         r1 = getpct(i.ravel(),95)
         mask = between(i,m-20*(m-r0),m+20*(r1-m)).astype(float32)
#         Bi,u1 = MakeWaves(i,sig=3,mask=mask,nowave=1)
#         m = Bi*u1
#         d = i - divz(VTKMeanSmooth(i*m,256,256,1), VTKMeanSmooth(m,256,256,1))
#         r0 = getpct(d.ravel(),5)
#         m = getpct(d.ravel(),50)
#         r1 = getpct(d.ravel(),95)
#         mask = between(i,m-20*(m-r0),m+20*(r1-m)).astype(float32)
         i = i*mask
         n,ng = len(i.ravel()),len(compress(i.ravel(),i.ravel()))
         print chip,r0,m,r1,1.*ng/n
         name = targ+"/%sc%d.fits" % (obj,chip)
         F = FITS(name,create=1,template=targ+"/sky%d/Darkc%d.fits" % (stage,chip))
         F.modkey("exptime",1.0)
         F.putdata(i)
         F.close()
         MakeThumbnail(name,mag=1,pct=50,z1=0,z2=0,s1=-10,s2=10,inv=1)
         MakeThumbnail(name,mag=8,pct=50,z1=0,z2=0,s1=-10,s2=10,inv=1)
   if targ[-1:] != "B" and "Dark" not in nfilt:
      if ("flat" not in targ.lower() and "dome" not in targ.lower()): targ,tflat = "flat_"+targ,1
      else: tflat = 0
      print "Finalizing flats..."
      ii = []
      mi = []
      for chip in chips:
         F = FITS(targ+"/sky%d/Flat%sc%d.fits" % (stage,nfilt,chip))
         i = F.data()
         F.close()
         if targ[-1] == "A":
           F = FITS(targ[:-1]+"B/sky%d/Flat%sc%d.fits" % (stage,nfilt,chip))
           i0 = F.data()
           F.close()
           mi0x = median(i0)
           si0x = 1.49*median(abs(i0-mi0x[NewAxis,::]))
           si0x = si0x - VTKMedian(si0x,101,1,1)
           ri0x = 1.49*median(abs(si0x))
           bi0x = less(si0x,10*ri0x).astype(float32)
           mi0y = median(transpose(i0))
           si0y = 1.49*median(abs(transpose(i0)-mi0y[NewAxis,::]))
           si0y = si0y - VTKMedian(si0y,101,1,1)
           ri0y = 1.49*median(abs(si0y))
           bi0y = less(si0y,10*ri0y).astype(float32)
           okay = bi0x[NewAxis,::]*bi0y[::,NewAxis]
           i = where(okay,i - i0,0.0)
         m = median(i.ravel())
         ii.append(i)
         mi.append(m)
      norm = mean(mi)
      print "Normalizing with %.3f" % (norm)
      for i,chip in zip(ii,chips):
         i = i/norm
         F = FITS(targ+"/Flat"+nfilt+"c%d.fits" % (chip),create=1,template=targ+"/sky%d/Flat%sc%d.fits" % (stage,nfilt,chip))
         F.putdata(i)
         F.close()
         MakeThumbnail(targ+"/Flat"+nfilt+"c%d.fits" % (chip),mag=1,pct=50,z1=0,z2=0,s1=-10,s2=10,inv=1)
         MakeThumbnail(targ+"/Flat"+nfilt+"c%d.fits" % (chip),mag=8,pct=50,z1=0,z2=0,s1=-10,s2=10,inv=1)
         im = VTKMedianSmooth(i,128,2128,numret=1)
#         qdump("%sim0c%d.fits"%(nfilt,chip),i)
#         qdump("%sim1c%d.fits"%(nfilt,chip),im)
#         qdump("%sim2c%d.fits"%(nfilt,chip),divz(i,im))
         mask = 1.*between(divz(i,im),0.5,1.5)
         img = divz(VTKMedianSmooth(mask*i,128,128,1), VTKMeanSmooth(mask,128,128,1))
         imr = i-img
         ims = 1.49*divz(VTKMedianSmooth(mask*VTKAbs(imr),128,128,1), VTKMeanSmooth(mask,128,128,1))
         if UseExposures[0].fclass == "fourstar": simo = 20.0
         elif UseExposures[0].fclass == "newfirm": simo = 5.0
         imo = 1*greater(abs(divz(imr,ims)),simo)
         iml = VTKConvolve(1.*less(i/im,0.2),k=3,numret=1)
         imb = 1*logical_and(imo,logical_not(iml))
         imbx = add.reduce(imb,0)
         imby = add.reduce(imb,1)
         imbxm = median(imbx)
         imbxr = 1.49*median(abs(imbx-imbxm))
         imbym = median(imby)
         imbyr = 1.49*median(abs(imby-imbym))
         imu = 1.*logical_not(imo)
         print chip,imbxm,imbxr,
         print imbym,imbyr
         if UseExposures[0].fclass == "newfirm":
             imu = imu * less_equal(imbx,imbxm+4*simo*imbxr)[NewAxis,::]
             imu = imu * less_equal(imby,imbym+4*simo*imbyr)[::,NewAxis]
#         qdump(targ+"/imbx%d.fits"%(chip),imbx)
#         qdump(targ+"/imby%d.fits"%(chip),imby)
#         qdump(targ+"/imb%d.fits"%(chip),imb)
#         qdump(targ+"/ims%d.fits"%(chip),ims)
#         qdump(targ+"/imd%d.fits"%(chip),divz(imr,ims))
#         qdump(targ+"/imm%d.fits"%(chip),imu)
         mask = imu*greater(i,0.2*median(compress(imu.ravel(),i.ravel())))
         if chip in DARKD[ci]:
            dark = DARKD[ci][chip]
            if (UseExposures[0].noctim, UseExposures[0].ncoadd) in dark:
               dark = DARKD[ci][chip][(UseExposures[0].noctim, UseExposures[0].ncoadd)]
               print "Using dark to help make the mask..."
               mask = mask*not_equal(dark,0)
         G = FITS(targ+"/mask"+nfilt+"c%d.fits" % (chip),create=1,template=targ+"/Flat"+nfilt+"c%d.fits" % (chip))
         G.modkey("bitpix",16)
         G.putdata(mask)
         G.close()
         MakeThumbnail(targ+"/mask"+nfilt+"c%d.fits" % (chip),mag=1,pct=50,z1=0,z2=1,inv=1)
         MakeThumbnail(targ+"/mask"+nfilt+"c%d.fits" % (chip),mag=8,pct=50,z1=0,z2=1,inv=1)
         if 0:
            try: os.remove(targ+"/Flat"+nfilt+"c%d.fits" % (chip))
            except: pass
            try: os.remove(targ+"/mask"+nfilt+"c%d.fits" % (chip))
            except: pass
            if not tflat:
               try: os.remove(targ+"/Flat"+nfilt+"c%d.fits" % (chip),"Flat"+nfilt+"c%d.fits" % (chip))
               except: pass
               os.symlink(targ+"/Flat"+nfilt+"c%d.fits" % (chip),"Flat"+nfilt+"c%d.fits" % (chip))
               try: os.path.remove(targ+"/mask"+nfilt+"c%d.fits" % (chip),"mask"+nfilt+"c%d.fits" % (chip))
               except: pass
               os.symlink(targ+"/mask"+nfilt+"c%d.fits" % (chip),"mask"+nfilt+"c%d.fits" % (chip))

def GetUsingTimeStamps(m,fs):
    mjds = []
    for f in fs:
       d = os.path.dirname(f)
       mjds.append( float(d.split("_")[-1][1:]))
#       print f,d,mjds[-1]
    mjds = asarray(mjds)
    return fs[argmin(abs(m-mjds))]

def ClearCalibs(first=1,second=0):
    if first: ci = 1
    elif second: ci = 2
    [[D[ci].pop(k) for k in D[ci].keys()] \
            for D in [MASKD,MASKN,FLATD,FLATN,DARKD,DARKN,SKYFD,SKYFN]]

def ReadCalibs(E,chip,nfilt="J",flat=0,first=1,second=0):
    cwd = os.getcwd()
    if first: ci = 1
    elif second: ci = 2
#    print "Checking for",os.getcwd(),"in",MASKD.keys()
#    for D in [MASKD[ci],MASKN[ci],FLATD[ci],FLATN[ci],DARKD[ci],DARKN[ci]]:
#       if os.getcwd() not in D:
#          [D.pop(k) for k in D.keys()]
#          D.setdefault(os.getcwd())

    if E.fclass == "fourstar":  gm = E.gain[0]
    else: gm = ""

    maskf = None
    maskfs = glob("flat_obj%s_%s*/mask%sc%d.fits" % (nfilt,gm,nfilt,chip))
    if not maskfs:
       if gm == "L": ngm = "F"
       elif gm == "F": ngm = "L"
       maskfs = glob("flat_obj%s_%s*/mask%sc%d.fits" % (nfilt,ngm,nfilt,chip))
    if maskfs:
       maskf = GetUsingTimeStamps(E.mjd,maskfs)

    if not flat and maskf and os.path.exists(maskf):
       if chip not in MASKD[ci] or MASKN[ci][chip] != maskf:
          print "Reading",maskf
          F = FITS(maskf)
          MASKD[ci][chip] = F.data()
          MASKN[ci][chip] = os.path.realpath(maskf)
          F.close()
    else:
       MASKD[ci][chip] = None
       MASKN[ci][chip] = "None"

    if nfilt != "Dark":
       if chip not in DARKD[ci]:
          DARKD[ci][chip] = {}
          DARKN[ci][chip] = {}
#       if (not flat or os.path.basename(os.getcwd())[:8] == "flat_obj") and nfilt != "Dark":
       if not flat or E.fclass == "fourstar":
          darks = glob("dark*_%s*/dark*c%d.fits" % (gm,chip))
          if darks:
             darkc = [os.path.basename(dark).split("_")[0].replace("dark","") for dark in darks]
             darkt = array([float(dark.split("x")[0]) for dark in darkc])
             darkn = array([int(dark.split("x")[1].split("c")[0]) for dark in darkc])
#             print darkc, "%fx%d" % (E.noctim,E.ncoadd)
#             exit()
             dt = abs(darkt-E.noctim)
             dn = abs(darkn-E.ncoadd)
             dnt = (dt**2+dn**2)
             if min(dnt) == 0:
                k = argmin(dt**2+dn**2)
                darkf = darks[k]
                if os.path.exists(darkf):
                   if DARKD[ci][chip].get((E.noctim,E.ncoadd),None) == None or \
                           DARKN[ci][chip].get((E.noctim,E.ncoadd),None) != darkf:
                      print "Reading",darkf
                      F = FITS(darkf)
                      dd = F.data()
                      DARKD[ci][chip][E.noctim,E.ncoadd] = where(between(dd,-1e24,1e24), dd, 0)
                      DARKN[ci][chip][E.noctim,E.ncoadd] = darkf
                      F.close()
             else:
                DARKN[ci][chip][E.noctim,E.ncoadd] = "None"

    flatf = None
    flatfs = glob("flat_obj%s_%s*/Flat%sc%d.fits" % (nfilt,gm,nfilt,chip))
    ngm = gm
    if not flatfs:
       if gm == "L": ngm = "F"
       elif gm == "F": ngm = "L"
       flatfs = glob("flat_obj%s_%s*/Flat%sc%d.fits" % (nfilt,ngm,nfilt,chip))
    if flatfs:
       flatf = GetUsingTimeStamps(E.mjd,flatfs)
    if not flat and flatf and os.path.exists(flatf):
       if chip not in FLATD[ci] or FLATN[ci][chip] != flatf:
          print "Reading",flatf
          F = FITS(flatf)
          Fd = F.data()
          if E.fclass == "fourstar" and gm != ngm:
             if gm == "F":
                gfix = (FSFullWellGains[chip-1] / FSLowNoiseGains[chip-1]) * (mean(FSLowNoiseGains) / mean(FSFullWellGains))
             elif gm == "L":                                             
                gfix = (FSLowNoiseGains[chip-1] / FSFullWellGains[chip-1]) * (mean(FSFullWellGains) / mean(FSLowNoiseGains))
             Fd = gfix * Fd
          FLATD[ci][chip] = where(between(Fd,-1e24,1e24), Fd, 0)
          FLATN[ci][chip] = os.path.realpath(flatf)
          F.close()
          illumf = E.scratch("Illum%sc%d.fits" % (nfilt,chip))
          if os.path.exists(illumf):
             print "Reading",illumf
             F = FITS(illumf)
             FLATD[ci][chip] = FLATD[ci][chip] * F.data()
             F.close()

    else:
       FLATD[ci][chip] = None
       FLATN[ci][chip] = "None"

class CIRCERun:
   def __init__(self,ddir,nfilt,targ,chips,fclass,NP=0,HUB=None,ssh=0,\
                first=0,second=0,verb=0,inter=0,\
                mag=5,histo=0,msh=0,nume=3,nmax=60,nf=10,nn=4,nt=4,check=0,\
                flat=0,lift=0,freeze=0,calib=0,show=0,noshift=0,\
                start=0,end=0,sigclip=5.0,yskip=16,skyskip=32,st=0.05):
        self.fclass = fclass
        self.paused = True
        self.manhole = None
        self.success = 0
        self.ongoing = False
        self.progress = 0
        self.current = 0
        self.NP=NP
        self.HUB=HUB
        self.ssh=ssh
        self.basedir = os.getcwd()
        self.nfilt = nfilt
        self.targ = targ
        zdir = ddir.replace("/"," ").strip().split(" ")[-1]
        self.zodb = "zodb/%sinfo.fs" % (zdir)
        if not os.path.exists(os.path.dirname(self.zodb)):
           print "Making",os.path.dirname(self.zodb)
           os.mkdir(os.path.dirname(self.zodb))
        self.chips=chips
        self.first=first
        self.second=second
        self.verb=verb
        self.inter=inter
        self.mag=mag
        self.histo=histo
        self.nume = nume
        self.msh=msh
        self.nmax=nmax
        self.nf=nf
        self.nn=nn
        self.nt=nt
        self.check=check
        self.flat=flat
        self.lift=lift
        self.freeze=freeze
        self.calib=calib
        self.show=show
        self.noshift=noshift
        self.start=start
        self.end=end
        self.sigclip=sigclip
        self.yskip=yskip
        self.skyskip=skyskip
        self.st=st
        self.dark = False
        self.donefirst = False
        self.donesecond = False
        self.errfirst = False
        self.errsecond = False
        if "flats" in self.targ:
           self.flat = True
        if "Dark" in self.nfilt:
           self.flat = True
           self.dark = True
        if self.first:
           self.stage = "1"
        if self.second:
           self.stage = "2"

   def Already(self,verb=0,first=-1,second=-1):
#       cwd = os.getcwd()
#       os.chdir(self.basedir)
       if verb:
          print "Already?",self.targ
       retval = 0
       if first: stage = "1"
       elif second: stage = "2"
       if self.errfirst and not os.path.exists("%s.err" % (self.targ)):
          self.errfirst = False
          self.donefirst = False
       if self.errsecond and not os.path.exists("%s.err" % (self.targ)):
          self.errsecond = False
          self.donesecond = True
       if os.path.exists("bad_%s" % (self.targ)):
          if first: self.errfirst = True
          elif second: self.errsecond = True
          retval = True
       if os.path.exists("%s.err" % (self.targ)):
          if first: self.errfirst = True
          elif second: self.errsecond = True
          retval = True
       if (first and self.donefirst) or (second and self.donesecond): retval = True
#       if "obj021824-050800_J1_L55547_2457_2528" in self.targ:
#           print "Already",first,second,retval
       if not retval:
          if self.flat:
             if "dark" in self.targ:
                t = [c for c in self.chips if \
                   FileCheck(self.basedir+"/"+self.targ+"/"+self.targ+"c%d.fits" % (c)) \
                   ] == self.chips
             elif self.targ == "flat%sB" % (self.nfilt):
                t = [c for c in self.chips if \
                   FileCheck(self.basedir+"/"+self.targ+"/sky1/Flat"+self.nfilt+"c%d.fits" % (c)) \
                   ] == self.chips
             elif self.targ == "flat%sA" % (self.nfilt) or  "flat_obj%s" % (self.nfilt) in self.targ:
                t = [c for c in self.chips if \
                   (FileCheck(self.basedir+"/"+self.targ+"/mask"+self.nfilt+"c%d.fits" % (c)) and \
                   FileCheck(self.basedir+"/"+self.targ+"/Flat"+self.nfilt+"c%d.fits" % (c))) \
                   ] == self.chips
             elif "sky" in self.targ:
                t = [c for c in self.chips if \
                   FileCheck(self.basedir+"/"+self.targ+"/Stack%sc%dmag01.jpg" % (stage,c)) \
                   ] == self.chips
             else:
                t = [c for c in self.chips if \
                   (FileCheck(self.basedir+"/"+"flat_"+self.targ+"/mask"+self.nfilt+"c%d.fits" % (c)) and \
                   FileCheck(self.basedir+"/"+"flat_"+self.targ+"/Flat"+self.nfilt+"c%d.fits" % (c))) \
                   ] == self.chips
#                if "obj021824-050800_J1_L55547_2457_2528" in self.targ:
#                   print "Already Inside",first,second,t
          else:
             t = ([c for c in self.chips if FileCheck(self.basedir+"/"+self.targ+"/Stack"+stage+"c%dmag01.jpg"%(c))] == self.chips) \
                     and FileCheck(self.basedir+"/"+self.targ+"/Mosaic"+stage+".pdf")
#             if "obj021824-050800_J1_L55547_2457_2528" in self.targ: print t
          if t and first: self.donefirst = True
          elif t and second: self.donesecond = True
          retval = t
#       if "obj021824-050800_J1_L55547_2457_2528" in self.targ:
#          print "Already Outside",self.targ,retval
#          print first,self.donefirst,second,self.donesecond
       if retval:
          if (first and self.first) or (second and self.second):
             self.current = 8
             self.success = 1
       return retval

   def Ready(self):
#       os.chdir(self.basedir)
       if self.flat:  targ = "flat_"+self.targ
       else: targ = self.targ
       if self.second and not FileCheck(self.basedir+"/"+targ+"/Mosaic1.cat"):
          print "Not ready yet!",os.getcwd()
          raise IOError,"File not found: "+self.basedir+"/"+targ+"/Mosaic1.cat"

   def Execute(self,ttasks,items,serial=False):
       while self.paused:
           print "Waiting to be unpaused"
           time.sleep(1)
       if self.HUB and isinstance(ttasks,ListType):
          print "Have hub"
          mhost,mport = self.HUB.split(":")
          mport = int(mport)
          print "hub host is %s, port %d" % (mhost,mport)
          [item.setFromModules(["CIRCEProcess"]) for item in ttasks]
#          func,arg = "time.sleep",1
#          ttasks = [TwistedTask(-1,func,arg) for j in range(len(ttasks))]
          nin = len(ttasks)
          namespace = globals()
          if serial:
             print "Evaluating in serial"
             [setattr(ttask,"path",self.basedir) for ttask in ttasks]
             results = [EvalTask(ttask) for ttask in ttasks]
#             [reactor.callInThread(EvalTask,ttask) for ttask in ttasks]
#             while len([ttask for ttask in ttasks if hasattr(ttask,"result")]) < len(ttasks):
#                   time.sleep(0.1)
          else:
             print "Dispatching them to the hub"
             self.dispatch = \
                     HubDispatch(ttasks,(mhost,mport),\
                   mport=self.ssh+self.second,namespace=namespace,verb=self.verb,sleeptime=0.01)
             nout = len([t for t in self.dispatch.completed if t.result[5]])
             fails = [t.result[3] for t in self.dispatch.completed if not t.result[5]]
             assert fails == [], fails[0]
       elif self.NP and not serial:
#          if not self.HUB: [ReadCalibs(self.UseExposures[0],chip,nfilt=self.nfilt,flat=self.flat,first=self.first,second=self.second) for chip in self.chips]
          if isinstance(ttasks,ListType):
             [item.setFromModules(["CIRCEProcess"]) for item in ttasks]
             RunForkedQueue(EvalTask,ttasks,MaxThreads=self.NP,monitor=0,sleeptime=self.st,ThreadOutput=True)
          else:
             RunForkedQueue(ttasks,items,MaxThreads=self.NP,monitor=0,sleeptime=self.st,ThreadOutput=True)
       else:
          print "Not using hub"
#          if not self.HUB: [ReadCalibs(self.UseExposures[0],chip,nfilt=self.nfilt,flat=self.flat,first=self.first,second=self.second) for chip in self.chips]
          if isinstance(ttasks,ListType) and isinstance(ttasks[0],TwistedTask):
             [setattr(ttask,"path",self.basedir) for ttask in ttasks]
             results = [EvalTask(ttask) for ttask in ttasks]
          else:
             map(ttasks,items)

   def Error(self,result):
       logging.err(result)
       e = open(self.targ+".err","w")
       result.printTraceback(e)
       e.close()
       return self.ShutUp(result)

   def ShutUp(self,result=None):
       self.ongoing = False
       print "Turn off the manhole?"
       if self.ssh and self.manhole and hasattr(self.manhole,"stopListening"):
          print "Turn off the manhole!"
          stop = self.manhole.stopListening()
          return stop
       else:
          return True

   def WhatUp(self,*args):
       print args

   def Manhole(self):
       print "Turn on the manhole?"
       if self.ssh:
          namespace = globals()
          namespace["self"] = self
          print "Turn on the manhole!"
          self.manhole = AddManhole(getpass.getuser(),"",namespace,self.ssh)
          return self.manhole
       else:
          return True

   def FirstPass(self,d=None):
       self.progress = 1
       self.current = 0
       if d==None: self.chain1 = defer.Deferred()
       else: self.chain1 = d
       print "Already?",self.Already(first=self.first,second=self.second)
       if self.Already(first=self.first,second=self.second):
          self.donefirst = True
       else:
          self.chain1.addCallback(lambda result: self.GetExposures())
          self.chain1.addCallback(lambda result: self.Stage0())
          self.chain1.addCallback(lambda result: self.Stage1())
          self.chain1.addCallback(lambda result: self.Stage2())
          if not self.flat:
             self.chain1.addCallback(lambda result: self.Stage3())
             self.chain1.addCallback(lambda result: self.Stage4())
             self.chain1.addCallback(lambda result: self.Stage5())
             if 1 or not "sky" in self.targ:
                self.chain1.addCallback(lambda result: self.Stage6())
                self.chain1.addCallback(lambda result: self.Stage7())
          self.chain1.addCallback(lambda result: self.Stage0())
          self.chain1.addCallback(lambda result: self.ShutUp())
          self.chain1.addErrback(self.Error)
       return self.chain1

   def SecondPass(self,d=None):
       self.progress = 2
       self.current = 0
       if d==None: self.chain2 = defer.Deferred()
       else: self.chain2 = d
       if self.Already(first=self.first,second=self.second):
          self.donesecond = True
       else:
          self.chain2.addCallback(lambda result: self.GetExposures())
          self.chain2.addCallback(lambda result: self.Ready())
          self.chain2.addCallback(lambda result: self.GetExposures())
          self.chain2.addCallback(lambda result: self.Stage0())
          self.chain2.addCallback(lambda result: self.Stage1())
          self.chain2.addCallback(lambda result: self.Stage2())
          if not self.flat:
             self.chain2.addCallback(lambda result: self.Stage3())
             self.chain2.addCallback(lambda result: self.Stage4())
             self.chain2.addCallback(lambda result: self.Stage5())
             if 1 or not "sky" in self.targ:
                self.chain2.addCallback(lambda result: self.Stage6())
                self.chain2.addCallback(lambda result: self.Stage7())
          self.chain2.addCallback(lambda result: self.Stage0())
          self.chain2.addCallback(lambda result: self.ShutUp())
          self.chain2.addErrback(self.Error)
       return self.chain2

   def GetExposures(self):
       self.ongoing = True
       print "RESETTING DIRECTORY TO",self.basedir
       storage = ZODB.FileStorage.FileStorage(self.basedir+"/"+self.zodb,read_only=True)
       zdb = ZODB.DB( storage)
       conn = zdb.open()
       dbroot = conn.root()
       tree = dbroot["Data"]

       CIRCEDataset = IRSetup(tree,self.targ,self.nfilt,flat=self.flat,nume=self.nume)
       if not CIRCEDataset:
          raise NameError,"Target %s not found" % (self.targ)

       self.AllExposures = [Exposure(N) for N in CIRCEDataset]
       if self.flat and "flat" not in self.targ and "Dark" not in self.nfilt:
         for E in self.AllExposures:
            E.objname = "flat_"+E.objname
       transaction.abort()
       conn.close()
       zdb.close()
       storage.close()

       if self.end > self.start:
          self.AllExposures = [E for E in self.AllExposures if (self.start <= E.expno <= self.end)]
       [E.Bad() for E in self.AllExposures]
       self.AllExposures = [E for E in self.AllExposures if not E.bad]

       if self.show: os._exit(0)

       if self.freeze:
          self.AllExposures = [E for E in self.AllExposures if os.path.exists(E.mfits1(self.chips[0]))]
       self.Check()

   def Clear(self):
       [[E.data.pop(chip) for chip in E.data.keys()] for E in self.UseExposures]
       [[E.sdata.pop(chip) for chip in E.data.keys()] for E in self.UseExposures]
       [[E.masks.pop(chip) for chip in E.masks.keys()] for E in self.UseExposures]

   def Cull(self):
       [E.Bad() for E in self.AllExposures]
       self.UseExposures = [E for E in self.AllExposures if not E.bad]

   def Check(self):
       self.Cull()
       if self.HUB:
         self.caching = False
       else:
         self.caching = len(self.UseExposures) < self.nmax
       print "Caching",self.caching
       [setattr(E,"cache",self.caching) for E in self.UseExposures]
       setattr(self,"cats",{})

   def Stage0(self):
       self.current = 0
       ttasks1,items1 = self.Stage0Tasks()
       self.Execute(ttasks1,items1)

   def Stage0Tasks(self):
        ttasks1 = [TwistedTask(-1,"lambda first=0,second=0: (ClearCalibs(first=first,second=second),ClearWaves(first=first,second=second))",\
                first=self.first,second=self.second)]
        ttasks1[0].setFromModules(["CIRCEProcess"])
        ttasks1[0].all = True
        return ttasks1,[]

   def Stage1(self):
       self.current = 1
       [self.cats.setdefault(chip,None) for chip in self.chips]
       ttasks,items = self.Stage1Tasks()
       if ttasks:
          self.Execute(ttasks,items)
          self.newmode = True

       if "Dark" not in self.nfilt and not self.flat:
          if len(self.chips) == 1: skyplot = self.UseExposures[0].scratch("skyplot%sc%d.ps" % (self.stage,self.chips[0]))
          else: skyplot = self.UseExposures[0].scratch("skyplot%s.ps" % (self.stage))
          if self.newmode or not FileCheck(skyplot):
             print "Check consistency of the medians..."
             PlotModes(self.UseExposures,self.chips,first=self.first,second=self.second,flat=self.flat)
          [E.Bad() for E in self.UseExposures]

   def Stage1Tasks(self):
       print "Pulling the individual frames out, doing quick stats..."
       Exposures = self.UseExposures
       easydir(Exposures[0].scratch("mode"+self.stage))
       fsky = array([e.sky for e in Exposures])
       bab = not self.flat and (sum(fsky) and (sum(fsky) < len(fsky)))
       if not self.first:
          easydir(Exposures[0].scratch("mask"+str(int(self.stage)-1)))
          easydir(Exposures[0].scratch("pers"+str(int(self.stage)-1)))

       self.newmode = False
       items = []
       coll0 = {}
       coll = {}
       for chip in self.chips:
          if chip not in coll:
             coll0[chip] = []
             coll[chip] = []
          for j,E in enumerate(self.UseExposures):
              coll0[chip].append(E)
              if (not bab or bab and not E.sky) and not FileCheck(eval("E.mfits"+self.stage)(chip)):
                 item = [E,chip]
                 loopno = E.loopno
                 coll[chip].append(E)
                 if j>=loopno and not self.flat:
                     item.append(self.UseExposures[(j-loopno)])
                 elif j<(len(self.UseExposures)-loopno) and not self.flat:
                     item.append(self.UseExposures[(j+loopno)])
                 else:
                     item.append(None)
                 items.append(item)
                 if self.second and not self.cats.get(chip,None):
                    self.GetStackCat(chip)

       if items:
          if self.HUB:
             ttasks=[TwistedTask(-1,"lambda *args,**kw: args[0].Mode(*args[1:],**kw)",\
                Echip[0],Echip[1],other=Echip[2],first=self.first,second=self.second,msh=self.msh,\
                cat=self.cats[Echip[1]],histo=self.histo,mag=self.mag,\
                flat=self.flat,verb=self.verb) \
                for j,Echip in enumerate(items)]
          else:
             ttasks=lambda Echip: Echip[0].Mode(Echip[1],other=Echip[2],\
                first=self.first,second=self.second,msh=self.msh,\
                cat=self.cats[Echip[1]],histo=self.histo,mag=self.mag,\
                flat=self.flat,verb=self.verb)
       else:
          ttasks=None

       return ttasks,items

   def GetStackCat(self,chip):
       print "Getting catalog for the mask making..."
       catfile=self.UseExposures[0].scratch("Stack1c%d.cat")
       catfits = FITS(catfile.replace(".cat",".fits") % (chip))
       catnaxes = catfits.naxes()
       catfits.close()
       Xz,Yz,Xs,Ys = Offsets(self.UseExposures,chip)
       self.cats[chip] = {"Xz":Xz,"Yz":Yz,"Xs":Xs,"Ys":Ys,"cat":catfile % ( chip),"exp":self.UseExposures,"naxes":catnaxes}

   def Stage2(self):
       self.current = 2
#       self.Cull()
       if self.lift:
          ttasks,items = self.Stage2TasksWaveletTransforms()
          self.Execute(ttasks,items)
          self.Clear()
       else:
          ttasks1,items1,ttasks2,items2 = self.Stage2TasksRunning()
          if items1 or items2:
             if not self.HUB:
                self.Execute(ttasks1,items1,serial=1)
             else:
                items1 = map(ttasks1,items1)
                ttasks1 = []
                [[ttasks1.append(i) for i in item[0]] for item in items1 if item]
                N = len(ttasks1)
                Nd = []
                while len(Nd) < N: [Nd.append(k) for k in randint(0,N,N) if k not in Nd]
                print Nd
                ttasks1 = [ttasks1[j] for j in Nd]
                self.Execute(ttasks1,items1)
                tempframes = [item[1] for item in items1 if item]
                skyframes = [item[2] for item in items1 if item]
                print tempframes
                print skyframes
                ttasks1a = [TwistedTask(-1,"Assemble",t1) for t1 in tempframes]
                self.Execute(ttasks1a,tempframes,serial=not self.HUB)
                ttasks1b = [TwistedTask(-1,"os.rename",item[1],item[2]) for item in items1 if item]
                self.Execute(ttasks1b,items1,serial=not self.HUB)
                ttasks1c = [TwistedTask(-1,"MakeThumbnail",item[2],mag=1,pct=40,s1=-10,s2=10,inv=1,q=85) for item in items1 if item]
                self.Execute(ttasks1c,items1,serial=not self.HUB)
                ttasks1d = [TwistedTask(-1,"MakeThumbnail",item[2],mag=16,pct=40,s1=-10,s2=10,inv=1,q=85) for item in items1 if item]
                self.Execute(ttasks1d,items1,serial=not self.HUB)
             if ttasks2:
                self.Execute(ttasks2,items2)
             self.Clear()

       if self.lift:
          ttasks2,items2 = self.Stage2TasksWaveletAnalysis()
          if ttasks2:
             atasks2 = [ttask for ttask,item in zip(ttasks2,items2) if item[1] == 0]
             aitems2 = [item for ttask,item in zip(ttasks2,items2) if item[1] == 0]
             self.Execute(atasks2,aitems2)
             btasks2 = [ttask for ttask,item in zip(ttasks2,items2) if item[1] != 0]
             bitems2 = [item for ttask,item in zip(ttasks2,items2) if item[1] != 0]
             self.Execute(btasks2,bitems2)
          ttasks3,items3 = self.Stage2TasksWaveletInvert()
          if ttasks3:
             self.Execute(ttasks3,items3)
#       else:
#          if ttasks2:
#             self.Execute(ttasks2,items2)

       if self.flat and ((self.targ[:8] == "flat_obj" and self.first) or ("flat" in self.targ and self.first) or \
                       self.nfilt == "Dark"):
          if self.flat and "dark" in self.targ: kind = self.targ
          else: kind = "Flat"
          if [chip for chip in self.chips if \
               ((kind == "Flat" and not FileCheck("mask"+self.nfilt+"c%d.fits" % (chip))) \
               or not FileCheck("%s%sc%d.fits" % (kind,self.nfilt,chip)))]:
             FinalizeFlats(self.targ,self.nfilt,self.chips,self.UseExposures,first=self.first,second=self.second)
#          os._exit(0)

   def Stage2TasksRunning(self):
       items = []
       awaits = []
       skyframes = []
       nes = []
       ne30s = []
       Exposures = self.UseExposures
       fsky = array([e.sky for e in Exposures])
       bab = not self.flat and (sum(fsky) and (sum(fsky) < len(fsky)))
       print sum(fsky),len(fsky),bab
       for chip in self.chips:
           for E in self.UseExposures:
               if self.flat and "dark" in self.targ: kind = self.targ
               else: kind = "Flat"
               if (self.first and not os.path.exists(eval("E.sfits"+self.stage)(chip))) \
                       or (self.second and E == self.UseExposures[0] and \
                        not FileCheck("sky%s/%s%sc%d.fits" % (self.nfilt,kind,self.stage,chip))):
                   if not bab or not E.sky:
                      items.append((E,chip))
                      await,skyframe,ne,ne30 = E.SkySubtract(chip,self.UseExposures,first=self.first,second=self.second,\
                         flat=self.flat,nf=self.nf,test=1)
                      awaits.append(await)
                      skyframes.append(skyframe)
                      nes.append(ne)
                      ne30s.append(ne30)

#       print items
#       print nes
#       print ne30s
#       print [os.path.basename(s) for s in skyframes]
#       print set([os.path.basename(s) for s in skyframes])
#       print awaits
#       print [os.path.basename(s) for a,s in zip(awaits,skyframes) if a]
#       print [os.path.basename(s) for a,s in zip(awaits,skyframes) if not a]
#       print
#       os._exit(1)

       sitem = [s for a,s in zip(awaits,items) if not a]
       frame = [s for a,s in zip(awaits,skyframes) if not a]
       s = {}
       for j in range(len(awaits)):
          if not awaits[j]:
             if skyframes[j] in s: awaits[j] = True
             else: s.setdefault(skyframes[j])

#       print sitem
#       print skyframes
#       print awaits
#       print items
#       print s
       if items:
          bigsky = logical_not(awaits)
          items1 = [items[j] for j in nonzero(bigsky)]
#          print
#          print items1
#          print
#          os._exit(1)

          if self.HUB:
             ttasks1 = lambda Echip: Echip[0].SkySubtract(Echip[1],self.UseExposures,\
                first=self.first,second=self.second,flat=self.flat,nf=self.nf,verb=1,\
                yskip=self.skyskip/(len(self.UseExposures)/80+1),sigclip=self.sigclip,\
                Hub=self.HUB,bigsky=1,NP=0)
          else:
             ttasks1 = lambda Echip: Echip[0].SkySubtract(Echip[1],self.UseExposures,\
                first=self.first,second=self.second,flat=self.flat,nf=self.nf,verb=1,\
                yskip=self.skyskip/(len(self.UseExposures)/80+1),sigclip=self.sigclip,\
                Hub=None,bigsky=1,NP=self.NP)

          if self.flat and "dark" in self.targ: items2 = []
          elif self.flat and "flat" in self.targ: items2 = []
          else: items2 = items
#          print items2

          if self.HUB:
             ttasks2=[TwistedTask(-1,"lambda *args,**kw: args[0].SkySubtract(*args[1:],**kw)",\
                Echip[0],Echip[1],self.UseExposures,already=1,\
                first=self.first,second=self.second,flat=self.flat,nf=self.nf,verb=1,\
                yskip=self.skyskip/(len(self.UseExposures)/80+1),sigclip=self.sigclip,NP=0) \
                for j,Echip in enumerate(items2)]
          else:
             ttasks2=lambda Echip: Echip[0].SkySubtract(Echip[1],self.UseExposures,\
                already=1,first=self.first,second=self.second,flat=self.flat,nf=self.nf,verb=1,\
                yskip=self.skyskip/(len(self.UseExposures)/80+1),sigclip=self.sigclip)
       else:
          ttasks1=None
          ttasks2=None
          items1=None
          items2=None

       return ttasks1,items1,ttasks2,items2

   def Stage2TasksWaveletTransforms(self):
       self.newwaves = False
       items = []
       Exposures = [E for E in self.UseExposures if not E.bad]
       for chip in self.chips:
           for E in self.UseExposures:
             if not FileCheck(eval("E.wfits"+self.stage)(chip)) \
                    and (not self.flat or (self.flat and E == self.UseExposures[0])):
                   items.append((E,chip))
                   self.newwaves = True

       if items:
          MakeJXY(nn=self.nn,nt=self.nt)
          if self.HUB:
             ttasks=[TwistedTask(-1,"lambda *args,**kw: args[0].flwt2d(*args[1:],**kw)",\
                Echip[0],Echip[1],Exposures,first=self.first,second=self.second,verb=self.verb) \
                for j,Echip in enumerate(items)]
          else:
             ttasks = lambda Echip: Echip[0].flwt2d(Echip[1],Exposures,\
                first=self.first,second=self.second,verb=self.verb)
       else:
          ttasks=None

       return ttasks,items

   def Stage2TasksWaveletAnalysis(self):
       Exposures = [E for E in self.UseExposures if not E.bad]
       MakeJXY(nn=self.nn,nt=self.nt)
       self.newcons = False
       items = []
       for chip in self.chips:
           if self.newwaves or len([g for g in glob(Exposures[0].scratch("smow"+self.stage+"/wtrendc"+str(chip)+"*.fits")) if FileCheck(g)]) < 28:
              self.newcons = True
              L = JXY["L"]
              wpars = "wavelets%s.pars" % (self.stage)
              wpars = readtable(wpars,header=0,icols=[0,1,2,3])
              luse = sort(max(L)-compress(wpars["u"],wpars["l"]))

              if not self.inter:
                 luse = luse[::-1]

              wxy = [(l,s) for l in luse for s in [[3],[1,2,3]][l>0]]
              print "Full set:"
              print wxy
              wxy = [(l,s) for (l,s) in wxy if not FileCheck(Exposures[0].scratch("smow%s/wtrendc%d_%d%s.fits" % (self.stage,chip,L[1]-l,"xyr"[s-1])))]
              print "Left to do:"
              print wxy

              if wxy:
                 items = items + [(chip,)+ls for ls in wxy]

       if items:
          if self.HUB:
             ttasks=[TwistedTask(-1,"WaveAnalyze",\
                     Exposures,lsc[0],self.first,self.second,lsc[1],lsc[2],\
                      inter=self.inter,nn=self.nn,nt=self.nt,verb=1) \
                      for j,lsc in enumerate(items)]
          else:
             ttasks = lambda lsc: WaveAnalyze(Exposures,lsc[0],self.first,self.second,lsc[1],lsc[2],inter=self.inter,nn=self.nn,nt=self.nt,verb=1)
       else:
          ttasks = None
          items = []
       return ttasks,items

   def Stage2TasksWaveletInvert(self):

       Exposures = [E for E in self.UseExposures if not E.bad]
       items = []
       self.newinv = False
       for chip in self.chips:
           for j,E in enumerate(Exposures):
               if self.newcons or not FileCheck(eval("E.sfits"+self.stage)(chip)):
                  self.newinv = True
                  MakeJXY(nn=self.nn,nt=self.nt)
                  items.append((E,chip,j))

       if items:
          if self.HUB:
             ttasks=[TwistedTask(-1,"WaveInvert",\
                Echip[0],Echip[1],Echip[2],self.stage,nn=self.nn,nt=self.nt,origdir=Echip[0].scratchspace) \
                for j,Echip in enumerate(items)]
          else:
             ReadWaves(p=2,chip=chip,nn=self.nn,nt=self.nt)
             ttasks = lambda Echip: WaveInvert(Echip[0],Echip[1],Echip[2],self.stage,nn=self.nn,nt=self.nt,origdir=Echip[0].scratchspace)
       else:
          ttasks = None
       return ttasks,items

   def Stage3(self):
       self.current = 3
       go = 1
       force = 0
       p = 0
       bad = 0
       while go and p < 10:
         p += 1
         [E.Bad() for E in self.UseExposures]
         Exposures = [E for E in self.UseExposures if not E.bad]
         ttasks,items = self.Stage3Tasks(force=force)
         if ttasks or bad:
            self.Execute(ttasks,items)
            bad = self.Stage3Check(p)
         if self.flat:
            go = 0
         else:
            if bad: force = 1
            else: go = 0
#         go = 0
#         raw_input("<cr> ")

   def Stage3Check(self,p):
       print "Checking the shifts, iteration No. %d" % (p)
       chip = self.chips[0]
       xd,yd,md,xs,ys,ms,nn = [{} for j in range(7)]
       fn = {}
       tn = {}
       qn = {}
       en = {}
       sn = {}
       Exposures = [E for E in self.UseExposures if not E.bad]
       fsky = array([e.sky for e in Exposures])
       bab = not self.flat and (sum(fsky) and (sum(fsky) < len(fsky)))
       if bab:
          Exposures = [E for E in Exposures if not E.sky]
          sExposures = [e for e in Exposures if e.sky]
       else:
          sExposures = Exposures

       for E in Exposures:
           f = eval("E.sfits"+self.stage)(chip)
           if FileCheck(f):
              F = FITS(f)
              xd[f] = F["xdiff1"]
              yd[f] = F["ydiff1"]
              md[f] = F["mdiff1"]
              xs[f] = F["xscat1"]
              ys[f] = F["yscat1"]
              ms[f] = F["mscat1"]
              nn[f] = F["lmatch1"]
              qn[f] = F["exptime"]
              F.close()
              fn[f] = E.expno
              tn[f] = E.mjd
              en[f] = E
              sn[f] = E.sky
       fk = fn.keys()

       o = argsort([tn[f] for f in fk])
       xd = take([xd[f] for f in fk],o)
       yd = take([yd[f] for f in fk],o)
       md = take([md[f] for f in fk],o)
       xs = take([xs[f] for f in fk],o)
       ys = take([ys[f] for f in fk],o)
       ms = take([ms[f] for f in fk],o)
       tn = take([tn[f] for f in fk],o)
       nn = take([nn[f] for f in fk],o)
       sn = take([sn[f] for f in fk],o)
       qn = take([qn[f] for f in fk],o)

       xe = divz(xs,sqrt(nn))
       ye = divz(ys,sqrt(nn))
       me = divz(ms,sqrt(nn))

       xo,xr = bwt(xd)
       yo,yr = bwt(yd)
       mo,mr = bwt(md)
       no,nr = bwt(nn)

       print md
       print mo,mr
       print nn
       print no,nr
       tn = tn - tn[0]

       opendev(Exposures[0].scratch("Shifts%s_%d.ps"%(self.stage,p))+"/vcps",nx=2,ny=2,land=0,asp=1,size=7.5)

       xm = extrema(xd)
       xm = add(xm,[-0.1*diff(xm)[0],0.1*diff(xm)[0]])
       ym = add(extrema(yd),[-xr,xr])
       newbox(xm[::-1],ym,ch=2)
       xlabel("\gdx (arcsec)",ch=2)
       ylabel("\gdy (arcsec)",ch=2)
       drawpt(xd,yd,ps=17,ci=4,ch=3)
       drawpterr(xd,xs,yd,ys,ci=4,ch=3)
       drawpt1(xo,yo,ps=5,ci=2,ch=5)
       th = arange(0,2*pi,pi/100,float32)
       cth,sth = cos(th), sin(th)
       drawln(xo+xr*cth,yo+yr*sth,ls=1,ci=3,lw=2)

       st = diff(extrema(tn))[0]/100
       xm = add(extrema(tn),[-st,st])

       ym = multiply([-1,1],max(abs(md)+me+2*mr))
       newbox(xm,ym,ch=2)
       xlabel("t-t\d0\u (d)",ch=2)
       ylabel("\gdm (mag)",ch=2)
       drawpt(tn,md,ps=17,ci=4,ch=3)
       drawpterr(tn,0*tn,md,me,ci=4,ch=3)
       drawln([xm[0],xm[-1]],[-mr,-mr],ls=2,ci=3,lw=2)
       drawln([xm[0],xm[-1]],[+mr,+mr],ls=2,ci=3,lw=2)

       ym = [-0.1*max(nn),1.1*max(nn)]
       newbox(xm,ym,ch=2)
       xlabel("t-t\d0\u (d)",ch=2)
       ylabel("N",ch=2)
       drawpt(tn,nn,ps=17,ci=4,ch=3)
       drawln([xm[0],xm[-1]],[no-nr,no-nr],ls=2,ci=3,lw=2)
       drawln([xm[0],xm[-1]],[no+nr,no+nr],ls=2,ci=3,lw=2)

       n1 = len(md)/(len(md)-1)
       b1 = greater_equal(abs(md-mo),50*mr)
       b2 = where(arange(len(nn)),logical_or(b1,equal(nn,0)),0)
       b = nonzero(b2)
       f = None
       if len(b):
          E = en[fk[o[b[0]]]]
          f = eval("E.sfits"+self.stage)(chip)
          print "Bad:",E,f
          open(f+".bad","w").close()
       print "Before check",len(self.UseExposures)
       self.Cull()
       print "After check",len(self.UseExposures)
       closedev(pstopdf=True)
       return f

   def Stage3Tasks(self,force=0):
       print "Make frame catalogs and compute the shifts..."
       self.doshifts = False
       Exposures = [E for E in self.UseExposures if not E.bad]
       sky = array([E.sky for E in Exposures])
       if sum(sky) < len(sky):
          Exposures = [E for E in Exposures if not E.sky]
       for E in Exposures:
         for chip in self.chips:
             if FileCheck(eval("E.cat"+self.stage)(chip)):
                F = FITS(eval("E.sfits"+self.stage)(chip))
                xdiff1 = F["xdiff1"]
                ydiff1 = F["ydiff1"]
                mdiff1 = F["mdiff1"]
                F.close()
                if type(xdiff1) == StringType or type(ydiff1) == StringType:
                   self.doshifts=True
             else:
                self.doshifts=True
             if self.doshifts: break
       if self.doshifts or force:
          easydir(Exposures[0].scratch("shift"+self.stage))
          items = Exposures
          if self.HUB:
             ttasks=[TwistedTask(-1,"lambda *args,**kw: args[0].Shift(*args[1:],**kw)",\
                E,Exposures,chips=self.chips,first=self.first,second=self.second,verb=1,mport=-1) \
                for j,E in enumerate(items)]
          else:
             ttasks = lambda E: E.Shift(Exposures,chips=self.chips,first=self.first,second=self.second,verb=1,mport=-1)
       else:
          ttasks = None
          items = []
       return ttasks,items

   def Stage4(self):
       self.current = 4
       self.Stage4a()
       ttasks,items = self.Stage4Tasks()
       ttasks3 = self.Stage4c()
       if ttasks or ttasks3:
          if isinstance(ttasks,ListType):
             N = len(ttasks)
             if N:
                Nd = []
                while len(Nd) < N: [Nd.append(k) for k in randint(0,N,N) if k not in Nd]
                ttasks = [ttasks[j] for j in Nd]
                if self.HUB: self.Execute(ttasks,items)
                else: self.Execute(ttasks,items,serial=1)
                self.Clear()
                ttasks2,items = self.Stage4bTasks()
                if self.HUB: self.Execute(ttasks2,items)
                else: self.Execute(ttasks2,items)
          self.Execute(ttasks3,[])
#          os._exit(1)

   def Stage4bTasks(self):
       ttasks = []
       items = []
       if 1 or self.newstacks:
          Exposures = self.UseExposures
          Exposures = [E for E in Exposures if not E.bad]
          fsky = array([e.sky for e in Exposures])
          bab = not self.flat and (sum(fsky) and (sum(fsky) < len(fsky)))
          if bab:
             Exposures = [E for E in Exposures if not E.sky]
          for chip in self.chips:
             cnames = [Exposures[0].scratch("Stack%s_%d_%dc%d.fits" % (self.stage,no1,no2,chip)) for no1,no2 in zip(self.no1s,self.no2s)]
             snames = [Exposures[0].scratch("Stack%s_%d_%dc%dsig.fits" % (self.stage,no1,no2,chip)) for no1,no2 in zip(self.no1s,self.no2s)]
             enames = [Exposures[0].scratch("Stack%s_%d_%dc%dmap.fits" % (self.stage,no1,no2,chip)) for no1,no2 in zip(self.no1s,self.no2s)]
             scale = Exposures[0].scale
             catalog = lambda cname: cname.replace(".fits",".cat")
             saturates = []
             ttasks0 = []
             for cname,sname,ename,no1,no2 in zip(cnames,snames,enames,self.no1s,self.no2s):
                ttasks0 += [TwistedTask(-1,"Assemble",t1) for t1 in [cname,sname,ename]]
                F = [FITS(eval("E.sfits%s"%(self.stage))(chip)) for E in Exposures if no1 <= E.expno <= no2]
                exptimes = [f["exptime"] for f in F]
                [f.close() for f in F]
                saturates.append( 50e3 * len(exptimes) / sum(exptimes))
             self.Execute(ttasks0,cnames,serial=not self.HUB)
             ttasks += [TwistedTask(-1,"Sextractor",cname,catalog(cname),scale=scale,saturate=saturate,thresh=2.5,seeing=3*scale,segm=1) for cname,saturate in zip(cnames,saturates)]
             ttasks += [TwistedTask(-1,"MakeThumbnail",cname,mag=8,pct=40,s1=-10,s2=10,inv=1,q=85) for cname in cnames]
             ttasks += [TwistedTask(-1,"MakeThumbnail",sname,mag=8,pct=40,s1=-10,s2=10,inv=1,q=85) for sname in snames]
             ttasks += [TwistedTask(-1,"MakeThumbnail",ename,mag=8,pct=40,s1=-10,s2=10,inv=1,q=85) for ename in enames]
             ttasks += [TwistedTask(-1,"MakeThumbnail",cname,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85) for cname in cnames]
             ttasks += [TwistedTask(-1,"MakeThumbnail",sname,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85) for sname in snames]
             ttasks += [TwistedTask(-1,"MakeThumbnail",ename,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85) for ename in enames]
             items += cnames
       return ttasks,items

   def Stage4c(self):
       Exposures = self.UseExposures
       Exposures = [E for E in Exposures if not E.bad]
       fsky = array([e.sky for e in Exposures])
       bab = not self.flat and (sum(fsky) and (sum(fsky) < len(fsky)))
       if bab:
          Exposures = [E for E in Exposures if not E.sky]
       ttasks = []
       for chip in self.chips:
          name = Exposures[0].scratch("Stack%sc%d.fits" % (self.stage,chip))
          name2 = Exposures[0].scratch("Stack%sc%dsig.fits" % (self.stage,chip))
          catalog = Exposures[0].scratch("Stack%sc%d.cat" % (self.stage,chip))
          if self.newstacks or not FileCheck(name) or not FileCheck(name2) or not FileCheck(catalog):
             names = [Exposures[0].scratch("Stack%s_%d_%dc%d.fits" % (self.stage,no1,no2,chip)) for no1,no2 in zip(self.no1s,self.no2s)]
             names2 = [Exposures[0].scratch("Stack%s_%d_%dc%dsig.fits" % (self.stage,no1,no2,chip)) for no1,no2 in zip(self.no1s,self.no2s)]
             exptime = sum([E.exptime for E in Exposures])
             saturate = 50e3*len(Exposures)/exptime
             scale = Exposures[0].scale
             ttasks.append( TwistedTask(-1,"StackStacks",name,name2,catalog,names,names2,exptime,saturate,scale))
       return ttasks

   def Stage4a(self):
       if self.end > self.start:
         no1 = self.start
         no2 = self.end
       else:
         Exposures = self.UseExposures
         sky = array([E.sky for E in Exposures])
         if sum(sky) < len(sky):
            Exposures = [E for E in Exposures if not E.sky]
         no1 = Exposures[0].expno
         no2 = Exposures[-1].expno

       if 0 and (no2-no1-1) > self.nmax:
         nch = int((no2-no1-1)/(0.75*nmax) + 1)
         print "Number of chunks=",nch
         sch = int(1.*(no2-no1)/nch+1)
         print "Size of chunks=",sch
         self.no1s = arange(no1,no2,sch,int32)
         self.no2s = self.no1s+(sch-1)
       else:
         self.no1s = [no1]
         self.no2s = [no2]

   def Stage4Tasks(self):
       if self.caching:
         print "Cache the images and make stacks..."
       else:
         print "Make stacks..."

       Exposures = self.UseExposures
       Exposures = [E for E in Exposures if not E.bad]
       sky = array([E.sky for E in Exposures])
       if sum(sky) < len(sky):
          Exposures = [E for E in Exposures if not E.sky]

       items = []
       for chip in self.chips:
           for no1,no2 in zip(self.no1s,self.no2s):
              name = Exposures[0].scratch("Stack%s_%d_%dcD.fits" % (self.stage,no1,no2))
              SubExposures = [E for E in Exposures if no1 <= E.expno <= no2]

              self.newstacks = False
              if self.doshifts or not (FileCheck(name.replace("cD.fits","c%d.fits" % (chip))) \
                and FileCheck(name.replace("cD.fits","c%d.cat" % (chip)))):
                 self.newstacks = True
                 item = chip,SubExposures,name
                 items.append(item)

       if items:
          if self.HUB:
#              ttasks = [TwistedTask(-1,"CIRCEStack",item[0],item[1],name=item[2],first=self.first,second=self.second,\
#                    usemed=True,justsex=False,yskip=self.yskip/(len(item[1])/80+1),sigclip=self.sigclip,\
#                    SuperExposures=Exposures,Hub=(None,self.ssh)) for item in items]
              ttasks = []
              for item in items:
                  ttasks += CIRCEStack(item[0],item[1],name=item[2],first=self.first,second=self.second,
                    usemed=True,justsex=False,yskip=self.yskip/(len(item[1])/80+1),sigclip=self.sigclip,
                    SuperExposures=Exposures,Hub=(self.HUB,self.ssh),create=1)
          else:
              ttasks = lambda item: CIRCEStack(item[0],item[1],name=item[2],first=self.first,second=self.second,
                    usemed=True,justsex=False,yskip=self.yskip/(len(item[1])/80+1),sigclip=self.sigclip,
                    SuperExposures=Exposures,Hub=(self.HUB,self.ssh))
       else:
          ttasks = None
          items = []
       return ttasks,items

   def Stage5(self):
       self.current = 5
#       os._exit(1)
       Exposures = self.UseExposures
       Exposures = [E for E in Exposures if not E.bad]
       fsky = array([e.sky for e in Exposures])
       bab = not self.flat and (sum(fsky) and (sum(fsky) < len(fsky)))
       if bab:
          Exposures = [E for E in Exposures if not E.sky]

       nastro = sum([len(glob(Exposures[0].scratch("Stack%s_%d_%dc?.soln" % (self.stage,no1,no2)))) for no1,no2 in zip(self.no1s,self.no2s)])
       self.calib = nastro < (len(self.no1s)*len(self.chips))
       F = [FITS(Exposures[0].scratch("Stack%s_%d_%dc%d.fits" % (self.stage,no1,no2,c))) for no1,no2 in zip(self.no1s,self.no2s) for c in self.chips]
       astrom = [f["astrom"]=="T" for f in F if f["astrom"] != "T"]
       zpts = [f["photzpt"]!="N/A" for f in F if f["photzpt"] == "N/A"]
       [f.close() for f in F]
       self.calib = self.calib or (len(astrom) < len(F)) or len(zpts)

       print "Patch up the astrometry, get the calibration, and make mosaics..."
       if self.calib or self.newstacks:

          if not FileCheck(Exposures[0].scratch("multiastro%s.out" % (self.stage))) or not FileCheck("psc.sim"):
             name = Exposures[0].scratch("Stack%sc?.fits" % (self.stage))
             if self.fclass == "newfirm":
                command = "newfirm2soln %s" % (name)
             elif self.fclass == "fourstar":
                command = "fourstar2soln %s" % (name)
             verbsh(command)
             dat = Exposures[0].scratch("psc.dat")
             sim = Exposures[0].scratch("psc.sim")
             pho = Exposures[0].scratch("psc.pho")
             if not os.path.exists(sim):
                command = "get2mass %s -o %s" % (name,dat)
                verbsh(command)
             os.environ["PGPLOT_DEV"] = Exposures[0].scratch("multiastro%s.ps" % (self.stage)) + "/vcps"
             name = Exposures[0].scratch("Stack%sc\\?.fits" % (self.stage))
             command = "MultiSetup -n 20  %s -s cat -astro %s -nosex -mtol 1.0 -stol 1.0 -atol 1.0 > " % (name,sim)
             command += Exposures[0].scratch("multiastro%s.out" % (self.stage))
             verbsh(command)
             pg = Exposures[0].scratch("multiastro%s.ps" % (self.stage))
             command ="fixout "+pg
             verbsh(command)
             command ="pstopdf "+pg
             verbsh(command)

             for chip in self.chips:
               name = Exposures[0].scratch("Stack%sc%d.fits" % (self.stage,chip))
               F = FITS(name,readonly=0)
               F["astrom"] = "T"
               F.close()

          for no1,no2 in zip(self.no1s,self.no2s):

              F = [FITS(Exposures[0].scratch("Stack%s_%d_%dc%d.fits" % (self.stage,no1,no2,chip)),readonly=0) for chip in self.chips]
              photzpts = [f.name() for f in F if f["photzpt"] == "N/A"]
              [f.close() for f in F]
              if len(photzpts) or not FileCheck(Exposures[0].scratch("multiastro%s_%d_%d.out" % (self.stage,no1,no2))):
                 SubExposures = [E for E in Exposures if no1 <= E.expno <= no2]

                 name = Exposures[0].scratch("Stack%s_%d_%dc?.fits" % (self.stage,no1,no2))
                 if self.fclass == "newfirm":
                    command = "newfirm2soln %s" % (name)
                 elif self.fclass == "fourstar":
                    command = "fourstar2soln %s" % (name)
                 verbsh(command)
                 os.environ["PGPLOT_DEV"] = Exposures[0].scratch("multiastro%s_%d_%d.ps" % (self.stage,no1,no2)) + "/vcps"
                 name = Exposures[0].scratch("Stack%s_%d_%dc\\?.fits" % (self.stage,no1,no2))
                 command = "MultiSetup -n 20 %s -s cat -astro %s -nosex -mtol 1.0 -stol 1.0 -atol 1.0 >" % (name,sim)
                 command += Exposures[0].scratch("multiastro%s_%d_%d.out" % (self.stage,no1,no2))
                 verbsh(command)
                 pg = Exposures[0].scratch("multiastro%s_%d_%d.ps"%(self.stage,no1,no2))
                 command = "fixout "+pg
                 verbsh(command)
                 command = "pstopdf "+pg
                 verbsh(command)

                 for chip in self.chips:
                   name = Exposures[0].scratch("Stack%s_%d_%dc%d.fits" % (self.stage,no1,no2,chip))
                   F = FITS(name,readonly=0)
                   F["astrom"] = "T"
                   F.close()

                 for chip in self.chips:
                   name = Exposures[0].scratch("Stack%s_%d_%dc%d.cat" % (self.stage,no1,no2,chip))
                   command = "sex2sim -a %s" % (name)
                   verbsh(command)

                 for chip in self.chips:
                   name = Exposures[0].scratch("Stack%s_%d_%dc%d.cat" % (self.stage,no1,no2,chip))
                   command ="Assign -tol 1.5 -sky %s %s" % (sim,name)
                   verbsh(command)

                 exptime = sum([E.exptime for E in SubExposures])
                 if self.fclass == "newfirm":
                    gain = 8*exptime*len(SubExposures)
                    hi = 1e3
                 elif self.fclass == "fourstar":
                    if SubExposures[0].gain[0] == "F":
                       gain = 2.5*exptime*len(SubExposures)
                    elif SubExposures[0].gain[0] == "L":
                       gain = 1.3*exptime*len(SubExposures)
                    hi = 50e3*len(SubExposures)/exptime
                 commands = []
                 for chip in self.chips:
                   name = Exposures[0].scratch("Stack%s_%d_%dc%d.fits" % (self.stage,no1,no2,chip))
                   command = "apphot %s -catsuf std -sig sig -hi %f -lo -5e5 -g %.3f -ap apertures.dat" % (name,hi,gain)
                   commands.append(command)
                 if self.NP > 1:
                    RunForkedQueue(verbsh,commands,MaxThreads=self.NP,monitor=0,ThreadOutput=True,sleeptime=0.01)
                 else:
                    map(verbsh,commands)

                 os.environ["PGPLOT_DEV"] = Exposures[0].scratch("gcurve%s_%d_%d.ps" % (self.stage,no1,no2)) + "/vcps"
                 name = Exposures[0].scratch("Stack%s_%d_%dc?.mag" % (self.stage,no1,no2))
                 command = "gcurve -q %s -e 0.10 -p 2 -c 3 -v -ext 2 -lo 13 -hi 23" % (name)
                 verbsh(command)
                 pg = Exposures[0].scratch("gcurve%s_%d_%d.ps" % (self.stage,no1,no2))
                 command ="fixout "+pg
                 verbsh(command)
                 command ="pstopdf "+pg
                 verbsh(command)


                 if self.nfilt in ["J","H","Ks"]:
                    os.environ["PGPLOT_DEV"] = Exposures[0].scratch("calphot%s_%d_%d.ps" % (self.stage,no1,no2)) + "/vcps"
                    name = Exposures[0].scratch("Stack%s_%d_%dc?.cor" % (self.stage,no1,no2))
                    cal = Exposures[0].scratch("psc%s_%d_%d.cal" % (self.stage,no1,no2))
                    if self.nfilt[0] == "K":
                       command="calphot %s -pho %s -n 1 -N 1 -bright 13 -faint 15.5 -filter %s -nocolor -color J,K -mcolor 0.9 -chi 5e3 -err 0.15 -nox -o %s" % (name,pho,self.nfilt[0],cal)
                    else:
                       command="calphot %s -pho %s -n 1 -N 1 -bright 13 -faint 15.5 -filter %s -nocolor -color %s,K -mcolor 0.9 -chi 5e3 -err 0.15 -nox -o %s" % (name,pho,self.nfilt[0],self.nfilt[0],cal)
                    verbsh(command)

                    cal = readtable(cal)
                    F = [FITS(Exposures[0].scratch("Stack%s_%d_%dc%d.fits" % (self.stage,no1,no2,chip)),readonly=0) for chip in self.chips]
                    [f.modkey("photzpt",cal[2][0]) for f in F]
                    [f.close() for f in F]

   def Stage6(self):
       self.current = 6
       if self.fclass == "newfirm": scale = 0.4
       elif self.fclass == "fourstar": scale = 0.16

       Exposures = self.UseExposures
       Exposures = [E for E in Exposures if not E.bad]
       sky = array([E.sky for E in Exposures])
       if sum(sky) < len(sky):
          Exposures = [E for E in Exposures if not E.sky]

       if len(self.no1s) > 1:
          for no1,no2 in zip(self.no1s,self.no2s):

              SubExposures = [E for E in Exposures if no1 <= E.expno <= no2]

              name = Exposures[0].scratch("Stack%s_%d_%dc?.fits" % (self.stage,no1,no2))
              name2 = Exposures[0].scratch("Mosaic%s_%d_%d.fits" % (self.stage,no1,no2))
              if len(self.chips) > 1 and not FileCheck(name2.replace(".fits",".cat")):
                 if self.nfilt in ["J","H","Ks"]: photoff = "-photoff photzpt"
                 else: photoff = ""
                 command="ImageRect %s -o %s -fork %d -nocolor -scale %f -step 512 -nointerp -v %s" % (name,name2,self.NP,scale,photoff)
                 verbsh(command)

                 exptime = sum([E.exptime for E in SubExposures])
                 if self.fclass == "newfirm":
                    saturate = 50e3*len(SubExposures)/exptime
                 elif self.fclass == "fourstar":
                    saturate = 60e3*len(SubExposures)/exptime
                 Sextractor(name2,name2.replace(".fits",".cat"),scale=scale,saturate=saturate,thresh=2.5,seeing=3*scale,segm=1)


       names = Exposures[0].scratch(" ".join(["Stack%s_%d_%dc?.fits" % (self.stage,no1,no2) for no1,no2 in zip(self.no1s,self.no2s)]))
       name = Exposures[0].scratch("Mosaic%s.fits" % (self.stage))
       if len(self.chips) > 1 and not (FileCheck(name) \
               and FileCheck(name.replace(".fits",".cat")) \
               and FileCheck(name.replace(".fits",".jpg"))) \
               or self.calib or self.newstacks:
          if self.nfilt in ["J","H","Ks"]: photoff = "-photoff photzpt"
          else: photoff = ""
          command="ImageRect %s -o %s -fork %d -scale %f -step 512 -nointerp -v %s" % (names,name,self.NP,scale,photoff)
          verbsh(command)

          exptime = sum([E.exptime for E in Exposures])
          if self.fclass == "newfirm":
             saturate = 50e3*len(Exposures)/exptime
          elif self.fclass == "fourstar":
             saturate = 60e3*len(Exposures)/exptime
          Sextractor(name,name.replace(".fits",".cat"),scale=scale,saturate=saturate,thresh=2.5,seeing=3*scale,segm=1)
          MakeThumbnail(name,mag=1,pct=40,z1=0,z2=0,s1=-10,s2=10,inv=1)
          MakeThumbnail(name,mag=8,pct=40,z1=0,z2=0,s1=-10,s2=10,inv=1)

   def Stage7(self):
       self.current = 7
       name = self.UseExposures[0].scratch("Mosaic%s.fits" % (self.stage))
       catalog = name.replace(".fits",".cat")
       pgdev = name.replace(".fits",".ps/vcps")
       if not FileCheck(name.replace(".fits",".pdf")):
          d = readsex(catalog)
          mag = d["MAG_APER"]
          opendev(pgdev,nx=2,ny=2,land=0,asp=1,size=6.5)
          mm = [median(mag)-7,median(mag)+2]

          newbox(mm,[0.0,10.0],ch=2)
          xlabel("MAG_APER",ch=2)
          ylabel("FWHM (pixels)",ch=2)
          drawpt(d["MAG_APER"],d["FWHM_IMAGE"],ps=17,ch=2,ci=1)

          newbox(mm,[0.0,1.0],ch=2)
          xlabel("MAG_APER",ch=2)
          ylabel("MAGERR_APER",ch=2)
          drawpt(d["MAG_APER"],d["MAGERR_APER"],ps=17,ch=2,ci=1)
          closedev(pstopdf=True)

       self.success = 1
       self.current = 8

class RunThroughTargets:
    def __init__(self,DataDirs,DoFirst,DoSecond,mport=8022,tvport=7070,\
                first=0,second=0,flat=0,going=0,darks=0,fclass=newfirm,nume=3,NP=0):
        self.curdir = os.getcwd()
        self.datadirs = DataDirs
        self.fclass = fclass
        self.basedir = os.getcwd()+"/"
        self.targ = None
        self.darks = darks
        self.groups = {}
        self.going = going
        self.okay = True
        self.flat = flat
        self.first = first
        self.second = second
        self.targs = [None]
        self.DoFirst = DoFirst
        self.DoSecond = DoSecond
        self.Data1 = {}
        self.Data2 = {}
        self.status = {}
        self.Filters = {}
        self.nfiles = {}
        for ddir in self.datadirs: self.nfiles[ddir] = 0
        self.nume = nume
        self.left = []
        self.left1 = []
        self.left2 = []
        self.priorities = {None:0}
        self.NP = NP
        namespace = globals()
        namespace["Runner"] = self
        self.tvport = tvport
        print "Opening ssh on",mport+self.second
        self.manhole = AddManhole(getpass.getuser(),"",namespace,mport+self.second)
    def StartSite(self):
        self.site = appserver.NevowSite(CIRCEMainPage(runner=self))
        application = service.Application("CIRCE TV")
        reactor.listenTCP(self.tvport,self.site)
    def Fire(self):
        self.running = False
        self.running1 = False
        self.running2 = False
        self.ntargs = 0
        reactor.callInThread(self.StartSite)
        reactor.callInThread(self.Collate)
        self.runloop = task.LoopingCall(self.Run)
        self.runloop.start(1)
    def TickOne(self,watcher,f0,ifile,c):
        b = os.path.basename(f0).replace(".fits","c%d.jpg" % (c))
        j1 = "jpegs/"+b
        j = self.basedir+"/jpegs/"+b
        if self.fclass == "fourstar": f = f0.replace(".fits","c%d.fits" % (c))
        elif self.fclass == "newfirm": f
        print j1,os.getcwd()
        if os.path.exists(f):
           F = FITS(f)
           exptime = F["exptime"]
           if self.fclass == "newfirm":
              ncoadd = F["ncoadd"]
              if ncoadd == "N/A": ncoadd = 1
              F.goto(1+c)
           data = F.data()
           F.close()
           data = data[:2048,:2048]
           data = where(between(data,-1e24,1e24),data,0).astype(float32)
           os.makedirs(self.basedir+"/jpegs/")
           MakeThumbnail(data,mag=16,pct=40,s1=-10,s2=10,inv=1,q=85,j=j)
        reactor.callInThread(watcher.TickOneStep2,f,ifile,c,j1)
    def CollateCheck(self):
        if getattr(self,"collating",{}):
           print "Are we still collating?"
           print "I guess so!"
           return
        self.collateloop.stop()
        self.collating = {}
        zdirs = [datadir.replace("/"," ").strip().split(" ")[-1] for datadir in self.datadirs]
        for datadir in self.datadirs:
           self.collating[datadir] = True
           nfiles = len(os.listdir(datadir))
           if nfiles == self.nfiles[datadir]:
              self.collating.pop(datadir)
        if self.collating:
           if self.NP:
              RunForkedQueue(self.UpdateDir,self.collating.keys(),MaxThreads=self.NP,monitor=False,ThreadOutput=True)
           else:
              map(self.UpdateDir,self.collating.keys())
#        [reactor.callInThread(self.CollateDir,d) for d in self.collating.keys()]
#        os._exit(1)
        [self.CollateDir(d) for d in self.collating.keys()]
        reactor.callLater(1,self.Collate)
    def Collate(self):
        targs = self.targs
        if self.ntargs != len(self.targs):
           print "Reordering targets!"
           print "Number of new targets = ", len(self.targs)-self.ntargs
#           print len(self.targs)
           mlist = copy.copy(self.targs)
           slist = copy.copy(self.targs)
           plist = copy.copy(self.priorities)
           for targ in mlist:
              if not ((targ in self.Data1 and self.Data1[targ].ongoing) or
                      (targ in self.Data2 and self.Data2[targ].ongoing)):
                 plist.pop(targ)
                 slist.remove(targ)
                 self.priorities.pop(targ)
           tlist = {}
           for targ in mlist:
               if targ in self.Data1: mjd = self.Data1[targ].mjd
               elif targ in self.Data2: mjd = self.Data2[targ].mjd
               elif not targ: mjd = -999
#               print targ,mjd
               if mjd not in tlist: tlist[mjd] = []
               tlist[mjd].append(targ)
           [slist.append(o) for t in sort(tlist.keys()) for o in tlist[t] if o not in slist]
           if not plist:
              plist[None] = 0
           p = max(plist.values()) + 1
           print "starting priority",p
           for targ in slist:
              if targ not in plist:
                 if (targ in self.Data1 and self.Data1[targ].type == "dark") or \
                         (targ in self.Data2 and self.Data2[targ].type == "dark"):
                     plist.setdefault(targ,p)
#                     print "DARK",targ,p
                     p += 1
           for targ in slist:
              if targ not in plist:
                 if (targ in self.Data1 and "flat" in targ) or \
                         (targ in self.Data2 and "flat" in targ):
                     plist.setdefault(targ,p)
#                     print "FLAT",targ,p
                     p += 1
           for targ in slist:
              if targ not in plist:
                 if (targ in self.Data1) or \
                         (targ in self.Data2):
                     if targ[:3] == "obj" and ("sky"+targ[3:]) in slist:
                        plist.setdefault("sky"+targ[3:],p)
                        plist.setdefault(targ,p+1)
#                        print "OBJ,SKY",targ,p
                        p += 2
                     elif targ[:3] == "sky" and ("obj"+targ[3:]) in slist:
                        plist.setdefault(targ,p)
                        plist.setdefault("obj"+targ[3:],p+1)
#                        print "SKY,OBJ",targ,p
                        p += 2
                     else:
                        plist.setdefault(targ,p)
#                        print "OBJ",targ,p
                        p += 1
#           print len(slist),len(self.targs)
           for targ in self.targs:
             if targ not in slist: print targ
           for targ in slist:
               if targ not in self.priorities:
                  self.priorities[targ] = plist[targ]
           self.rpriorities = {}
           [self.rpriorities.setdefault( self.priorities[targ], targ) for targ in self.targs]
           for p in argsort(self.rpriorities.keys()):
               targ = self.rpriorities[p]
               print p,targ
           self.ntargs = len(self.priorities)
        self.collateloop = task.LoopingCall(self.CollateCheck)
        self.collateloop.start(10)
    def UpdateDir(self,datadir):
#        if datadir in self.collating: return
        zdir = datadir.replace("/"," ").strip().split(" ")[-1]
        nfiles = len(os.listdir(datadir))
        if nfiles == self.nfiles[datadir]:
           self.collating.pop(datadir)
        else:
           zodb = self.curdir+"/zodb/%sinfo.fs" % (zdir)
           if not os.path.exists(os.path.dirname(zodb)):
              print "Making",os.path.dirname(zodb)
              os.mkdir(os.path.dirname(zodb))
           self.nfiles[datadir] = nfiles
           if self.first or not os.path.exists(zodb):
              newfs = not os.path.exists(zodb)
              if newfs: print "Have to generate a ZODB for %d files." % (nfiles)
              UpdateZODB(datadir,zodb,newfs,verb=0,fclass=self.fclass)
              if newfs: print "Done generating the ZODB for %d files." % (nfiles)
#              PackZODB(zodb)
    def CollateDir(self,datadir):
        if datadir in self.collating:
           self.nfiles[datadir] = len(os.listdir(datadir))
           zdir = datadir.replace("/"," ").strip().split(" ")[-1]
           zodb = self.curdir+"/zodb/%sinfo.fs" % (zdir)
           if not os.path.exists( self.curdir+"/zodb/"):
              os.mkdir(self.curdir+"/zodb/")
           storage = ZODB.FileStorage.FileStorage(zodb,read_only=True)
           zdb = ZODB.DB( storage)
           conn = zdb.open()
           dbroot = conn.root()
           tree = dbroot["Data"]
           groups = copy.deepcopy(CollateDB(tree,targ="",filt="",nume=self.nume,watching=True,fclass=self.fclass))
           targs = groups.keys()
           transaction.abort()
           conn.close()
           zdb.close()
           storage.close()
           targs = [targ for targ in targs if (groups[targ][0].type in ["flat","dark"]) or \
                (groups[targ][0].type in ["object"] and groups[targ][0].filter not in ["Dark"])]
           addtargs = []
           for targ in targs:
               if targ not in self.targs:
                  filt = groups[targ][0].filter
                  uflat = groups[targ][0].type in ["flat","dark","tflat","dflat"]
                  if groups[targ][0].type == "flat":
                     addit = True
                  elif groups[targ][0].type == "dark":
                     addit = True
                  elif 1 or filt in ["J","H","Ks"]:
                     addit = True
                  else:
                     addit = False
                  if filt in ["undef"]: addit = False
                  if "flat" in targ and filt == "dark": addit = False
                  if addit:
                     uflat = "flat" in targ or "dark" in targ
                     t = groups[targ]
                     self.Filters[targ] = filt
                     self.Data1[targ] = self.DoFirst(datadir,targ,filt,uflat)
                     self.Data1[targ].mjd = groups[targ][0].mjd
                     self.Data1[targ].type = groups[targ][0].type
                     self.Data1[targ].dark = "dark" in targ
                     self.Data1[targ].paused = not self.going
                     self.Data2[targ] = self.DoSecond(datadir,targ,filt,uflat)
                     self.Data2[targ].mjd = groups[targ][0].mjd
                     self.Data2[targ].type = groups[targ][0].type
                     self.Data2[targ].dark = "dark" in targ
                     self.Data2[targ].paused = not self.going
#                     if targ == "obj071456+000000_Ks_F55544_3033_3045":
#                        self.Data1[targ].paused = False
#                        self.Data2[targ].paused = False
                     self.groups[targ] = groups[targ]
                     if targ not in self.priorities:
                        self.priorities[targ] = len(self.priorities)
                     addtargs.append(targ)
           if len(addtargs):
              print "New targets!"
              [sys.stdout.write("%4d %s\n" % (1+t,targ)) for t,targ in enumerate(addtargs) if targ]
           adddarks = []
           addflats = []
           addskies = []
           [adddarks.append(targ) for targ in addtargs if "dark" == targ[:4]]
           [addflats.append(targ) for targ in addtargs if "flat" in targ]
           [addskies.append(targ) for targ in addtargs if "sky" in targ]
           [addtargs.remove(targ) for targ in adddarks]
           [addtargs.remove(targ) for targ in addflats]
           [addtargs.remove(targ) for targ in addskies]
           self.targs += adddarks
           order = argsort([groups[targ][0].mjd for targ in addflats])
           self.targs += [addflats[o] for o in order]
           order = argsort([groups[targ][0].mjd for targ in addskies])
           self.targs += [addskies[o] for o in order]
           order = argsort([groups[targ][0].mjd for targ in addtargs])
           self.targs += [addtargs[o] for o in order]
           change = 0
           for targ in self.targs:
              if targ not in self.status:
                 self.status[targ] = False
                 change = True
           self.collating.pop(datadir)
    def Problem(self,reason):
        logging.err(reason)
    def Left(self,n=0):
        if n == 0:
           left = [targ for targ in self.targs if targ and \
                (not self.Data1[targ].Already(first=1,second=0) or not self.Data2[targ].Already(first=0,second=1))]
           [self.left.remove(targ) for targ in self.left if targ not in left]
           [self.left.append(targ) for targ in left if targ not in self.left]
           self.left = [self.left[j] for j in argsort([self.priorities[targ] for targ in self.left])]
        if n in [0,1]:
           left1 = [targ for targ in self.targs if targ and not self.Data1[targ].Already(first=1,second=0)]
           [self.left1.remove(targ) for targ in self.left1 if targ not in left1]
           [self.left1.append(targ) for targ in left1 if targ not in self.left1]
           self.left1 = [self.left1[j] for j in argsort([self.priorities[targ] for targ in self.left1])]
        if n in [0,2]:
           left2 = [targ for targ in self.targs if targ \
               and (self.Data1[targ].Already(first=1,second=0) and not self.Data2[targ].Already(first=0,second=1))]
           [self.left2.remove(targ) for targ in self.left2 if targ not in left2]
           [self.left2.append(targ) for targ in left2 if targ not in self.left2]
           self.left2 = [self.left2[j] for j in argsort([self.priorities[targ] for targ in self.left2])]
    def toggleGoing(self):
        self.going = not self.going
        for item in self.Data1:
            self.Data1[item].paused = not self.going
        for item in self.Data2:
            self.Data2[item].paused = not self.going
    def Run(self):
        self.Left(0)
        self.Left(1)
        if self.going and not self.running1 and self.first and len(self.left1):
           reactor.callInThread(self.Run1)
        self.Left(2)
        if self.going and not self.running2 and self.second and len(self.left2):
           reactor.callInThread(self.Run2)
    def Run1(self):
        items = [item for item in self.left1 if self.status[item]]
        if items:
           self.running = True
           self.running1 = True
           item = self.Data1[items[0]]
           self.left1.remove(items[0])
           item.FirstPass()
           self.RunItem(item,item.chain1)
           self.running1 = False
           self.Left(1)
    def Run2(self):
        items = [item for item in self.left2 if self.status[item]]
        if items:
           self.running = True
           self.running2 = True
           item = self.Data2[items[0]]
           self.left2.remove(items[0])
           item.SecondPass()
           self.RunItem(item,item.chain2)
           self.running2 = False
           self.Left(2)
    def RunItem(self,Item,Chain):
#        if not Item.Already(first=self.first,second=self.second) and not Chain.called:
        if not Chain.called:
            Chain.callback(True)
            if Chain.result is failure.Failure:
               return -1
        return 1
#    def RunTarg(self,targ):
#        if not self.Data1[targ].Already(first=self.first,second=self.second):
#            self.Data1[targ].chain1.callback(True)
#            if self.Data1[targ].chain1.result is failure.Failure:
#                reactor.callFromThread(reactor.stop)
#                self.okay = False
#                return -1
#        if not self.Data2[targ].Already(first=self.first,second=self.second):
#            self.Data2[targ].chain2.callback(True)
#            if self.Data2[targ].chain2.result is failure.Failure:
#                reactor.callFromThread(reactor.stop)
#                self.okay = False
#                return -1
#        return 1

def Die(x):
   logging.err(x)
   reactor.callFromThread(reactor.stop)
   return failure.Failure

def Assemble(gname):
    if os.path.basename(gname)[0] == ".":
       gname1 = glob(os.path.dirname(gname)+"/"+os.path.basename(gname).replace(".fits","_????.fits"))
    else:
       gname1 = glob(os.path.dirname(gname)+"/."+os.path.basename(gname).replace(".fits","_????.fits"))
    if gname1:
       gname1.sort()
       F = [FITS(gn) for gn in gname1]
       D = [f.data() for f in F]
       [f.close() for f in F]
       D = concatenate(D,0)
       [os.remove(gn) for gn in gname1]
       F = FITS(gname,readonly=0)
       F.putdata(D)
       F.close()

def StackStacks(name,name2,catalog,names,names2,exptime,saturate,scale):
      print "Making",name
      F = FITS(name,create=1,template=names[0])
      F2 = FITS(name2,create=1,template=names[0])
      F.modkey("exptime",1.0)
      F2.modkey("exptime",1.0)
      G = [FITS(i) for i in names]
      G2 = [FITS(i) for i in names2]
      naxis1,naxis2 = F.naxes()
      numer = zeros((naxis2,naxis1),float64)
      denom = zeros((naxis2,naxis1),float64)
      for g1,g2 in zip(G,G2):
         d1 = g1.data()
         d2 = VTKInvert(power(g2.data(),2))
         add(numer,d1*d2,numer)
         add(denom,d2,denom)
      [g.close() for g in G]
      result = VTKDivide(numer,denom)
      result2 = VTKInvert(VTKSqrt(denom))
      F.putdata(result)
      F2.putdata(result2)
      F.close()
      F2.close()

      MakeThumbnail(name,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
      MakeThumbnail(name,mag=8,pct=40,s1=-10,s2=10,inv=1,q=85)
      MakeThumbnail(name2,mag=1,pct=40,s1=-10,s2=10,inv=1,q=85)
      MakeThumbnail(name2,mag=8,pct=40,s1=-10,s2=10,inv=1,q=85)
      Sextractor(name,catalog,scale=scale,saturate=saturate,thresh=2.5,seeing=3*scale,segm=1)


